@@ -1,6 +1,6 @@
 
 # Metarace : Cycle Race Abstractions
-# Copyright (C) 2012  Nathan Fraser
+# Copyright (C) 2012-2018  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,13 +33,14 @@ from metarace import jsonconfig
 # 'check' python version
 assert sys.version >= '2.6', "Missing pre-requisite: python >= 2.6"
 
-VERSION = u'1.6.64'
+VERSION = u'1.11.3'
 LIB_PATH = os.path.realpath(os.path.dirname(__file__))
 UI_PATH = os.path.join(LIB_PATH, u'ui')
 DB_PATH = os.path.join(LIB_PATH, u'data')
 DATA_PATH = os.path.realpath(os.path.expanduser(
                              os.path.join(u'~', u'Documents', u'metarace')))
 DEFAULTS_PATH = os.path.join(DATA_PATH, u'.default')
+SHARE_PATH = os.path.join(DATA_PATH, u'share')
 LOGS_PATH = os.path.join(DATA_PATH, u'logs')
 LOGO_FILE = u'metarace_icon.svg'
 SYSCONF_FILE = u'metarace.json'
@@ -50,11 +51,11 @@ sysconf = jsonconfig.config() # system-defaults, populated by init() method
 
 def init(withgtk=True):
     """Shared metarace program initialisation."""
-    print (u"\n\
-metarace(" + VERSION + u") Copyright (C) 2010-2012  Nathan Fraser\n\
-This program comes with ABSOLUTELY NO WARRANTY.\n\
-This is free software, and you are welcome to redistribute it\n\
-under certain conditions.\n\n")
+    #print (u"\n\
+#metarace(" + VERSION + u") Copyright (C) 2010-2015  Nathan Fraser\n\
+#This program comes with ABSOLUTELY NO WARRANTY.\n\
+#This is free software, and you are welcome to redistribute it\n\
+#under certain conditions.\n\n")
 
     # Check for data path and change working directory
     mk_data_path()
@@ -110,6 +111,9 @@ def mk_data_path():
     if not os.path.exists(DEFAULTS_PATH):
         print (u'metarace: Creating system defaults in ' + repr(DEFAULTS_PATH))
         os.makedirs(DEFAULTS_PATH)
+    if not os.path.exists(SHARE_PATH):
+        print (u'metarace: Creating shared export in ' + repr(SHARE_PATH))
+        os.makedirs(SHARE_PATH)
 
 def default_file(filename=u''):
     """Search the current setup for the named file and return a full path."""
@@ -197,10 +201,10 @@ def unlockpath(configpath):
 
 def help_docs(window):
     """Shell out to display help documents."""
+    pass
     # TODO
     # windows: use os.startfile
     # nix: use gtk.help (old-style)
-    print(u'Help Docs Not Implemented.')
 
 def about_dlg(window):
     """Display shared about dialog."""
@@ -208,7 +212,7 @@ def about_dlg(window):
     dlg.set_transient_for(window)
     dlg.set_name(u'metarace')
     dlg.set_version(VERSION)
-    dlg.set_copyright(u'Copyright \u00a9 2010-2012 Nathan Fraser')
+    dlg.set_copyright(u'Copyright \u00a9 2010-2018 Nathan Fraser')
     dlg.set_comments(u'Cycle Race Astractions')
     dlg.set_website(u'http://metarace.com.au/')
     dlg.set_license(LICENSETEXT)
