@@ -69,8 +69,9 @@ class classification(object):
     def loadconfig(self):
         """Load race config from disk."""
         cr = ConfigParser.ConfigParser({'id':EVENT_ID,
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'showevents':'',
+					'comment':'',
                                         'placesrc':'',
                                         'medals':'' })
         cr.add_section('event')
@@ -86,6 +87,7 @@ class classification(object):
         self.showevents = cr.get('event', 'showevents')
         self.placesrc = cr.get('event', 'placesrc')
         self.medals = cr.get('event', 'medals')
+        self.comment = cr.get('event', 'comment')
         self.recalculate()	# model is cleared and loaded in recalc
 
         # After load complete - check config and report. This ensures
@@ -140,6 +142,7 @@ class classification(object):
         cw.set('event', 'showevents', self.showevents)
         cw.set('event', 'placesrc', self.placesrc)
         cw.set('event', 'medals', self.medals)
+        cw.set('event', 'comment', self.comment)
         cw.set('event', 'showinfo', self.info_expand.get_expanded())
         cw.set('event', 'id', EVENT_ID)
         self.log.debug('Saving race config to: ' + self.configpath)
@@ -182,10 +185,27 @@ class classification(object):
         sec.lines = []
         for r in self.riders:
             rno = r[COL_BIB]
+            rh = self.meet.newgetrider(rno, self.series)
+            rname = u''
+            plink = u''
+            rcat = u''
             if u't' in self.series: # Team no hack
                 rno = u' '  # force name
-            rname = strops.resname(r[COL_FIRST], r[COL_LAST],
-                                       r[COL_CLUB])
+                if rh is not None:
+                    rname = rh[u'first']
+            else:
+                if rh is not None:
+                    rname = rh[u'namestr']
+                    if rh[u'ucicode']:
+                        rcat = rh[u'ucicode']   # overwrite by force
+
+                    # consider partners here
+                    if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                        ph = self.meet.newgetrider(rh[u'note'], self.series)
+                        if ph is not None:
+                            plink = [u'', u'', ph[u'namestr'] + u' - Pilot', ph[
+u'ucicode'], u'', u'',u'']
+
             rank = ''
             if r[COL_PLACE] != '':
                 rank = r[COL_PLACE]
@@ -200,8 +220,16 @@ class classification(object):
                 sec.lines.append([None, None, None])
             prevmedal = medal
 
-            nrow = [rank, rno, rname, None, None, medal]
+            nrow = [rank, rno, rname, rcat, None, medal,plink]
             sec.lines.append(nrow)
+            if u't' in self.series:
+                for trno in strops.reformat_riderlist(rh[u'note']).split():
+                    trh = self.meet.newgetrider(trno) #!! SERIES?
+                    if trh is not None:
+                        trname = trh[u'namestr']
+                        trinf = trh[u'ucicode']
+                        sec.lines.append([None, trno, trname, trinf,
+                                                 None, None, None])
         ret.append(sec)
 
         if recurse:
@@ -524,6 +552,7 @@ class classification(object):
         self.winopen = ui
         self.placesrc = ''
         self.medals = ''
+        self.comment = ''
 
         self.riders = gtk.ListStore(gobject.TYPE_STRING, # 0 bib
                                     gobject.TYPE_STRING, # 1 first name
