@@ -15,85 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Event 'database' utilities
-
-Manage an event db with the columns described below. Columns may be 
-arranged in any order, but the column header must be present. Only the
-first 4 characters are checked.
-
-	evid	"EvID"
-		Event reference id - Compulsory, primary key
-
-	numb	"Reference Number"
-		If present, this event is part of a composite event whose
-		evid id is the reference number. All links generated
-		will be to the reference number instead of the evid. NOTE:
-		this normally implies resu == False, since the result will
-		be exported in the referenced event, but this is not required.
-
-	pref	"Prefix"
-		Event prefix string
-
-	info	"Information"
-		Event information string
-
-	seri	"Series"
-		Rider number series
-
-	type	"Type Handler"
-		Race type/handler key.
-
-	star	"Starters"
-		Transfer the listed rider nos into race handler
-		or if 'auto' use the provided string as an autospec
-
-	depe	"Depends On Events"
-		List of event ids that this event depdends on
-
-	resu	"Result Include?"
-		Include this result in live result and final exports
-
-	inde	"Index Include?"
-		Include this event in the index of events (required?)
-
-	prin	"Printed Program Include?"
-		Include this event in the printed program of events
-
-	plac	"Placeholder Count"
-		Include placeholders for this many riders
-
-	sess	"Session"
-		Event runs in this session.
-
-	cate	"Categories Include?"
-		Include categories along with rider names in outputs?
-
-	laps	"Laps Count"
-		Count of laps in the event if applicable
-
-	dist	"Distance String"
-		Distance of event if aspplicable with units: 500m, 5km
-
-	prog	"Progression Rules String"
-		String describing progression rules and or medals
-
-	reco	"Record String"
-		Informative record string
-
-        dura	"Duration"
-		Expected duration of the event in minutes (?)
-
-	prov	"Provisional Start Time"
-		Expected Start time for the event as string.
-
-	dirt	"Dirty?"
-		Event is probably out of sync with export
-
-"Primary key" is (evid)
-
-When Series is not present the empty series '' is assumed.
-
-"""
+"""Event 'database' utilities"""
 
 import gtk
 import gobject
@@ -115,10 +37,10 @@ defracetypes=[u'sprint', u'keirin',		# sprint types
            u'flying 200', u'flying lap',	# individual TTs
            u'indiv tt', u'indiv pursuit',
            u'pursuit race',			# indiv tt with twist
-           #'team sprint', 'team pursuit',	# team time trials [replace]
            u'points', u'madison',		# point score types
            u'omnium',				# aggregate types
            u'classification',			# final classification
+           u'competition',			# competition container
            u'presentation',			# medal ceremony
            u'break',				# break/non-race
            u'race' ]		# esoterics/generic
@@ -133,7 +55,6 @@ EVENT_DEFAULTS = {u'evid':None,	# empty not allowed
 	u'resu':True,
 	u'inde':False,
 	u'prin':False,
-	u'cate':False,
 	u'dirt':False,
 	u'plac':None,
 	u'laps':None,
@@ -154,32 +75,30 @@ EVENT_COLUMNS = {
 	u'prin':	u"Printed Program Include?",
 	u'plac':	u"Placeholder Count",
 	u'sess':	u"Session",
-	u'cate':	u"Categories Include?",
 	u'laps':	u"Laps Count",
 	u'dist':	u"Distance String",
 	u'prog':	u"Progression Rules String",
 	u'reco':	u"Record String",
-	u'dura':	u"Duration",
-	u'prov':	u"Provisional Start Time",
-	u'dirt':	u"Dirty?"
+	u'dirt':	u"Dirty?",
+	u'evov':	u"EVOverride"
 }
 # for any non-strings, types as listed
 EVENT_COLUMN_CONVERTERS = {
 	u'resu':strops.confopt_bool,
 	u'inde':strops.confopt_bool,
 	u'prin':strops.confopt_bool,
-	u'cate':strops.confopt_bool,
 	u'dirt':strops.confopt_bool,
 	u'plac':strops.confopt_posint,
 	u'laps':strops.confopt_posint,
 }
 
 DEFAULT_COLUMN_ORDER = [
-	u'evid', u'refe', u'pref', u'info',
-	u'seri', u'type', u'star', u'depe',
+	u'evid', u'refe',
+        u'pref', u'info', u'seri', u'type',
+        u'star', u'depe',
 	u'resu', u'inde', u'prin', u'plac',
-	u'sess', u'cate', u'laps', u'dist',
-	u'prog', u'reco', u'dura', u'prov',
+	u'sess', u'laps', u'dist', u'prog', 
+        u'evov', u'reco',
         u'dirt'
 ]
 
@@ -272,6 +191,39 @@ class eventdb(object):
         self.__notify(None)
         self.log.debug(u'Event model cleared.')
 
+    def change_evno(self, oldevent, newevent):
+        """Attempt to change the event id."""
+        if oldevent not in self:
+            self.log.error(u'Event id ignored on unknown event: ' 
+                           + repr(oldevent))
+            return False
+
+        if newevent in self:
+            self.log.error(u'New event id already exists: '
+                           + repr(newevent))
+            return False
+ 
+        oktochg = True
+        if self.__evno_change_cb is not None:
+            oktochg = self.__evno_change_cb(oldevent, newevent)
+        if oktochg:
+            ref = self.__store[oldevent]
+            ref.set_value(u'evid', newevent)
+            cnt = 0
+            for j in self.__index:
+                if j == oldevent:
+                    break
+                cnt += 1
+            if cnt < len(self.__index):
+                self.__index[cnt] = newevent
+            del(self.__store[oldevent])
+            self.__store[newevent] = ref
+            self.log.warn(u'Updated event id: '
+                                      + repr(oldevent) + u' to: '
+                                      + repr(newevent))
+            return True
+        return False
+
     def add_event(self, newevent):
         """Append newevent to model."""
         evno = newevent[u'evid']
@@ -475,8 +427,13 @@ class eventdb(object):
         self.add_empty()
         # Select new row?
 
+    def set_evno_change_cb(self, cb, data=None):
+        """Set the event no change callback."""
+        self.__evno_change_cb = cb
+
     def popup_edit_cb(self, menuitem, data=None):
         """Edit event extended attributes."""
+        evno = None
         ref = None
         model, plist = self.view.get_selection().get_selected_rows()
         if len(plist) > 0:
@@ -503,19 +460,21 @@ class eventdb(object):
             prompt = EVENT_COLUMNS[i]
             ne = uiutil.mktextentry(prompt, rof, tbl)
             ne.set_text(unicode(ref[i]))
-            if i == u'evid':
-                ne.set_sensitive(False)
+            #if i == u'evid':
+                #ne.set_sensitive(False)
             entmap[i] = ne
             rof += 1
         response = dlg.run()
         if response == 1:       # id 1 set in glade for "Apply"
             self.log.info(u'Updating event.')
-            ## TODO
-            # does event id change, if so is it an allowed change?
-            # if allowed, make change and arrange to callback in caller
-            # as in very old trackmeet
-
             do_notify = False
+
+            # check for event id change
+            newevno = entmap[u'evid'].get_text().decode('utf-8',
+                                                        'ignore').strip()
+            if newevno != evno:
+                do_notify = self.change_evno(evno, newevno)
+
             for i in DEFAULT_COLUMN_ORDER:
                 if i == u'evid':
                     continue	# ignore change for now
@@ -564,14 +523,12 @@ class eventdb(object):
         if self.__viewmodel is None:
             self.__viewmodel = gtk.ListStore(
                  gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
-            self.__viewmodel.connect('rows-reordered',
-                                      self.__viewmodel_reordered_cb)
         self.__notify(None)	# trigger reload
         if self.view is not None:
             return self.view
         self.__viewparent = parentwin
         v = gtk.TreeView(self.__viewmodel)
-        v.set_reorderable(True)
+        #v.set_reorderable(True)
         v.set_enable_search(False)
         v.set_rules_hint(True)
         v.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
@@ -651,17 +608,6 @@ class eventdb(object):
     def __contains__(self, item):
         return item in self.__store
 
-    def __viewmodel_reordered_cb(self, model, path, iter, new_order, data=None):
-        """Update internal db index to match view model."""
-        self.__lock.acquire()
-        try:
-            newindex = []
-            for r in self.__viewmodel:
-                newindex.append(r[COL_EVNO])
-            self.__index = newindex
-        finally:
-            self.__lock.release()
-
     def __def_notify(self, data=None):
         """Notify view/model of changes in db."""
         if self.__viewmodel is not None:
@@ -697,6 +643,7 @@ class eventdb(object):
         self.__popup_result_data = None
         self.__popup_program_cb = None
         self.__popup_program_data = None
+        self.__evno_change_cb = None
 
         self.__lock = threading.Lock()
         self.view = None
