@@ -46,6 +46,8 @@ import metarace
 from metarace import timy
 from metarace import scbwin
 from metarace import tod
+from metarace import unt4
+from metarace import loghandler
 from metarace import uiutil
 from metarace import eventdb
 from metarace import riderdb
@@ -294,7 +296,7 @@ class f200(object):
                                         'lstart':'',
                                         'fsbib':'',
                                         'fsstat':'idle',
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'distance':'200',
                                         'autospec':'',
                                         'inomnium':'No',
@@ -302,6 +304,7 @@ class f200(object):
                                         'autoarm':defautoarm})
         cr.add_section('event')
         cr.add_section('riders')
+        cr.add_section('traces')
         if os.path.isfile(self.configpath):
             self.log.debug(u'Attempting to read config from '
                                + repr(self.configpath))
@@ -348,6 +351,11 @@ class f200(object):
             self.settimes(nri, st, ft, sp, doplaces=False, comment=co)
         self.placexfer()
 
+        # re-load any traces
+        for r in cr.get('event', 'startlist').split():
+            if cr.has_option('traces',r):
+                self.traces[r] = cr.get('traces', r).split('\n')
+
         # re-join any existing timer state
         curstart = tod.str2tod(cr.get('event', 'start'))
         lstart = tod.str2tod(cr.get('event', 'lstart'))
@@ -404,6 +412,12 @@ class f200(object):
             cw.set('event', 'lstart', self.lstart.rawtime())
         cw.set('event', 'fsstat', self.fs.getstatus())
         cw.set('event', 'fsbib', self.fs.getrider())
+
+
+        cw.add_section('traces')
+        for rider in self.traces:
+            cw.set('traces', rider, u'\n'.join(self.traces[rider]))
+
         cw.add_section('riders')
 
         # save out all starters
@@ -466,9 +480,16 @@ class f200(object):
         if placeholders == 0:
             for r in self.riders:
                 rno = r[COL_BIB]
-                rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                       r[COL_CLUB])
-                hlist.append([str(count)+'.1', rno, rname, None])
+                rh = self.meet.newgetrider(rno, self.series)
+                info = None
+                rname = u''
+                if rh is not None:
+                    rname = rh[u'namestr']
+                    if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                        ph = self.meet.newgetrider(rh[u'note'], self.series)
+                        if ph is not None:
+                            info = [[u' ',ph[u'namestr'] + u' - Pilot', ph[u'ucicode']]]
+                hlist.append([str(count)+'.1', rno, rname, info])
                                  # all heats are one up
                 count -= 1
         else:
@@ -491,7 +512,7 @@ class f200(object):
                 ret.append(rec)
                 rec = []
             heat = str(h)
-            rec.extend([heat, r[1], r[2], None])
+            rec.extend([heat, r[1], r[2], r[3]])
             lcnt += 1
             lh = h
         if len(rec) > 0:
@@ -659,8 +680,16 @@ class f200(object):
             slist = strops.reformat_riderlist(
                           b.get_object('race_starters_entry').get_text(),
                                         self.meet.rdb, self.series).split()
-            for s in slist:
-                self.addrider(s)
+
+            # if no starters yet - automatically seed by order entered
+            if len(self.riders) == 0:
+                cnt = 1
+                for s in slist:
+                    self.addrider(s, cnt)
+                    cnt += 1
+            else:
+                for s in slist:
+                    self.addrider(s)
 
             self.log.debug('Edit race properties done.')
             glib.idle_add(self.delayed_announce)
@@ -708,17 +737,25 @@ class f200(object):
         sec.lines = []
         ftime = None
         for r in self.riders:
+            rcat = None		# should check ev[u'cate']
+            plink = u''
             rno = r[COL_BIB].decode('utf-8')
             rank = None
-            rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8'),
-                                   r[COL_LASTNAME].decode('utf-8'),
-                                   r[COL_CLUB].decode('utf-8'))
+            rh = self.meet.newgetrider(rno, self.series)
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
+                # consider partners here
+                if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                    ph = self.meet.newgetrider(rh[u'note'], self.series)
+                    if ph is not None:
+                        plink = [u'', u'', ph[u'namestr'] + u' - Pilot', ph[u'ucicode'], u'', u'',u'']
+                if self.event[u'cate']:
+                    if rh[u'cat']:
+                        rcat = rh[u'cat']
+                if rh[u'ucicode']:
+                    rcat = rh[u'ucicode']   # overwrite by force
             rtime = None
-            rcat = None		# should check ev[u'cate']
-            if self.event[u'cate']:
-                dbr = self.meet.rdb.getrider(rno, self.series)
-                if dbr is not None:
-                    rcat = self.meet.rdb.getvalue(dbr,4)
             info = None
             dtime = None
             if self.onestart:
@@ -735,7 +772,7 @@ class f200(object):
                         dtime = u'+' + (time - ftime).rawtime(2)
                     rtime = time.rawtime(3)
 
-            sec.lines.append([rank, rno, rname, rcat, rtime, dtime])
+            sec.lines.append([rank, rno, rname, rcat, rtime, dtime, plink])
         ret.append(sec)
         return ret
 
@@ -794,11 +831,21 @@ class f200(object):
             self.meet.scbwin.sett1(self.fs.ck.get_text())
             dist = self.meet.get_distance(self.distance, self.units)
             if dist is not None:
-                spstr = (t - self.curstart).speedstr(dist).strip()
+                elap = t - self.curstart
+                spstr = elap.speedstr(dist).strip()
                 glib.timeout_add_seconds(1, self.clear_200_ttb,
                                             self.meet.scbwin,
                                             'Avg:',
                                             spstr.rjust(12))
+                spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
+                msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
+                                    xx=1,
+                                    yy=4,
+                                    text=spt)
+                self.log.info(u'sending: ' + msg.text)
+                self.meet.udptimer.sendto(msg.pack(),
+                                          (self.meet.udpaddr,6789))
+ 
             else:
                 glib.timeout_add_seconds(2, self.clear_200_ttb,
                                             self.meet.scbwin)
@@ -930,9 +977,15 @@ class f200(object):
         place = 1
         for t in self.results:
             bib = t[0].refid
-            if t[0] == tod.FAKETIMES['dsq']:
-                #place = 'comment'	# no need for lookup now
-                place = 'dsq'
+            if t[0] > tod.FAKETIMES['max']:
+                if t[0] == tod.FAKETIMES['dsq']:
+                    place = 'dsq'
+                elif t[0] == tod.FAKETIMES['dns']:
+                    place = 'dns'
+                elif t[0] == tod.FAKETIMES['dnf']:
+                    place = 'dnf'
+                #else:
+                    #place = 'dnf' # leave to naturally sort
             else:
                 place = self.results.rank(bib)+1
             i = self.getiter(bib)
@@ -1004,6 +1057,12 @@ class f200(object):
                 self.meet.timer.dearm(cid)
                 self.meet.timer.dearm(timy.CHAN_FINISH)# assume manual override
 
+    #!def trace(self, rider, line):
+        #!"""Append the suppliedmessage to rider trace."""
+        #!if rider not in self.traces:
+            #!self.traces[rider] = u''
+        #!self.traces[rider] += '\n' + line
+
     def abortrider(self, sp):
         """Abort the selected lane."""
         if sp.getstatus() not in ['idle', 'finish']:
@@ -1021,8 +1080,8 @@ class f200(object):
         if self.timerstat == 'running':
             if self.fs.getstatus() not in ['idle', 'finish']:
                 self.fs.toload()
-                self.meet.timer_log_msg(self.fs.getrider(),
-                                        '- False start -')
+                bib = fs.getrider()
+                self.meet.timer_log_msg(self.fs.getrider(), '- False start -')
                 if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
                     self.meet.scbwin.setr1('False')
                     self.meet.scbwin.sett1('Start')
@@ -1038,8 +1097,8 @@ class f200(object):
         if self.timerstat == 'running':
             if sp.getstatus() in ['running', 'finish']:
                 if sp.getstatus() == 'finish':
-                     self.meet.timer_log_msg(sp.getrider(),
-                                             '- False finish -')
+                     bib = sp.getrider()
+                     self.meet.timer_log_msg(sp.getrider(), '- False Finish -')
                      self.meet.scbwin.setr1('')
                      self.meet.scbwin.setr2('')
                 sp.toarmfin()
@@ -1081,8 +1140,7 @@ class f200(object):
         self.meet.scbwin = None
         self.meet.scbwin = scbwin.scbtt(self.meet.scb,
                                 self.meet.racenamecat(self.event),
-                                self.fmtridername(self.fs),
-                                '')
+                                self.fmtridername(self.fs))
         self.meet.gemini.reset_fields()
         self.meet.gemini.set_bib(self.fs.getrider())
         self.meet.gemini.show_brt()
@@ -1104,15 +1162,19 @@ class f200(object):
             self.lstart = None
             self.meet.timer.arm(timy.CHAN_200)
             self.showtimerwin()
-            self.meet.printimp(True)
             self.meet.delayimp('0.01')
             if self.fs.status == 'armstart':
+                bib = self.fs.getrider()
+                if bib not in self.traces:
+                    self.traces[bib] = []
+                self.fslog = loghandler.traceHandler(self.traces[bib])
+                logging.getLogger().addHandler(self.fslog)
                 self.meet.scbwin.sett1('       0.0     ')
                 nstr = self.fs.biblbl.get_text()
                 if not self.onestart:
                     self.meet.timer.printline(self.meet.racenamecat(self.event))
-                self.meet.timer_log_msg(self.fs.getrider(), nstr)
-                self.meet.gemini.set_bib(self.fs.getrider())
+                self.meet.timer_log_msg(bib, nstr)
+                self.meet.gemini.set_bib(bib)
                 self.meet.gemini.set_time(' 0.0 ')
                 self.meet.gemini.set_rank('')
                 self.meet.gemini.show_brt()
@@ -1120,11 +1182,13 @@ class f200(object):
 
     def toidle(self, idletimers=True):
         """Set timer to idle state."""
+        if self.fslog is not None:
+            logging.getLogger().removeHandler(self.fslog)
+            self.fslog = None
         if idletimers:
             self.fs.toidle()
         self.timerstat = 'idle'
-        self.meet.printimp(False)
-        self.meet.delayimp('1.00')
+        self.meet.delayimp('2.00')
         self.curstart = None
         self.lstart = None
         for i in range(0,8):
@@ -1210,6 +1274,15 @@ class f200(object):
             #self.log_clear(self.riders.get_value(sel[1], COL_BIB))
             glib.idle_add(self.delayed_announce)
 
+    def tod_context_print_activate_cb(self, menuitem, data=None):
+        """Print Rider trace"""
+        sel = self.view.get_selection().get_selected()
+        if sel is not None:
+            bib = self.riders.get_value(sel[1], COL_BIB)
+            if bib in self.traces:
+                self.meet.timer_reprint(self.meet.racenamecat(self.event),
+                                        self.traces[bib])
+
     def now_button_clicked_cb(self, button, entry=None):
         """Set specified entry to the 'now' time."""
         if entry is not None:
@@ -1345,12 +1418,16 @@ class f200(object):
         self.info_ent.connect('changed', self.editent_cb,u'info')
         self.info_ent.set_text(self.event[u'info'])
         b.get_object('race_type').set_text('Flying 200')
+        self.traces = {}
 
         # Timer Pane
         mf = b.get_object('race_timer_pane')
         self.fs = timerpane.timerpane('Timer')
+        self.fs.uport = self.meet.udptimer
+        self.fs.uaddr = self.meet.udpaddr
         self.fs.bibent.connect('activate', self.bibent_cb, self.fs)
         self.fs.hide_laps()
+        self.fslog = None
         mf.pack_start(self.fs.frame)
 
         # Result Pane
