@@ -272,7 +272,7 @@ class flap(object):
                                         'lstart':'',
                                         'fsbib':'',
                                         'fsstat':'idle',
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'distance':'250',
                                         'autospec':'',
                                         'inomnium':'No',
@@ -1088,7 +1088,6 @@ class flap(object):
             self.lstart = None
             self.meet.timer.arm(timy.CHAN_FINISH)
             self.showtimerwin()
-            self.meet.printimp(True)
             if self.fs.status == 'armstart':
                 self.meet.scbwin.sett1('       0.0     ')
                 nstr = self.fs.biblbl.get_text()
@@ -1106,7 +1105,6 @@ class flap(object):
         if idletimers:
             self.fs.toidle()
         self.timerstat = 'idle'
-        self.meet.printimp(False)
         self.curstart = None
         self.lstart = None
         for i in range(0,8):
