@@ -185,7 +185,7 @@ class hl970:
     def quit_cb(self, menuitem, data=None):
         """Quit the application."""
         self.running = False
-        self.window.destroy()
+        #self.window.destroy()
 
     def uscb_activate_cb(self, menuitem, data=None):
         """Request a re-connect to the uSCBsrv IRC connection."""
@@ -216,11 +216,13 @@ class hl970:
     ## Help menu callbacks
     def menu_help_docs_cb(self, menuitem, data=None):
         """Display program help."""
-        metarace.help_docs(self.window)
+        #metarace.help_docs(self.window)
+        pass
 
     def menu_help_about_cb(self, menuitem, data=None):
         """Display metarace about dialog."""
-        metarace.about_dlg(self.window)
+        #metarace.about_dlg(self.window)
+        pass
 
     def timeout(self):
         """Update status."""
@@ -244,13 +246,19 @@ class hl970:
 
         # 3: Re-Schedule
         tt = tod.tod(u'now')+tod.tod(u'0.01')
+        count = 0
         while self.nc < tt:     # ensure interval is positive
             if tod.MAX - tt < tod.ONE:
                 self.log.debug(u'Midnight rollover.')
                 break
             self.log.debug(u'May have missed an interval, catching up.')
             self.nc += tod.ONE  # 0.01 allows for processing delay
+            count += 1
+            if count > 10:
+                break
         ival = int(1000.0 * float((self.nc - tod.tod(u'now')).timeval))
+        if ival < 0 or ival > 10000:
+            ival = 1000	# assume host time change
         glib.timeout_add(ival, self.timeout)
 
         # 4: Return False
@@ -277,7 +285,7 @@ class hl970:
                         elap = tod.ZERO
                     estr = elap.rawtime(0)
                     if self.distance is not None and self.distance > 0.5:
-                        dstr = u'~{0:1.1f}km'.format(self.distance)
+                        dstr = u'\u009f{0:1.1f}km'.format(self.distance)
                     if self.lapfin is not None:
                         # lap down time overwrites dist, but only if valid
                         laptm = self.tod - self.lapfin
@@ -286,7 +294,7 @@ class hl970:
                 estr = estr.rjust(7)
                 dstr = dstr.rjust(16-len(estr))
                 self.set_bot_line(estr + dstr)
-            elif self.ttno:	# Time trial override
+            elif self.ttno or self.ttname:	# Time trial override
                 rstr = u''
                 if self.ttrank:
                     rstr = u'(' + self.ttrank + u')' + ' '
@@ -311,6 +319,8 @@ class hl970:
                     nmstr = self.ttname
                 nmstr = nmstr.ljust(12)
                 topline = nostr + u' ' + nmstr
+            elif self.ttname: # assume full size for TTT
+                topline = strops.truncpad(self.ttname, 16, align='c')
             elif self.finstr:	# finish string overwrites top line
                 topline = strops.truncpad(self.finstr, 16, align='c')
             elif self.laplbl or self.laptype:
@@ -345,20 +355,32 @@ class hl970:
             if self.obuf[j] != self.lbuf[j]:
                 self.setline(j)
 
+        # check connection status
+        if not self.remote.connected():
+            self.failcount += 1
+            if self.failcount > self.failthresh:
+                self.remote.set_portstr(force=True)
+                self.failcount = 0
+            self.log.debug(u'Telegraph connection failed, count = '
+                           + repr(self.failcount))
+        else:
+            self.failcount = 0
+
+
     ## Window methods
     def app_set_title(self, extra=u''):
         """Update window title from meet properties."""
         title = u'hl970 Console'
         if self.title:
             title += u': ' + self.title
-        self.window.set_title(title)
+        #self.window.set_title(title)
 
     def app_destroy_cb(self, window, msg=u''):
         """Handle destroy signal and exit application."""
         if self.dosave:
             self.saveconfig()
         self.log.removeHandler(self.sh)
-        self.window.hide()
+        #self.window.hide()
         self.log.info(u'App destroyed. ' + msg)
         glib.idle_add(self.app_destroy_handler)
 
@@ -377,7 +399,7 @@ class hl970:
         """Cleanly shutdown threads and close application."""
         self.started = False
 
-        self.window.hide()	# usually already called
+        #self.window.hide()	# usually already called
         self.remote.exit(msg)
         self.close_display()
         print (u'Waiting for remote to exit...')
@@ -520,8 +542,8 @@ class hl970:
     def but_man_clicked_cb(self, button, data=None):
         """Manual display set."""
         self.log.debug(u'Manual display override.')
-        self.set_top_line(self.line0.get_text().decode('utf-8'))
-        self.set_bot_line(self.line1.get_text().decode('utf-8'))
+        #self.set_top_line(self.line0.get_text().decode('utf-8'))
+        #self.set_bot_line(self.line1.get_text().decode('utf-8'))
 
     def set_top_line(self, msg):
         msg = strops.truncpad(msg, 16, elipsis=False)
@@ -547,6 +569,23 @@ class hl970:
         self.gfxcmd(0x30, 'V',
                 bytearray([0xf0]) + i2b24(0x180) + i2b16(csum16(self.logo)))
 
+    def do_scroll(self, scrollmsg):
+        scrollmsg = scrollmsg.rstrip()
+        self.scrolltxt = scrollmsg
+        if len(scrollmsg) > SCROLLMAX:
+            scrollmsg = strops.truncpad(scrollmsg, SCROLLMAX)
+            # try to truncate on a word boundary
+            idf = scrollmsg.rfind(u' ')
+            if idf > SCROLLCUT:
+                scrollmsg = scrollmsg[0:idf].rstrip() + u' ...'
+        if len(scrollmsg) > 0:
+            self.serialwrite(chr(STX) + chr(0x80) + '11' +
+                          scrollmsg.encode(ENCODING, 'replace') + chr(LF))
+            #self.log.debug(u'Wrote msg: ' + repr(scrollmsg))
+        else:
+            #self.log.debug('Nothing to scroll.')
+            self.scrolloff_clicked_cb(None)
+
     def scrolltext_activate_cb(self, entry, data=None):
         """Copy scrolling text to display on bottom line."""
         scrollmsg = self.scrolltext.get_text().decode('utf-8').rstrip()
@@ -572,16 +611,16 @@ class hl970:
         self.serialwrite(chr(STX) + chr(0x90) + '111234567890' + chr(LF))
 
     def set_remote_enable(self):
-        self.line0.set_sensitive(False)
-        self.line1.set_sensitive(False)
-        uiutil.buttonchg(self.but_remote, uiutil.bg_armstart,'Remote')
+        #self.line0.set_sensitive(False)
+        #self.line1.set_sensitive(False)
+        #uiutil.buttonchg(self.but_remote, uiutil.bg_armstart,'Remote')
         self.clearlines()	# refresh cache
         self.remote_enable = True
 
     def set_remote_disable(self):
-        self.line0.set_sensitive(True)
-        self.line1.set_sensitive(True)
-        uiutil.buttonchg(self.but_remote, uiutil.bg_armfin,'Local')
+        #self.line0.set_sensitive(True)
+        #self.line1.set_sensitive(True)
+        #uiutil.buttonchg(self.but_remote, uiutil.bg_armfin,'Local')
         self.remote_enable = False
 
     def but_remote_clicked_cb(self, button, data=None):
@@ -730,8 +769,9 @@ class hl970:
                 if fpvec[4]:
                     self.tttime = fpvec[4]
         elif cmd.header == u'scrollmsg':
-            self.scrolltext.set_text(cmd.text)
-            self.scrolltext.activate()
+            self.do_scroll(cmd.text)
+            #self.scrolltext.set_text(cmd.text)
+            #self.scrolltext.activate()
         elif cmd.header == u'timerstat':
             self.timerstat = cmd.text
         elif cmd.header == u'brightness':
@@ -745,6 +785,9 @@ class hl970:
                 self.but_clear_clicked_cb(None)
         elif cmd.header == u'clearall':
             self.but_clear_clicked_cb(None)
+        elif cmd.header == u'catlaps':
+            lvec = cmd.text.split(unichr(unt4.US))
+            glib.idle_add(self.draw_laptogo, lvec)
         elif cmd.header == u'leaderboard':
             lvec = cmd.text.split(unichr(unt4.US))
             glib.idle_add(self.draw_vpage, lvec)
@@ -781,10 +824,117 @@ class hl970:
         #logo2pbm(self.logo)
         return False
 
+    def draw_laptogo(self, lapsrc):
+        if self.logo is None:
+            self.logo = bytearray(384)	# 192x16/8
+        linesrc = [0,0,0,1, 1,1,1,1,
+                   1,1,1,1, 1,1,1,1,
+                   1,1,1,1, 1,1,1,1,
+                   1,1,1,1, 1,0,0,0]
+        topprompt = lapsrc[0]
+        topstr = strops.truncpad(lapsrc[1].strip(),2,align='r')
+        botprompt = lapsrc[2]
+        botstr = strops.truncpad(lapsrc[3].strip(),2,align='r')
+
+        if self.vfont is not None and self.vlfont is not None:
+            oft = 0 #offset down panel
+            if topprompt:
+                line = strops.truncpad(
+                              topprompt.translate(strops.PLACELIST_UTRANS),
+                              5).encode('ascii','ignore')
+                cset(self.logo, 0, oft, self.vfont[ord(line[0])-0x20])
+                cset(self.logo, 6, oft, self.vfont[ord(line[1])-0x20])
+                cset(self.logo, 12, oft, self.vfont[ord(line[2])-0x20])
+                for i in range(0,8):
+                    bset(self.logo, 18,oft+i, 0)
+                    bset(self.logo, 19,oft+i, 0)
+                cset(self.logo, 20, oft, self.vfont[ord(line[3])-0x20])
+                cset(self.logo, 26, oft, self.vfont[ord(line[4])-0x20])
+            else:
+                for j in range(0,8):
+                    for i in range(0,32):
+                        bset(self.logo, i, oft, 0)
+                    oft += 1
+
+            oft = 8 #offset down panel
+            while oft < 10:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+            oft = 10 #offset down panel
+            if topstr:
+                num = topstr.translate(strops.INTEGER_UTRANS)
+                nl = 10
+                nh = 10
+                if len(num) > 0 and num[0].isdigit():
+                    nl = int(num[0])
+                if len(num) > 1 and num[1].isdigit():
+                    nh = int(num[1])
+                nset(self.logo, 0, oft, self.vlfont[nl])
+                nset(self.logo, 16,oft, self.vlfont[nh])
+                oft += 22
+            while oft < 46:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+
+            oft = 46
+            for i in range(0,32):
+                bset(self.logo, i, oft, linesrc[i])
+                bset(self.logo, i, oft+1, 0)
+
+            oft = 48
+            if botprompt:
+                line = strops.truncpad(
+                              botprompt.translate(strops.PLACELIST_UTRANS),
+                              5).encode('ascii','ignore')
+                cset(self.logo, 0, oft, self.vfont[ord(line[0])-0x20])
+                cset(self.logo, 6, oft, self.vfont[ord(line[1])-0x20])
+                cset(self.logo, 12, oft, self.vfont[ord(line[2])-0x20])
+                for i in range(0,8):
+                    bset(self.logo, 18,oft+i, 0)
+                    bset(self.logo, 19,oft+i, 0)
+                cset(self.logo, 20, oft, self.vfont[ord(line[3])-0x20])
+                cset(self.logo, 26, oft, self.vfont[ord(line[4])-0x20])
+            else:
+                for j in range(0,8):
+                    for i in range(0,32):
+                        bset(self.logo, i, oft, 0)
+                    oft += 1
+                
+            oft = 56 #offset down panel
+            while oft < 58:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+            oft = 58 #offset down panel
+            if botstr:
+                num = botstr.translate(strops.INTEGER_UTRANS)
+                nl = 10
+                nh = 10
+                if len(num) > 0 and num[0].isdigit():
+                    nl = int(num[0])
+                if len(num) > 1 and num[1].isdigit():
+                    nh = int(num[1])
+                nset(self.logo, 0, oft, self.vlfont[nl])
+                nset(self.logo, 16,oft, self.vlfont[nh])
+                oft += 22
+            while oft < 96:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+            self.but_logo_clicked_cb(None)
+        #logo2pbm(self.logo)
+        return False
+        
+
     def draw_vpage(self, page):
         """Draw the supplied vertical page spec into dimg and display."""
         # based on geometry: 32x96, chars of 6x8, lines of 5 chars
-        # NOT OPTIMISED IN ANY WAY!
         if self.logo is None:
             self.logo = bytearray(384)	# 192x16/8
         linesrc = [0,0,0,1, 1,1,1,1,
@@ -838,8 +988,8 @@ class hl970:
 
     def mirror_lines(self):
         """Copy line buffers back to UI."""
-        self.line0.set_text(self.obuf[0]+self.obuf[1])
-        self.line1.set_text(self.obuf[4]+self.obuf[5])
+        #self.line0.set_text(self.obuf[0]+self.obuf[1])
+        #self.line1.set_text(self.obuf[4]+self.obuf[5])
         return False
 
     def setline(self, line):
@@ -873,7 +1023,7 @@ class hl970:
         self.close_display()
         self.log.debug('Connecting serial port: ' + repr(self.port))
         try:
-            self.scb = serial.Serial(self.port, HL970_BAUD, timeout=0.2)
+            self.scb = serial.Serial(self.port, HL970_BAUD, rtscts=False,timeout=0.2)
             self.stdcmd(0x40, ord('I'))
         except Exception as e:
             self.log.error(u'Opening serial port: ' + repr(e))
@@ -892,12 +1042,13 @@ class hl970:
         cw.set(u'hl970', u'logo', self.logofile)
         cw.set(u'hl970', u'loglevel', self.loglevel)
         cw.set(u'hl970', u'maxlaptime', self.maxlaptime.rawtime())
-        cw.set(u'hl970', u'toplinetext',
-                         self.line0.get_text().decode('utf-8'))
-        cw.set(u'hl970', u'botlinetext',
-                         self.line1.get_text().decode('utf-8'))
+        #cw.set(u'hl970', u'toplinetext',
+                         #self.line0.get_text().decode('utf-8'))
+        #cw.set(u'hl970', u'botlinetext',
+                         #self.line1.get_text().decode('utf-8'))
         cw.set(u'hl970', u'scrolltext',
-                         self.scrolltext.get_text().decode('utf-8'))
+                         self.scrolltxt)
+                         #self.scrolltext.get_text().decode('utf-8'))
 
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         self.log.debug(u'Saving app config to ' + repr(cwfilename))
@@ -963,7 +1114,7 @@ class hl970:
         # configurable log level on UI (does not change log file)
         self.loglevel = strops.confopt_posint(cr.get(u'hl970', u'loglevel'),
                                               logging.INFO)
-        self.sh.setLevel(self.loglevel)
+        #self.sh.setLevel(self.loglevel)
 
         # brightness
         self.set_brightness(cr.get(u'hl970', u'brightness'))
@@ -985,9 +1136,10 @@ class hl970:
             self.maxlaptime = mlap
 
         # restore strings
-        self.line0.set_text(cr.get(u'hl970', u'toplinetext')[0:16])
-        self.line1.set_text(cr.get(u'hl970', u'botlinetext')[0:16])
-        self.scrolltext.set_text(cr.get(u'hl970', u'scrolltext'))
+        #self.line0.set_text(cr.get(u'hl970', u'toplinetext')[0:16])
+        #self.line1.set_text(cr.get(u'hl970', u'botlinetext')[0:16])
+        #self.scrolltext.set_text(cr.get(u'hl970', u'scrolltext'))
+        self.scrolltxt = (cr.get(u'hl970', u'scrolltext'))
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
@@ -1027,21 +1179,22 @@ class hl970:
             self.obuf.append(u''.ljust(8))
             self.lbuf.append(u'01234567')
    
-        b = gtk.Builder()
-        b.add_from_file(os.path.join(metarace.UI_PATH, u'hl970.ui'))
-        self.window = b.get_object('appwin')
-        self.status = b.get_object('status')
+        #b = gtk.Builder()
+        #b.add_from_file(os.path.join(metarace.UI_PATH, u'hl970.ui'))
+        #self.window = b.get_object('appwin')
+        #self.status = b.get_object('status')
         self.brightness = 0x31
-        self.line0 = b.get_object('line0')
-        self.line1 = b.get_object('line1')
-        self.scrolltext = b.get_object('scrolltext')
-        self.scrolltext.grab_focus()
-        self.but_remote = b.get_object('but_remote')
-        for l in [self.line0, self.line1]:
-            l.modify_font(pango.FontDescription('unifont 24'))
+        #self.line0 = b.get_object('line0')
+        #self.line1 = b.get_object('line1')
+        #self.scrolltext = b.get_object('scrolltext')
+        self.scrolltxt = u''
+        #self.scrolltext.grab_focus()
+        #self.but_remote = b.get_object('but_remote')
+        #for l in [self.line0, self.line1]:
+            #l.modify_font(pango.FontDescription('unifont 24'))
             #l.modify_font(pango.FontDescription('monospace bold 24'))
-        self.context = self.status.get_context_id('metarace hl970')
-        b.connect_signals(self)
+        #self.context = self.status.get_context_id('metarace hl970')
+        #b.connect_signals(self)
         self.set_remote_enable()
 
         # run state
@@ -1071,13 +1224,15 @@ class hl970:
         self.ttcat = None
         self.tttime = None
         self.timeofday = True  # show timeofday on bottom line?
+        self.failcount = 0
+        self.failthresh = 30    # connect timeout ~30sec
 
         # format and connect status handlers
-        f = logging.Formatter('%(levelname)s:%(name)s: %(message)s')
-        self.sh = loghandler.statusHandler(self.status, self.context)
-        self.sh.setLevel(logging.INFO)	# show info+ on status bar
-        self.sh.setFormatter(f)
-        self.log.addHandler(self.sh)
+        #f = logging.Formatter('%(levelname)s:%(name)s: %(message)s')
+        #self.sh = loghandler.statusHandler(self.status, self.context)
+        #self.sh.setLevel(logging.INFO)	# show info+ on status bar
+        #self.sh.setFormatter(f)
+        #self.log.addHandler(self.sh)
 
         # start timer
         self.log.debug(u'Starting clock intervals at: ' + self.nc.rawtime(3))
@@ -1102,7 +1257,7 @@ def main():
         os.chdir(configpath)
     app = hl970(configpath)
     app.loadconfig()
-    app.window.show()
+    #app.window.show()
     app.start()
     try:
         metarace.mainloop()
