@@ -44,6 +44,10 @@ COL_CUR = 2
 COL_LAST = 3
 COL_BEST = 4
 
+# sort on time
+def sort_timefield(x,y):
+    return cmp(x[2],y[2])
+
 # scb function key mappings
 key_reannounce = 'F4'	# (+CTRL)
 key_abort = 'F5'	# (+CTRL)
@@ -65,7 +69,15 @@ class hotlap(object):
                                + repr(self.configpath))
             cr.read(self.configpath)
         for r in cr.get('event', 'startlist').split():
-            self.addrider(r)
+            lasttod = None
+            besttod = None
+            if cr.has_option('laps', r):
+                ril = csv.reader([cr.get('laps', r)]).next() 
+                if len(ril) > 0:
+                    lasttod = tod.str2tod(ril[0])
+                if len(ril) > 1: 
+                    besttod = tod.str2tod(ril[1])
+            self.addrider(r, last=lasttod, best=besttod)
             ## TODO : load/save comment for rider
 
         self.info_expand.set_expanded(strops.confopt_bool(
@@ -120,6 +132,15 @@ class hotlap(object):
         cw.set('event', 'showinfo', self.info_expand.get_expanded())
         cw.set('event', 'id', EVENT_ID)
         self.log.debug('Saving race config to: ' + self.configpath)
+        cw.add_section('laps')
+        for r in self.riders:
+            last = u''
+            if r[COL_CUR]:
+                last = r[COL_CUR].rawtime()
+            best = u''
+            if r[COL_BEST]:
+                best = r[COL_BEST].rawtime()
+            cw.set('laps', r[COL_BIB], u','.join([last, best]))
         with open(self.configpath, 'wb') as f:
             cw.write(f)
 
@@ -183,9 +204,9 @@ class hotlap(object):
         ret.append(sec)
         return ret
 
-    def addrider(self, bib='', info=None):
+    def addrider(self, bib='', info=None, last=None, best=None):
         """Add specified rider to race model."""
-        nr=[bib, '', None, None, None]
+        nr=[bib, '', last, best, best]
         if bib == '' or self.getrider(bib) is None:
             dbr = self.meet.rdb.getrider(bib, self.series)
             if dbr is not None:
@@ -330,26 +351,29 @@ class hotlap(object):
 
     def do_places(self):
         """Show race result on scoreboard."""
+        name_w = self.meet.scb.linelen - 9
+        coldesc = [(3, u'l'), u' ', (name_w,u'l'), u' ', (4,u'r')]
         resvec = []
         for r in self.riders:
-            if r[COL_PLACE] is not None and r[COL_PLACE] != '':
-                plstr = r[COL_PLACE].decode('utf-8')
-                if plstr.isdigit():
-                    if int(plstr) > 20:
-                        break
-                    plstr = plstr + u'.'
-                resvec.append([plstr,
-                    r[COL_BIB].decode('utf-8'),
-                    strops.fitname(r[COL_FIRST].decode('utf-8'),
-                                   r[COL_LAST].decode('utf-8'),
-                                        SCB_RESPOINTS_NW),
-                    str(r[COL_TOTAL])])
+            thetime = r[COL_BEST] 
+            if thetime:
+                resvec.append([None, r[COL_NAME].decode('utf-8', 'ignore'),thetime])
+        resvec.sort(sort_timefield)
+        lt = None
+        ct = 0
+        lp = None
+        for r in resvec:
+            ct += 1
+            if r[2] != lt:
+                lp = unicode(ct)+u'.'
+            r[0] = lp
+            r[2] = r[2].rawtime(1)
         ## TODO: detect standings after nn/provisonal/final
         evtstatus = self.standingstr()
         self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
                                     head=self.meet.racenamecat(self.event),
                                     subhead=evtstatus,
-                                    coldesc=SCB_RESPOINTS_FMT,
+                                    coldesc=coldesc,
                                     rows=resvec)
         self.meet.scbwin.reset()
         return False
@@ -398,6 +422,7 @@ class hotlap(object):
                         self.log.info(u'New Best Lap: ' + repr(bib) + ' :: '
                                  + laptime.rawtime(2))
                         lr[COL_BEST] = laptime
+                        glib.idle_add(self.do_places)
                 else:
                     lr[COL_BEST] = laptime
                 lr[COL_LAST] = laptime
@@ -556,8 +581,8 @@ class hotlap(object):
         self.resultvirtual = False
         self.eventstatusmap = {}
         self.hotlap = None
-        self.minlap = tod.tod(10)
-        self.maxlap = tod.tod(40)
+        self.minlap = tod.tod(15)
+        self.maxlap = tod.tod(32)
 
         # race run time attributes
         self.onestart = False
