@@ -30,39 +30,30 @@ Example for an empty element:
 
     hr(attrs={'id':'thehr'})
 
-returns the string: '<hr id="thehr" />'
-
 Example for an element with content:
 
     a(['link text'], attrs={'href':'#target'})
 
-returns the string: '<a href="#target">\nlink text\n</a>'
-
 Example paragraph:
 
     p(['Check the',
        a(['website'], attrs={'href':'#website'}),
        'for more.'])
 
-Returns: '<p>\nCheck the\n<a href="#website">\nwebsite\n</a>\nfor more.\n</p>'
-
 """
 
 def html(headlist=[], bodylist=[]):
     """Emit HTML document."""
     return u'\n'.join([
       preamble(),
-      u'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">',
+      u'<html lang="en">',
       head(headlist),
       body(bodylist),
       u'</html>'])
 
 def preamble():
-    """Emit XHTML preamble."""
-    return u"""<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE html 
-     PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
-     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">"""
+    """Emit HTML preamble."""
+    return u'<!DOCTYPE html>'
 
 def attrlist(attrs):
     """Convert attr dict into properly escaped attrlist."""
@@ -78,6 +69,14 @@ def escapetext(text=u''):
     """Return escaped copy of text."""
     return escape(text, {u'"':u'&quot;'})
 
+def tablestyles():
+    """Emit the fixed table styles."""
+    return u'\ntable.middle td { vertical-align: middle; }\n th.center { text-align: center; }\n th.right { text-align: right; }\n td.right { text-align: right; }\n'
+
+def shim(shivlink, respondlink):
+    """Emit the HTML5 shim for IE8."""
+    return u'\n<!--[if le IE 9]>\n<script src=' + quoteattr(shivlink) + '>\n</script>\n<script src=' + quoteattr(respondlink) + '>\n</script>\n<![endif]-->\n'
+
 def comment(commentstr=u''):
     """Insert comment."""
     return u'<!-- ' + commentstr.replace(u'--',u'') + u' -->'
@@ -86,12 +85,12 @@ def comment(commentstr=u''):
 for empty in [u'meta', u'link', u'base', u'param',
               u'hr', u'br', u'img', u'input', u'col']:
     def emptyfunc(attrs={}, elem=empty):
-        return u'<' + elem + attrlist(attrs) + u' />'
+        return u'<' + elem + attrlist(attrs) + u'>'
     setattr(sys.modules[__name__], empty, emptyfunc) 
 
 # Declare all the non-empties
 for nonempty in [u'head', u'body', u'title', u'div', u'style', u'script',
-                 u'object', u'p', u'h1', u'h2', u'h3', u'h4', u'h5', u'h6',
+                 u'p', u'h1', u'h2', u'h3', u'h4', u'h5', u'h6',
                  u'ul', u'ol', u'li', u'dl', u'dt', u'dd', u'address',
                  u'pre', u'blockquote', u'a', u'span', u'em', u'strong',
                  u'dfn', u'code', u'samp', u'kbd', u'var', u'cite',
@@ -108,6 +107,31 @@ for nonempty in [u'head', u'body', u'title', u'div', u'style', u'script',
                 + u'\n'.join(clist) + u'</' + elem + u'>')
     setattr(sys.modules[__name__], nonempty, nonemptyfunc) 
 
+# output a valid but empty html templatye
+def emptypage():
+    return html([
+                           meta(attrs={u'charset':u'utf-8'}),
+           meta(attrs={u'http-equiv':u'X-UA-Compatible',
+                       u'content':u'IE=edge'}),
+           meta(attrs={u'name':u'viewport',
+                       u'content':u'width=device-width, initial-scale=1'}),
+           title(u'__REPORT_TITLE__'),
+           link(attrs={u'href':u'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css',
+                       u'integrity':u'sha384-1q8mTJOASx8j1Au+a5WDVnPi2lkFfwwEAa8hDDdjZlpLegxhjVME1fgjWPGmkzs7',
+                       u'crossorigin':u'anonymous',
+                       u'rel':u'stylesheet'}),
+           style(tablestyles(), {u'type':'text/css'}),
+           shim(u'https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js',
+                u'https://oss.maxcdn.com/respond/1.4.2/respond.min.js'),
+                ],
+                div([h1(u'__REPORT_TITLE__'),
+                     u'\n', comment(u'Begin report content'),
+                     u'__REPORT_CONTENT__',
+                     comment(u'End report content'),u'\n',
+script(u'\n', attrs={'src':u'https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js'}),
+script(u'\n', attrs={'src':u'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/js/bootstrap.min.js', u'integrity':u'sha384-0mSbJDEHialfmuBBQP6A4Qrprq5OVfW37PRR3j5ELqxss1yVqOtnepnHVP9aJ7xS', u'crossorigin':u'anonymous'}),
+                 ],
+                    attrs={u'class':u'container'}))
 
 # test run
 if __name__ == "__main__":
