@@ -128,7 +128,7 @@ class rr_announce(object):
 
     def window_destroy_cb(self, window):
         """Handle destroy signal."""
-        self.hide()
+        #self.hide()
         glib.idle_add(self.app_destroy_handler)
     
     #def map_area_expose_event_cb(self, widget, event):
@@ -296,12 +296,12 @@ class rr_announce(object):
                     rs += '.'
                 self.fin_rank.set_text(rs)
                 self.fin_rider.set_text(sr[1] + ' ' + sr[2] + ' ' + sr[3])
-                #self.fin_box.show()
+                self.fin_box.show()
             else:		# clearit
                 self.fin_rank.set_text('')
                 self.fin_rider.set_text('')
                 self.fin_time.set_text('')
-                #self.fin_box.hide()
+                self.fin_box.hide()
 
     def append_rider(self, msg):
         sr = msg.split(chr(unt4.US))
@@ -334,19 +334,14 @@ class rr_announce(object):
         sr = msg.split(chr(unt4.US))
         if len(sr) == 6:
             rs = sr[0]
-            rftime = tod.str2tod(sr[4])
-            if rftime is not None:
-                    self.last_time = rftime
-                    nr=[rs,sr[1],sr[2],sr[3],rftime.rawtime(2),
-                        sr[5],None]
-            else: 
+            nr=[sr[0],sr[1],sr[2],sr[3],sr[4],sr[5],None]
+        else: 
                 # Informative non-timeline record
-                nr=[sr[0],sr[1],sr[2],sr[3],
-                        '', '', None]
+            nr=[sr[0],sr[1],sr[2],sr[3], '', '', None]
                 
-            self.inters.append(nr)
-            #self.map_redraw()		# update src map
-            #self.map_area.queue_draw()	# queue copy to screen
+        self.inters.append(nr)
+        #self.map_redraw()		# update src map
+        #self.map_area.queue_draw()	# queue copy to screen
 
     def msg_cb(self, m, nick, chan):
         """Handle message packet in main thread."""
@@ -500,8 +495,8 @@ class rr_announce(object):
         self.int_time = b.get_object('int_time_lbl')
         self.int_time.modify_font(self.font)
 
-        #self.fin_box = b.get_object('fin_box')
-        #self.fin_box.hide()
+        self.fin_box = b.get_object('fin_box')	# seems to work?
+        self.fin_box.hide()
 
         #self.map_winsz = 0
         #self.map_xoft = 0
@@ -528,9 +523,9 @@ class rr_announce(object):
         self.inters = gtk.ListStore(gobject.TYPE_STRING,  # rank
                                     gobject.TYPE_STRING,  # no.
                                     gobject.TYPE_STRING,  # namestr
-                                    gobject.TYPE_STRING,  # cat/com
-                                    gobject.TYPE_STRING,  # timestr
-                                    gobject.TYPE_STRING,  # downtime
+                                    gobject.TYPE_STRING,  # turn
+                                    gobject.TYPE_STRING,  # eta
+                                    gobject.TYPE_STRING,  # speed
                                     gobject.TYPE_PYOBJECT) # rftod
         t = gtk.TreeView(self.riders)
         self.view = t
@@ -556,6 +551,8 @@ class rr_announce(object):
         t.show()
         t.grab_focus()
 
+        ## let's suppress the arrivals for now
+        ##b.get_object('arrival_xpnd').hide()
         t = gtk.TreeView(self.inters)
         self.iview = t
         t.set_reorderable(False)
@@ -565,7 +562,7 @@ class rr_announce(object):
         uiutil.mkviewcoltxt(t, '', 0,width=40)
         uiutil.mkviewcoltxt(t, 'No.', 1,calign=1.0,width=40)
         uiutil.mkviewcoltxt(t, 'Rider', 2,expand=True,fixed=True)
-        uiutil.mkviewcoltxt(t, 'Rank', 3,calign=0.0)
+        uiutil.mkviewcoltxt(t, 'Turn', 3,calign=0.0)
         uiutil.mkviewcoltxt(t, 'Time', 4,calign=1.0,width=100)
         uiutil.mkviewcoltxt(t, 'Avg', 5,calign=1.0,width=90)
         t.show()
