@@ -34,11 +34,6 @@ times are reported via uscbsrv for the irtt race handler.
 import pygtk
 pygtk.require("2.0")
 
-# !! TODO: The following should be optional !!
-import pygst
-pygst.require("0.10")
-import gst
-
 import gtk
 import glib
 import pango
@@ -53,6 +48,7 @@ from metarace import timy
 from metarace import telegraph
 from metarace import strops
 from metarace import ucsv
+from metarace import auplay
 from metarace import jsonconfig
 
 # Global Defaults
@@ -64,8 +60,28 @@ LOGFILE=u'irtt_start.log'
 TIMEFONT='Nimbus Sans L Condensed Bold '
 COUNTERFONT='Nimbus Sans L Condensed Bold '
 NOHDR = [u'Start', u'start', u'time', u'Time', u'']
-DEFAUDIOSYNCTHRESH = 100000000L	# Tolerate no more than 100ms audio desync
+# TYPICAL SYNC ERROR: 17619048	# typical playback error ~20ms
+DEFAUDIOSYNCTHRESH = 100000000	# Tolerate no more than 100ms audio desync
 				# note: on Pi/ARM, 200ms reqd due to proc lag
+                                # this is now configurable
+				# TODO: not currently used
+
+# squeeze text into a desired width
+def fit_cent(cr, pr, oh, ow, msg, ch, fn):
+    """squeeze text centered at h."""
+    if msg is not None:
+        cr.save()
+        l = pr.create_layout()
+        l.set_font_description(pango.FontDescription(fn + str(ch)))
+        l.set_text(msg)
+        (tw,th) = l.get_pixel_size()
+        if tw > ow:
+            cr.scale(float(ow)/float(tw),1.0)
+            tw = ow
+        cr.move_to(0.5*(ow-tw), oh)
+        pr.update_layout(l)
+        pr.show_layout(l)
+        cr.restore()
 
 # draw text centred
 def text_cent(cr, pr, oh, ow, msg, ch, fn):
@@ -102,14 +118,17 @@ class irtt_start(object):
         if not self.started:
             self.timer.start()
             self.scb.start()
+            self.ap.start()
             self.started = True
 
     def shutdown(self):
         """Cleanly shutdown."""
+        self.ap.exit()
         self.scb.exit()
         self.timer.exit()
         self.scb.join()
         self.timer.join()
+        self.ap.join()
         self.started = False
 
     def window_destroy_cb(self, window):
@@ -168,8 +187,10 @@ class irtt_start(object):
         # rider name txt
         if self.riderstr is not None:
             oh = 0.80 * self.height
-            ch = 0.11 * self.height
-            text_cent(cr, pr, oh, 0.5 * self.width,
+            ch = 0.10 * self.height
+            #text_cent(cr, pr, oh, 0.5 * self.width,
+                      #self.riderstr, ch, COUNTERFONT)
+            fit_cent(cr, pr, oh, self.width,
                       self.riderstr, ch, COUNTERFONT)
 
         # starter bulbs
@@ -204,7 +225,6 @@ class irtt_start(object):
 
     def clear(self):
         """Clear all elements and dearm timer."""
-        self.tod = tod.tod('now')
         self.countdown = None
         self.riderstr = None
         self.bulb = None
@@ -217,6 +237,10 @@ class irtt_start(object):
                                     u'channel':USCBSRV_CHANNEL, 
                                     u'fullscreen':False,
                                     u'timer':'',
+                                    u'backlightlow':0.25,
+                                    u'backlighthigh':1.0,
+                                    u'backlightdev':None,
+                                    u'syncthresh':DEFAUDIOSYNCTHRESH,
                                     u'startlist':u'startlist.csv'}})
         cr.add_section(u'irtt_start')
         # check for config file
@@ -225,7 +249,7 @@ class irtt_start(object):
         cr.merge(metarace.sysconf, u'irtt_start')
 
         try:
-            self.log.debug(u'Reading config from: ' +repr(cfile))
+            self.log.info(u'Reading config from: ' +repr(cfile))
             with open(cfile, 'rb') as f:
                 cr.read(f)
         except Exception as e:
@@ -243,6 +267,40 @@ class irtt_start(object):
         self.timer.armlock(True)	# always re-armlock
         self.timer.arm(timy.CHAN_START)
 
+        # load backlight parameters
+        self.backlightdev = cr.get(u'irtt_start', u'backlightdev')
+        self.backlightlow = strops.confopt_float(cr.get(u'irtt_start',
+                                                 u'backlightlow'),0.25)
+        self.backlighthigh = strops.confopt_float(cr.get(u'irtt_start',
+                                                 u'backlighthigh'),1.0)
+        if os.path.exists(self.backlightdev):
+            try:
+                with open(os.path.join(self.backlightdev,u'max_brightness'),
+                                       'rb') as bf:
+                    mbstr = bf.read()
+                    self.backlightmax = strops.confopt_posint(mbstr,20)
+                    self.backlightdev = os.path.join(self.backlightdev,
+                                                     u'brightness')
+                    self.log.info(u'Using backlight dev '
+                                    + repr(self.backlightdev) 
+                                    + u'; Max={0}, Low={1}%, High={2}%'.format(
+                                        self.backlightmax,
+                                        int(100.0*self.backlightlow),
+                                        int(100.0*self.backlighthigh)))
+            except Exception as e:
+                self.log.error(u'Reading from backlight device: ' + repr(e))
+                self.backlightdev = None
+        else:
+            self.log.info(u'Backlight control not configured.')
+            self.backlightdev = None
+
+        # audio sync thresh
+        self.syncthresh = strops.confopt_posint(
+                                   cr.get(u'irtt_start', u'syncthresh'),
+                                   DEFAUDIOSYNCTHRESH)
+        self.log.info(u'Sync threshold set to: {0:0.3f}s'.format(
+                                            float(self.syncthresh)*1e-9))
+              
         # set sender port
         nhost = cr.get(u'irtt_start', u'uscbsrv')
         nchannel = cr.get(u'irtt_start', u'channel')
@@ -317,7 +375,7 @@ class irtt_start(object):
 
     def remote_msg(self, msg):
         """Log a message to the uscbsrv."""
-        self.log.debug(msg)
+        #self.log.debug(msg)
         self.scb.add_rider([msg], u'message')
 
     def starttrig(self, e):
@@ -330,11 +388,11 @@ class irtt_start(object):
             cr = self.ridermap[self.currider]
             self.scb.add_rider([cr[1], cr[2], e.rawtime(),
                                 cr[0].rawtime(), ls.rawtime()], u'starter')
-            self.log.debug(u'Starter: ' + u','.join([cr[1], e.rawtime()]))
+            self.log.info(u'Starter: ' + u','.join([cr[1], e.rawtime()]))
             self.armed = False
         else:
             # emit an anon tod
-            self.log.debug(u'Impulse: ' + e.rawtime())
+            self.log.info(u'Impulse: ' + e.rawtime())
             self.scb.add_rider([u'Impulse: ' + unicode(e)], u'message')
         return False
 
@@ -349,18 +407,20 @@ class irtt_start(object):
 
     def check_play_pos(self, expect):
         """Check and handle audio desync error."""
-        try:
-            curpos,postype = self.player.query_position(gst.FORMAT_TIME)
-            err = abs(expect - curpos)
-            if err > AUDIOSYNCTHRESH:
-                self.log.error(u'Audio desync detected: ' + repr(err) + 
-                                u'@' + repr(curpos))
-                self.player.set_state(gst.STATE_NULL)
-            else:
-                self.log.debug(u'Audio in sync, error : ' + repr(err))
-        except:
-            self.log.debug(u'Error reading audio position.')
         return False
+	## TODO: confirm play pos using stream 
+        #try:
+            #curpos,postype = self.player.query_position(gst.FORMAT_TIME, None)
+            #err = abs(expect - curpos)
+            #if err > self.syncthresh:
+                #self.log.error(u'Audio desync detected: ' + repr(err) + 
+                                #u'@' + repr(curpos))
+                #self.player.set_state(gst.STATE_NULL)
+            #else:
+                #self.log.debug(u'Audio in sync, error=' + repr(err))
+        #except Exception as e:
+            #self.log.error(u'Reading audio position: ' + repr(e))
+        #return False
 
     def timeout(self, data=None):
         """Handle timeout."""
@@ -397,69 +457,90 @@ class irtt_start(object):
         # 4: Return False
         return False	# must return False
 
+    def set_backlight(self, percent=None):
+        """Attempt to adjust screen brightness between riders."""
+        if self.backlightdev and abs(percent - self.backlight) > 0.05:
+            if percent < 0.0:
+                percent = 0.0
+            elif percent > 1.0:
+                percent = 1.0
+            nb = int(0.5 + percent * self.backlightmax)
+            try:
+                with open(self.backlightdev, 'wb') as f:
+                    f.write(str(nb))
+            except Exception as e:
+                self.log.error(u'Writing to backlight control: ' + repr(e))
+            self.backlight = percent
+
     def process_timeout(self):
         """Process countdown, redraw display if required."""
         curoft = tod2key(self.tod)
         if self.currider is not None:
             cdn = self.currider - curoft
-            if cdn == 50 or cdn == 24:
-                self.riderstr = self.ridermap[self.currider][3]
-                self.bulb = 'red'
-                self.log.debug(u'LOAD: ' + self.riderlogstr(self.currider))
-            if cdn == 15:
-                #self.player.set_state(gst.STATE_READY)
-                self.player.set_state(gst.STATE_PAUSED)
-                self.riderstr = self.ridermap[self.currider][3]
-                self.bulb = 'red'
-                self.remote_msg(u'15SEC: ' + self.riderlogstr(self.currider))
             if cdn == 10:
-                self.player.set_state(gst.STATE_PLAYING)
+                #self.player.set_state(gst.STATE_PLAYING)
+                self.ap.play()
                 self.riderstr = self.ridermap[self.currider][3]
                 self.bulb = 'red'
                 self.log.debug(u'Player started for: '
                                  + self.riderlogstr(self.currider))
-            if cdn == 8:	# check audio stream sync
-                self.check_play_pos(2000000000L)
-            if cdn == 7:	# check audio stream sync
-                self.check_play_pos(3000000000L)
-            if cdn == 6:	# check audio stream sync
-                self.check_play_pos(4000000000L)
-            if cdn == 5:
+            elif cdn in [8, 7, 6]:	# check audio stream sync
+                self.check_play_pos(int((10-cdn)*1e9))
+            elif cdn == 15:
+                self.ap.stop()
+                #self.player.set_state(gst.STATE_PAUSED)
+                self.riderstr = self.ridermap[self.currider][3]
+                self.bulb = 'red'
+                self.remote_msg(u'15SEC: ' + self.riderlogstr(self.currider))
+                self.set_backlight(self.backlighthigh)
+            elif cdn == 50 or cdn == 24:
+                self.riderstr = self.ridermap[self.currider][3]
+                self.bulb = 'red'
+                self.log.info(u'LOAD: ' + self.riderlogstr(self.currider))
+            elif cdn == 30:
+                self.set_backlight(self.backlighthigh)
+            elif cdn == 5:
                 self.timer.armlock(True)	# always re-armlock
                 self.timer.arm(timy.CHAN_START)
                 self.armed = True	# self arm is not same as timer arm
                 self.remote_msg(u'ARM: ' + self.riderlogstr(self.currider))
-            if cdn == 0:
+            elif cdn == 0:
                 self.bulb = 'green'
                 self.countdown = 0
-                self.log.debug(u'GO: ' + self.riderlogstr(self.currider))
-            if cdn == -4:	# load sets minimum gap-> ~25sec
+                self.log.info(u'GO: ' + self.riderlogstr(self.currider))
+            elif cdn == -4:	# load sets minimum gap-> ~25sec
                 self.remote_msg(u'CLEAR: ' + self.riderlogstr(self.currider))
-                self.clear()	# also does dearm
+                self.set_backlight(self.backlightlow)
+                self.clear() # note also removes the bulb
                 self.currider = self.ridermap[self.currider][4]
+
             if cdn >= 0 and cdn <= 30:
-                if self.bulb == 'red':
+                if self.bulb:
                     self.countdown = cdn
+                else:
+                    self.countdown = None
             else:
                 if cdn < 0 and cdn > -5:
                     self.countdown = cdn
                 else:
                     self.countdown = None
         else:
+            self.clear()	# also does dearm
             self.riderstr = u'[finished]'
             # no more riders or error in init.
         self.draw_and_update()
 
-    def gst_message(self, bus, message):
-        t = message.type
-        if t == gst.MESSAGE_EOS:
-            self.player.set_state(gst.STATE_NULL)	# close player
-        elif t == gst.MESSAGE_ERROR:
-            self.player.set_state(gst.STATE_NULL)
-            err, debug = message.parse_error()
-            self.log.error(u"gst error: %s" % err, debug)
-        else:
-            self.log.debug(u"gst message: " + repr(message))
+    #def gst_message(self, bus, message):
+        #t = message.type
+        #if t == gst.MESSAGE_EOS:
+            #self.player.set_state(gst.STATE_NULL)	# close player
+        #elif t == gst.MESSAGE_ERROR:
+            #self.player.set_state(gst.STATE_NULL)
+            #err, debug = message.parse_error()
+            #self.log.error(u"gst error: %s" % err, debug)
+        #else:
+            #pass
+            ##self.log.debug(u"gst message: " + repr(message))
 
     def __init__(self):
         # logger and handler
@@ -470,7 +551,7 @@ class irtt_start(object):
         self.loghandler.setFormatter(logging.Formatter(
                        '%(asctime)s %(levelname)s:%(name)s: %(message)s'))
         self.log.addHandler(self.loghandler)
-        self.log.debug(u'IRTT Starter - Init.')
+        self.log.info(u'IRTT Starter - Init.')
 
         # require one timy and one uscbsrv
         self.timer = timy.timy()
@@ -480,19 +561,18 @@ class irtt_start(object):
         self.running = True
 
         # Audio output
-        self.player = gst.element_factory_make("playbin2", "player")
-        self.player.set_property("video-sink",
-                         gst.element_factory_make("fakesink", "fakesink"))
-        bus = self.player.get_bus()
-        bus.add_signal_watch()
-        bus.connect("message", self.gst_message)
-        self.player.set_property('uri', u'file://'
-                             + os.path.join(metarace.DB_PATH, u'start.wav'))
+        self.ap = auplay.auplay(os.path.join(metarace.DB_PATH, u'start.wav'))
 
         # variables
         self.armed = False
         self.width = 0
         self.height = 0
+        self.backlight = 0.0
+        self.backlightmax = 20
+        self.backlightdev = None
+        self.backlightlow = 0.25
+        self.backlighthigh = 1.0
+        self.syncthresh = 100000000
         self.tod = tod.tod(u'now').truncate(0)
         self.nc = self.tod + tod.tod(u'1.22') # set interval a little off mark
         self.countdown = None
@@ -510,7 +590,7 @@ class irtt_start(object):
         self.area.set_size_request(400,220)
         self.area.show()
         self.window.add(self.area)
-        self.log.debug(u'Starting clock intervals at: ' + self.nc.rawtime(3))
+        self.log.info(u'Starting clock intervals at: ' + self.nc.rawtime(3))
         glib.timeout_add(2000, self.timeout)
         glib.timeout_add_seconds(5, self.delayed_cursor)
 
