@@ -42,6 +42,7 @@ from metarace import timy
 from metarace import scbwin
 from metarace import tod
 from metarace import uiutil
+from metarace import loghandler
 from metarace import eventdb
 from metarace import riderdb
 from metarace import strops
@@ -114,6 +115,8 @@ class ittt(object):
                     if self.teampursuit:	# HACK
                         self.chan_A = timy.CHAN_PA
                     self.armlap(self.fs, self.chan_A)
+                    if self.timetype == u'single':
+                        self.meet.timer.arm(self.chan_B)
                     return True
                 elif key == key_armlap_B:
                     if self.teampursuit:	# HACK
@@ -250,8 +253,9 @@ class ittt(object):
                                         'fsstat':'idle',
                                         'bsbib':'',
                                         'bsstat':'idle',
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'showcats':'No',
+                                        'comment':'',
                                         'distance':defdistance,
 					'distunits':defdistunits,
                                         'chan_S':defchans,
@@ -263,6 +267,7 @@ class ittt(object):
                                         'timetype':deftimetype})
         cr.add_section('event')
         cr.add_section('riders')
+        cr.add_section('traces')
         if os.path.isfile(self.configpath):
             self.log.debug('Attempting to read config from '
                                + repr(self.configpath))
@@ -271,6 +276,9 @@ class ittt(object):
         self.autospec = cr.get('event', 'autospec')
         self.set_timetype(cr.get('event', 'timetype'))
         self.distance = strops.confopt_dist(cr.get('event', 'distance'))
+        self.comment = cr.get('event', 'comment')
+        if not self.comment or self.comment == u'None':
+            self.comment = None
         self.units = strops.confopt_distunits(cr.get('event', 'distunits'))
         if self.event[u'laps']:
             self.units = 'laps'
@@ -319,10 +327,15 @@ class ittt(object):
                     nr[i] = self.meet.rdb.getvalue(dbr, i)
             nri = self.riders.append(nr)
             self.settimes(nri, st, ft, sp, doplaces=False, comment=co)
-        if self.series and self.series[0].lower() == u't':
+        if self.series and u't' in self.series:
             self.teamnames = True
         self.placexfer()
 
+        # re-load any traces
+        for r in cr.get('event', 'startlist').split():
+            if cr.has_option('traces',r):
+                self.traces[r] = cr.get('traces', r).split('\n')
+
         # re-join any existing timer state
         curstart = tod.str2tod(cr.get('event', 'start'))
         lstart = tod.str2tod(cr.get('event', 'lstart'))
@@ -375,6 +388,7 @@ class ittt(object):
         cw.set('event', 'timetype', self.timetype)
         cw.set('event', 'distance', self.distance)
         cw.set('event', 'distunits', self.units)
+        cw.set('event', 'comment', self.comment)
         cw.set('event', 'chan_S', self.chan_S)
         cw.set('event', 'chan_A', self.chan_A)
         cw.set('event', 'chan_B', self.chan_B)
@@ -393,6 +407,11 @@ class ittt(object):
         cw.set('event', 'fsbib', self.fs.getrider())
         cw.set('event', 'bsstat', self.bs.getstatus())
         cw.set('event', 'bsbib', self.bs.getrider())
+
+        cw.add_section('traces')
+        for rider in self.traces:
+            cw.set('traces', rider, u'\n'.join(self.traces[rider]))
+
         cw.add_section('riders')
 
         # save out all starters
@@ -422,6 +441,8 @@ class ittt(object):
         sec.showheats = True
         if self.timetype == 'single':
             sec.set_single()
+            if u't' in self.series and self.series != u'tmsl':
+                sec.showheats = True
 
         headvec = [u'Event', self.evno, u':', self.event[u'pref'],
                          self.event[u'info']]
@@ -489,22 +510,44 @@ class ittt(object):
                 self.addrider(unicode(miss))	# WARNING!
                 miss += 1
         blanknames = False	# HACK!!
+        teams = False
+        if u't' in self.series:	# Team no hack
+            teams = True
         if placeholders > 0:
             blanknames = True
         if self.timetype == 'single':
             for r in self.riders:
                 rno = r[COL_BIB].decode('utf-8')
-                if u't' in self.series:	# Team no hack
-                    rno = u' '	# force name
+                #if teams:	# Team no hack
+                    #rno = u' '	# force name
                 info = None
                 if cats and rno in cats:
                     info = cats[rno]
                 rh = self.meet.newgetrider(rno, self.series)
                 rname = u''
+                heat = unicode(count) + u'.1'
                 if rh is not None:
-                     rname = rh[u'namestr']
-                     if rh[u'ucicode']: info = rh[u'ucicode']
-                hlist.append([unicode(count)+'.1', rno, rname, info])
+                     if not teams:
+                         rname = rh[u'namestr']
+                     else:
+                         rname = rh[u'first']
+                         if teams:
+                           info = []
+                           col = u'black'
+                           for trno in strops.reformat_riderlist(rh[u'note']).split():
+                             trh = self.meet.newgetrider(trno) #!! SERIES?
+                             if trh is not None:
+                                 if self.series == u'tmsl':
+                                     trno = col
+                                     col = u'red'
+                                 info.append([trno, trh[u'namestr'],None])
+                     #if rh[u'ucicode']: info = rh[u'ucicode']
+                     # consider partners here
+                     if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                        ph = self.meet.newgetrider(rh[u'note'], self.series)
+                        if ph is not None:
+                            info = [[u' ',ph[u'namestr'] + u' - Pilot', ph[u'ucicode']]]
+                hlist.append([heat, rno, rname, info])
                                  # all heats are one up
                 count -= 1
         else:
@@ -521,8 +564,22 @@ class ittt(object):
                 if cats and rno in cats:
                     info = cats[rno]
                 if rh is not None:
-                    rname = rh[u'namestr']
-                    if rh[u'ucicode']: info = rh[u'ucicode']
+                    if not teams:
+                        rname = rh[u'namestr']
+                    else:
+                        rname = rh[u'first']
+                        if teams:
+                           info = []
+                           for trno in strops.reformat_riderlist(rh[u'note']).split():
+                             trh = self.meet.newgetrider(trno) #!! SERIES?
+                             if trh is not None:
+                                 info.append([trno, trh[u'namestr'],None])
+                    #if rh[u'ucicode']: info = rh[u'ucicode']
+                    # consider partners here
+                    if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                        ph = self.meet.newgetrider(rh[u'note'], self.series)
+                        if ph is not None:
+                            info = [[u' ',ph[u'namestr'] + u' - Pilot', ph[u'ucicode']]]
                 hlist.append([heat, rno, rname, info])
                 lane += 1
                 if lane > 2:
@@ -739,8 +796,15 @@ class ittt(object):
             slist = strops.reformat_riderlist(
                           b.get_object('race_starters_entry').get_text(),
                                         self.meet.rdb, self.series).split()
-            for s in slist:
-                self.addrider(s)
+            # if no starters yet - automatically seed by order entered
+            if len(self.riders) == 0:
+                cnt = 1
+                for s in slist:
+                    self.addrider(s, cnt)
+                    cnt += 1
+            else:
+                for s in slist:
+                    self.addrider(s)
 
             self.log.debug('Edit race properties done.')
             glib.idle_add(self.delayed_announce)
@@ -759,7 +823,7 @@ class ittt(object):
             rank = None
             time = None
             info = None
-            if r[COL_COMMENT] in ['caught', 'rel']:
+            if r[COL_COMMENT] in ['caught', 'rel', 'w/o']:
                 info = r[COL_COMMENT]
             if self.onestart:
                 if r[COL_PLACE] != '':
@@ -790,9 +854,9 @@ class ittt(object):
             rh = self.meet.newgetrider(rno, self.series)
             rank = None
             rname = u''
+            plink = None
             if not self.teamnames:
-                if rh is not None:
-                    rname = rh[u'namestr']
+                rname = rh[u'namestr']
             else:	# Team no hack
                 rno = u' '	# force name
                 if rh is not None:
@@ -824,16 +888,34 @@ class ittt(object):
                     if r[COL_START] != tod.tod('0'):	# auto time
                         rtime = time.rawtime(3)
                     else:
-                        rtime = time.rawtime(2) + ' '
+                        rtime = time.rawtime(2) + u'\u2007'
                 elif r[COL_COMMENT]:
                     rtime = str(r[COL_COMMENT])
 
-            sec.lines.append([rank, rno, rname, rcat, rtime, dtime])
+            sec.lines.append([rank, rno, rname, rcat, rtime, dtime,None])
+            # then add team members if relevant
+            if u't' in self.series:
+                for trno in strops.reformat_riderlist(rh[u'note']).split():
+                    trh = self.meet.newgetrider(trno) #!! SERIES?
+                    if trh is not None:
+                        trname = trh[u'namestr']
+                        trinf = trh[u'ucicode']
+                        sec.lines.append([None, trno, trname, trinf,
+                                                None, None, None])
+            # or add tandem partner
+            if rh is not None:
+                # consider partners here
+                if rh[u'cat'] and u'tandem' in rh[u'cat'].lower():
+                    ph = self.meet.newgetrider(rh[u'note'], self.series)
+                    if ph is not None:
+                        prname = ph[u'namestr'] + u' - Pilot'
+                        prinf = ph[u'ucicode']
+                        sec.lines.append([None, u'', prname, prinf,
+                                                None, None, None])
         
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
-
         ## Should be a method 
         if self.onestart:
             sec.subheading = substr
@@ -846,6 +928,14 @@ class ittt(object):
                 sec.subheading = substr
 
         ret.append(sec)
+
+        # comment
+        if self.comment is not None:
+            sec = printing.bullet_text()
+            sec.heading = u'Decisions of the commissaires panel'
+            sec.lines.append([None, self.comment])
+            ret.append(sec)
+
         return ret
 
     def editent_cb(self, entry, col):
@@ -905,6 +995,18 @@ class ittt(object):
     def fin_trig(self, sp, t):
         """Register finish trigger."""
         sp.finish(t)
+        if self.difftime:
+            if self.diffstart is None or self.difflane is sp:
+                self.diffstart = t
+                self.difflane = sp
+            else:
+                so = self.t_other(sp)
+                if so.lap == sp.lap and self.diffstart is not None:
+                    dt = t - self.diffstart
+                    if dt < 4:
+                        sp.difftime(dt)
+                    self.difflane = None
+                    self.diffstart = None
         ri = self.getiter(sp.getrider())
         if ri is not None:
             self.settimes(ri, self.curstart, t, sp.splits)
@@ -957,7 +1059,7 @@ class ittt(object):
             if chan == self.chan_S:
                 self.torunning(e)
         elif self.timerstat == 'running':
-            if chan == self.chan_A:
+            if chan == self.chan_A or (self.timetype == u'single' and self.chan_B):
                 stat = self.fs.getstatus()
                 if stat == 'armint':
                     self.lap_trig(self.fs, e)
@@ -1342,18 +1444,27 @@ class ittt(object):
             self.lstart = None
             self.meet.timer.arm(self.chan_S)
             self.showtimerwin()
-            self.meet.printimp(True)
             if not self.onestart:
                 self.meet.timer.printline(self.meet.racenamecat(self.event))
             self.meet.delayimp('0.01')
             if self.fs.status == 'armstart':
+                bib = self.fs.getrider()
+                if bib not in self.traces:
+                    self.traces[bib] = []
+                self.fslog = loghandler.traceHandler(self.traces[bib])
+                logging.getLogger().addHandler(self.fslog)
                 self.meet.scbwin.sett1('       0.0     ')
                 nstr = self.fs.biblbl.get_text()
-                self.meet.timer_log_msg(self.fs.getrider(), nstr)
+                self.meet.timer_log_msg(bib, nstr)
             if self.bs.status == 'armstart':
+                bib = self.bs.getrider()
+                if bib not in self.traces:
+                    self.traces[bib] = []
+                self.bslog = loghandler.traceHandler(self.traces[bib])
+                logging.getLogger().addHandler(self.bslog)
                 self.meet.scbwin.sett2('       0.0     ')
                 nstr = self.bs.biblbl.get_text()
-                self.meet.timer_log_msg(self.bs.getrider(), nstr)
+                self.meet.timer_log_msg(bib, nstr)
             if self.timetype == 'single':
                 self.bs.toidle()
                 self.bs.disable()
@@ -1361,12 +1472,17 @@ class ittt(object):
 
     def toidle(self, idletimers=True):
         """Set timer to idle state."""
+        if self.fslog is not None:
+            logging.getLogger().removeHandler(self.fslog)
+            self.fslog = None
+        if self.bslog is not None:
+            logging.getLogger().removeHandler(self.bslog)
+            self.bslog = None
         if idletimers:
             self.fs.toidle()
             self.bs.toidle()
         self.timerstat = 'idle'
-        self.meet.printimp(False)
-        self.meet.delayimp('1.00')
+        self.meet.delayimp('2.00')
         self.curstart = None
         self.lstart = None
         self.diffstart = None
@@ -1457,6 +1573,15 @@ class ittt(object):
             #self.log_clear(self.riders.get_value(sel[1], COL_BIB))
             glib.idle_add(self.delayed_announce)
 
+    def tod_context_print_activate_cb(self, menuitem, data=None):
+        """Print Rider trace"""
+        sel = self.view.get_selection().get_selected()
+        if sel is not None:
+            bib = self.riders.get_value(sel[1], COL_BIB)
+            if bib in self.traces:
+                self.meet.timer_reprint(self.meet.racenamecat(self.event),
+                                        self.traces[bib])
+
     def now_button_clicked_cb(self, button, entry=None):
         """Set specified entry to the 'now' time."""
         if entry is not None:
@@ -1574,6 +1699,7 @@ class ittt(object):
         self.distance = None
         self.units = 'laps'
         self.autoarm = False
+        self.comment = None
         self.difftime = False
         self.teampursuit = False
         self.teamnames = False		# team names only shown
@@ -1582,6 +1708,9 @@ class ittt(object):
         self.chan_S = timy.CHAN_START
         self.fsvec = None
         self.bsvec = None
+        self.fslog = None
+        self.bslog = None
+        self.traces = {}
 
         # race run time attributes
         self.autospec = ''
@@ -1632,10 +1761,12 @@ class ittt(object):
         # Timer Panes
         mf = b.get_object('race_timer_pane')
         self.fs = timerpane.timerpane('Front Straight')
-        #self.fs.uport = self.meet.udptimer
+        self.fs.uport = self.meet.udptimer
+        self.fs.uaddr = self.meet.udpaddr
         self.fs.bibent.connect('activate', self.bibent_cb, self.fs)
         self.bs = timerpane.timerpane('Back Straight')
-        #self.bs.uport = self.meet.udptimer
+        self.bs.uport = self.meet.udptimer
+        self.bs.uaddr = self.meet.udpaddr
         self.bs.urow = 6	# scb row for timer messages
         self.bs.bibent.connect('activate', self.bibent_cb, self.bs)
         mf.pack_start(self.fs.frame)
