@@ -73,7 +73,7 @@ class omnium(object):
         self.riders.clear()
         cr = ConfigParser.ConfigParser({'startlist':'',
                                         'id':EVENT_ID,
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'events':'',
                                         'autospec':'',
                                         'evnicks':''})
@@ -223,7 +223,10 @@ class omnium(object):
                         self.log.error('Invalid Omnium config: aborting.')
                         return
                     r.loadconfig()
-                    ret.extend(r.result_report())
+                    if r.onestart:
+                        ret.extend(r.result_report())
+                    else:
+                        ret.extend(r.startlist_report())
                     r.destroy()
         return ret
 
@@ -289,6 +292,13 @@ class omnium(object):
                 else:
                     ret = u'Standings'               
             else:
+                ev = self.meet.edb[self.curevent]
+                if ev is not None:
+                    if ev[u'type'] == u'points':
+                        if self.curevent in self.eventstatusmap:
+                            if self.eventstatusmap[self.curevent] =='running':
+                                # force virtual for race in progress
+                                self.resultvirtual = True
                 # TODO check for completion
                 if self.resultvirtual:
                     ret = u'Virtual Standings'
@@ -300,10 +310,11 @@ class omnium(object):
         """Sort results according to omnium rules."""
 
         # Comparison vecs: [idx, bib, rcnt, dnf, r[COL_TOTAL], r[COL_TIME]
-        if x[2] == y[2]:	# Same number of results so far
-            if x[3] == y[3]:	# Both dnf or not
+        if x[3] == y[3]:	# Both dnf or not
+            if x[2] == y[2]:	# Same number of results so far
                 if x[3] == 'dnf':
-                    return cmp(x[1], y[1])		# revert to bib
+                    return cmp(strops.riderno_key(x[1]),
+                               strops.riderno_key(y[1]))# revert to bib
                 else:
                     if x[4] == y[4]:			# same pts
                         if x[5] == y[5]:		# same aggregate time
@@ -314,9 +325,9 @@ class omnium(object):
                     else:
                         return cmp(x[4], y[4])
             else:
-                return cmp(x[3], y[3])		# In then DNF
+                return cmp(y[2], x[2]) # Sort descending on rcount
         else:
-            return cmp(y[2], x[2]) # Sort descending on rcount
+                return cmp(x[3], y[3])		# In then DNF
 
     def recalculate(self):
         """Update internal model."""
@@ -342,6 +353,12 @@ class omnium(object):
                     return
                 r.loadconfig()
                 self.eventstatusmap[eno] = r.timerstat
+                if r.evtype == u'points': # Override for point score
+                    sstr = r.standingstr().lower()
+                    if sstr == u'result':	# only finished reliably
+                        self.eventstatusmap[eno] = u'finished'
+                    elif u'provisional' in sstr or u'standing' in sstr:
+                        self.eventstatusmap[eno] = u'running'
                 lastrank = 0	# used to track dnfs
                 for res in r.result_gen():
                     bib = res[0]
@@ -416,7 +433,8 @@ class omnium(object):
             if r[COL_PLACE] != '':	# dnf
                 dnf = True
             if not dnf:
-                if rescheck is not None and rcnt != rescheck:
+                if rescheck is not None and rcnt != rescheck and rcnt != 0:
+                    self.log.info(u'Force Virtual: ' + repr(rcnt) + u' != ' + repr(rescheck))
                     self.resultvirtual = True
                 rescheck = rcnt
             arow = [idx, bib, rcnt, dnf, r[COL_TOTAL], r[COL_TIME]]
