@@ -80,6 +80,7 @@ key_results = 'F4'
 key_armstart = 'F5'
 key_showtimer = 'F6'
 key_armfinish = 'F9'
+key_lapdown = 'F11'
 
 # extended function key mappings
 key_abort = 'F5'
@@ -103,17 +104,21 @@ class ps(object):
 					'start':'',
                                         'lstart':'',
                                         'finish':'',
+                                        'comments':'',
                                         'sprintlaps':'',
                                         'distance':'',
+                                        'runlap':'',
                                         'distunits':'laps',
                                         'masterslaps':defmasterslaps,
                                         'inomnium':'No',
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'autospec':'',
                                         'scoring':defscoretype})
         cr.add_section('event')
         cr.add_section('sprintplaces')
         cr.add_section('sprintpoints')
+        cr.add_section('sprintsource')
+        cr.add_section('laplabels')
         cr.add_section('points')
 
         if os.path.isfile(self.configpath):
@@ -154,12 +159,24 @@ class ps(object):
             self.scoring = 'points'
         self.type_lbl.set_text(self.scoring.capitalize())
 
+        # race infos
+        self.comments = []
+        nc = cr.get('event', 'comments')
+        if nc:
+            self.comments.append(nc)
+
         self.autospec = cr.get('event', 'autospec')
         self.distance = strops.confopt_dist(cr.get('event', 'distance'))
         self.units = strops.confopt_distunits(cr.get('event', 'distunits'))
+        self.runlap = strops.confopt_posint(cr.get('event','runlap'))
         self.masterslaps = strops.confopt_bool(cr.get('event', 'masterslaps'))
-        self.reset_lappoints()
+        # override laps from event listings
+        if not self.onestart and self.event[u'laps']:
+            self.units = 'laps'
+            self.distance = strops.confopt_posint(self.event[u'laps'],
+                                                  self.distance)
 
+        self.reset_lappoints()
         self.sprintlaps = strops.reformat_biblist(
                             cr.get('event', 'sprintlaps'))
 
@@ -167,6 +184,14 @@ class ps(object):
         for (sid, spstr) in cr.items('sprintpoints'):
             self.sprintpoints[sid] = spstr	# validation in sprint model
 
+        # load lap labels
+        for (sid, spstr) in cr.items('laplabels'):
+            self.laplabels[sid] = spstr	# just plain text
+
+        # load any autospec'd sprint results
+        for (sid, spstr) in cr.items('sprintsource'):
+            self.sprintsource[sid] = spstr	# just plain text
+
         self.sprint_model_init()
 
         oft = 0
@@ -189,6 +214,19 @@ class ps(object):
                 oft = len(self.sprints) - 1 
             self.ctrl_place_combo.set_active(oft)
             self.onestart = True
+
+        ## for omnium - look up the places from event links if present
+        if self.inomnium:
+            for s in self.sprints:
+                sid = s[SPRINT_COL_ID]
+                if sid in self.sprintsource:
+###
+# what is the SID
+                    splac = self.meet.autoplace_riders(self,
+                                                    self.sprintsource[sid])
+                    self.log.debug(u'Loaded ' + sid + u' places from event ' + self.sprintsource[sid] + u' : ' + splac)
+                    if splac:
+                        s[SPRINT_COL_PLACES] = splac
         self.recalculate()
 
         self.info_expand.set_expanded(strops.confopt_bool(
@@ -239,6 +277,8 @@ class ps(object):
         cw.set('event', 'distance', self.distance)
         cw.set('event', 'distunits', self.units)
         cw.set('event', 'scoring', self.scoring)
+        if self.runlap is not None:
+            cw.set('event', 'runlap', self.runlap)
         if self.masterslaps:
             cw.set('event', 'masterslaps', 'Yes')
         else:
@@ -247,9 +287,17 @@ class ps(object):
         cw.set('event', 'inomnium', self.inomnium)
         cw.set('event', 'sprintlaps', self.sprintlaps)
 
+        thecom = u''
+        if len(self.comments) > 0:
+           thecom = self.comments[0]
+        cw.set('event', 'comments', thecom)
+
         cw.add_section('sprintplaces')
         cw.add_section('sprintpoints')
+        cw.add_section('sprintsource')
+        cw.add_section('laplabels')
         for s in self.sprints:
+            sid = s[SPRINT_COL_ID]
             cw.set('sprintplaces', s[SPRINT_COL_ID], s[SPRINT_COL_PLACES])
             if s[SPRINT_COL_200] is not None:
                 cw.set('sprintplaces', s[SPRINT_COL_ID] + '_200',
@@ -260,6 +308,10 @@ class ps(object):
             if s[SPRINT_COL_POINTS] is not None:
                 cw.set('sprintpoints', s[SPRINT_COL_ID], ' '.join(
                          map(str, s[SPRINT_COL_POINTS])))
+            if s[SPRINT_COL_ID] in self.laplabels:
+                cw.set('laplabels', sid, self.laplabels[sid])
+            if s[SPRINT_COL_ID] in self.sprintsource: 
+                cw.set(u'sprintsource', sid, self.sprintsource[sid])
 
         cw.add_section('points')
         for r in self.riders:
@@ -348,6 +400,8 @@ class ps(object):
                 plstr = r[RES_COL_PLACE]
                 if r[RES_COL_PLACE].isdigit():
                     plstr += '.'
+                elif r[RES_COL_INFO] in [u'dns',u'dsq']:
+                    plstr = r[RES_COL_INFO]
                 ptstr = ''
                 if r[RES_COL_TOTAL] != 0:
                     ptstr = str(r[RES_COL_TOTAL])
@@ -370,6 +424,15 @@ class ps(object):
                                          # dnf  or 
 					# placed in final sprint
                     sec.lines.append([plstr, rno, rname, rcat, fs, ptstr])
+                    ## TEAM HACKS
+                    if u't' in self.series and rh is not None:
+                        for trno in rh[u'note'].split():
+                            trh = self.meet.newgetrider(trno) #!! SERIES?
+                            if trh is not None:
+                                trname = trh[u'namestr']
+                                trinf = trh[u'ucicode']
+                                sec.lines.append([None, u'', trname, trinf,
+                                                        None, None, None])
                 fs = ''
 
         if self.onestart:
@@ -380,6 +443,13 @@ class ps(object):
 
         ret.append(sec)
 
+        if len(self.comments) > 0:
+            sec = printing.bullet_text()
+            sec.subheading = u'Decisions of the commisaires panel'
+            for c in self.comments:
+                sec.lines.append([None, c])
+            ret.append(sec)
+
         # output intermediate sprints?
 
         return ret
@@ -492,11 +562,18 @@ class ps(object):
             auxmap = []
             cnt = 0
             intmark = 0
-            if self.inomnium:
-                intmark = strops.mark2int(r[RES_COL_INFO].decode('utf-8','replace'))
             for r in self.riders:
+                if self.inomnium:
+                    # now os is omnium final, so sort on current rank
+                    if r[RES_COL_PLACE].isdigit() or r[RES_COL_PLACE]==u'':
+                        # but only add riders currently ranked, or unplaced
+                        intmark = strops.mark2int(r[RES_COL_PLACE].decode('utf-8','replace'))
+                    else:
+                        intmark = 999
+                    # old omnium had ps in series
+                    #intmark = strops.mark2int(r[RES_COL_INFO].decode('utf-8','replace'))
                 auxmap.append([cnt,
-                   r[RES_COL_BIB].decode('utf-8','replace'), intmark])
+                       r[RES_COL_BIB].decode('utf-8','replace'), intmark])
                 cnt += 1
             auxmap.sort(self.sort_handicap)
             self.riders.reorder([a[0] for a in auxmap])
@@ -505,6 +582,9 @@ class ps(object):
         """Return a startlist report."""
         ret = []
         sec = printing.twocol_startlist()
+        if self.evtype == 'madison':
+            # use the team singlecol method
+            sec = printing.section()
         headvec = [u'Event', self.evno, u':', self.event[u'pref'],
                          self.event[u'info']]
         if not program:
@@ -519,11 +599,10 @@ class ps(object):
         sec.lines = []
         col2 = []
         cnt = 0
-        if self.inomnium:
+        if self.inomnium and len(self.riders) > 0:
             sec.lines.append([u' ',u' ',u'The Fence', None, None, None])
             col2.append([u' ',u' ',u'Sprinters Lane', None, None, None])
         for r in self.riders:
-            cnt += 1
             rno = r[RES_COL_BIB].decode('utf-8')
             rh = self.meet.newgetrider(rno, self.series)
             inf = r[RES_COL_INFO].decode('utf-8','replace')
@@ -536,19 +615,57 @@ class ps(object):
             rname = u''
             if rh is not None:
                 rname = rh[u'namestr']
+
             if self.inomnium:
-                if cnt%2 == 1:
-                    sec.lines.append([None, rno, rname, inf, None, None])
-                else:
-                    col2.append([None, rno, rname, inf, None, None])
+                if r[RES_COL_PLACE].isdigit() or r[RES_COL_PLACE] == u'':
+                    cnt += 1
+                    if cnt%2 == 1:
+                        sec.lines.append([None, rno, rname, inf, None, None])
+                    else:
+                        col2.append([None, rno, rname, inf, None, None])
             else:
+                cnt += 1
                 sec.lines.append([None, rno, rname, inf, None, None])
+                if self.evtype == 'madison':
+                    # add the black/red entry
+                    if rh is not None:
+                        tvec = rh[u'note'].split()
+                        if len (tvec) == 2:
+                            trname = u''
+                            trinf = u''
+                            trh = self.meet.newgetrider(tvec[0])
+                            if trh is not None:
+                                trname = trh[u'namestr']
+                                trinf = trh[u'ucicode']
+                            sec.lines.append([None, u'Red', trname, trinf,
+                                                None, None, None])
+                            trname = u''
+                            trinf = u''
+                            trh = self.meet.newgetrider(tvec[1])
+                            if trh is not None:
+                                trname = trh[u'namestr']
+                                trinf = trh[u'ucicode']
+                            sec.lines.append([None, u'Black', trname, trinf,
+                                                None, None, None])
+                            #sec.lines.append([None, None, None, None,
+                                                #None, None, None])
+                    
         for i in col2:
             sec.lines.append(i)
         ret.append(sec)
+
+        # placeholders - why was this suppressed?
+        if self.event[u'plac']:
+            while cnt < self.event[u'plac']:
+                sec.lines.append([None, None, None, None, None, None])
+                cnt += 1
+
         if cnt > 0 and not program:
             sec = printing.bullet_text()
-            sec.lines.append([None, 'Total riders: ' + str(cnt)])
+            if self.evtype == 'madison': 
+                sec.lines.append([None, 'Total teams: ' + str(cnt)])
+            else:
+                sec.lines.append([None, 'Total riders: ' + str(cnt)])
             ret.append(sec)
         return ret
 
@@ -618,6 +735,10 @@ class ps(object):
                     self.do_places()
                     glib.idle_add(self.delayed_announce)
                     return True
+                elif key == key_lapdown:
+                    if self.runlap is not None and self.runlap > 0:
+                        self.runlap -= 1
+                    return True
         return False
 
     def delayed_announce(self):
@@ -761,6 +882,7 @@ class ps(object):
                 count = 0       
                 curline = 11
                 posoft = 0      
+                leaderboard = []
                 for r in self.riders:                     
                     count += 1
                     if count == 14:
@@ -788,11 +910,16 @@ class ps(object):
                           placestr, bibstr, namestr, ptstr]))
                     curline += 1
 
+                    if self.inomnium and r[RES_COL_INRACE]:
+                        leaderboard.append(r[RES_COL_BIB].rjust(2)
+                                           + psrc.rjust(3))
                 if posoft > 0:
                     self.meet.announce.postxt(10,0,'      # Rider                       Pts        # Rider                       Pts')
                 else:
                     self.meet.announce.postxt(10,0,'      # Rider                       Pts')
 
+                self.meet.announce.send_cmd(u'leaderboard',
+                         unichr(unt4.US).join(leaderboard))
         return False
 
 
@@ -844,11 +971,20 @@ class ps(object):
             fmt = [(3,u'l'),(3,u'r'),u' ',
                      (name_w,u'l'),(3,u'r')]
                         #self.meet.scb.linelen - 3) + ' pt'
+            if self.inomnium:
+                fmt = [(3,u'l'),(3,u'r'),u' ',
+                     (name_w-1,u'l'),(4,u'r')]
+                        #self.meet.scb.linelen - 3) + ' pt'
+            #ldr = None
             for r in self.riders:
                 if r[RES_COL_INRACE]:
                     plstr = r[RES_COL_PLACE] + u'.'
                     bstr = r[RES_COL_BIB]
+                    #if ldr is None and r[RES_COL_TOTAL] > 0:
+                        #ldr = r[RES_COL_TOTAL]	# current leader
                     pstr = str(r[RES_COL_TOTAL])
+                    #if self.inomnium and ldr is not None:
+                        #pstr = '-' + str(ldr - r[RES_COL_TOTAL])
                     if pstr == '0': pstr = '-'
                     resvec.append([plstr, bstr,
                          strops.fitname(r[RES_COL_FIRST], r[RES_COL_LAST],
@@ -971,24 +1107,28 @@ class ps(object):
                                    line1=self.meet.racenamecat(self.event),
                                    line2=u'',
                                    timepfx=tp)
+        wastimer = self.timerwin
         self.timerwin = True
-        self.meet.scbwin.reset()
         if self.timerstat == 'finished':
+            if not wastimer:
+                self.meet.scbwin.reset()
             elap = self.finish - self.start
             self.meet.scbwin.settime(elap.timestr(2))
             dist = self.meet.get_distance(self.distance, self.units)
             if dist:
                 self.meet.scbwin.setavg(elap.speedstr(dist))
                 ## UDP hack
-                #spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
-                #msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
-                                #xx=1,
-                                #yy=4,
-                                #text=spt)
-                #self.meet.udptimer.sendto(msg.pack(),
-                                          #(self.meet.udpaddr,6789))
+                spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
+                msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
+                                xx=1,
+                                yy=4,
+                                text=spt)
+                self.meet.udptimer.sendto(msg.pack(),
+                                          (self.meet.udpaddr,6789))
 
                 self.meet.scbwin.update()
+        else:
+            self.meet.scbwin.reset()
 
     def shutdown(self, win=None, msg='Exiting'):
         """Terminate race object."""
@@ -1000,6 +1140,9 @@ class ps(object):
     def starttrig(self, e):
         """React to start trigger."""
         if self.timerstat == 'armstart':
+            if self.distance and self.units == u'laps':
+                self.runlap = self.distance - 1
+                self.log.debug(u'SET RUNLAP: ' + repr(self.runlap))
             self.set_start(e, tod.tod('now'))
         elif self.timerstat == 'armsprintstart':
             uiutil.buttonchg(self.stat_but, uiutil.bg_armfin, 'Arm Sprint')
@@ -1030,11 +1173,27 @@ class ps(object):
 
     def rftimercb(self, e):
         """Handle rftimer event."""
-        if e.refid == '':       # got a trigger
+        if e.refid == '' or e.chan == u'STS':       # got a trigger
             #return self.starttrig(e)
             return False
+
+        # else assume this is a passing
+        r = self.meet.rdb.getrefid(e.refid)
+        if r is None:
+            self.log.info('Unknown tag: ' + e.refid + '@' + e.rawtime(2))
+            return False
+
+        bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
+        ser = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
+        if ser != self.series:
+            self.log.error(u'Ignored non-series rider: ' + repr(bib))
+            return False
+        r = self.getrider(bib)
+        if r is not None:
+            self.log.info(u'SAW ' + repr(bib))
         return False
 
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -1190,27 +1349,46 @@ class ps(object):
         if self.start is not None and self.finish is not None:
             et = self.finish - self.start
             ## UDP hack
-            #msg = unt4.unt4(
-                            #header=unichr(unt4.DC3) + u'N F$',
-                            #xx=0,
-                            #yy=0,
-                            #text=et.timestr(2)[0:12])
-            #self.meet.udptimer.sendto(msg.pack(),
-                                      #(self.meet.udpaddr,6789))
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'N F$',
+                            xx=0,
+                            yy=0,
+                            text=et.timestr(2)[0:12])
+            self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
             self.time_lbl.set_text(et.timestr(2))
         elif self.start is not None:    # Note: uses 'local start' for RT
             runtm  = (tod.tod('now') - self.lstart).timestr(1)
             ## UDP hack
-            #msg = unt4.unt4(
-                            #header=unichr(unt4.DC3) + u'R F$',
-                            #xx=0,
-                            #yy=0,
-                            #text=runtm[0:12])
-            #self.meet.udptimer.sendto(msg.pack(),
-                                      #(self.meet.udpaddr,6789))
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'R F$',
+                            xx=0,
+                            yy=0,
+                            text=runtm[0:12])
+            self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
             self.time_lbl.set_text(runtm)
+
+            if self.runlap is not None:
+                if self.runlap != self.lastrunlap:
+                    self.log.debug(u'SENT RUNLAP: ' + repr(self.runlap))
+                    lapmsg = (unicode(self.runlap) + u'    ').rjust(12)
+                    msg = unt4.unt4(header=unichr(unt4.DC3) + u'R F$',
+                                    xx=0, yy=6, text=lapmsg)
+                    self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
+                    self.lastrunlap = self.runlap
+
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).timestr(1))
+            if self.runlap and self.runlap != self.lastrunlap:
+                self.log.debug(u'SENT RUNLAP: ' + repr(self.runlap))
+                lapmsg = (unicode(self.runlap) + u'    ').rjust(12)
+                msg = unt4.unt4(header=unichr(unt4.DC3) + u'R F$',
+                                xx=0, yy=6, text=lapmsg)
+                self.meet.udptimer.sendto(msg.pack(),
+                                  (self.meet.udpaddr,6789))
+                self.lastrunlap = self.runlap
         else:
             self.time_lbl.set_text('')
 
@@ -1285,14 +1463,18 @@ class ps(object):
     def standingstr(self, width=None):
         """Return an event status string for reports and scb."""
         ret = u'Standings'
-        totsprints = len(self.sprints)
+        totsprints = 0
         lastsprint = None
+        sprintid = None
         cur = 1
         for s in self.sprints:
             self.log.debug(u'cur: ' + repr(cur) + u' val: ' + repr(s[SPRINT_COL_PLACES]))
             if s[SPRINT_COL_PLACES]:
                 lastsprint = cur
-            cur += 1
+                sprintid = s[SPRINT_COL_ID]
+            if s[SPRINT_COL_ID] not in self.laplabels:
+                totsprints += 1
+                cur += 1
         if lastsprint >= totsprints:
             ret = u'Result'
             # check for all places in final sprint?
@@ -1304,12 +1486,19 @@ class ps(object):
         else:
             ret = u'Standings'
             if lastsprint:
-                if width is not None and width < 25:
-                    ret += u' - Sprint {0}/{1}'.format(lastsprint,
+                if sprintid in self.laplabels:
+                    ret += ' After ' + self.laplabels[sprintid]
+                elif totsprints > 0:
+                    if width is not None and width < 25:
+                        ret += u' - Sprint {0}/{1}'.format(lastsprint,
                                                           totsprints)
-                else:
-                    ret += u' After Sprint {0} of {1}'.format(lastsprint,
+                    else:
+                        ret += u' After Sprint {0} of {1}'.format(lastsprint,
                                                              totsprints)
+                else:
+                    self.log.debug(u'Total sprints was 0: '
+                                    +  repr(lastsprint)
+                                    + u' / ' + repr(totsprints))
         return ret
 
     def delayed_result(self):
@@ -1433,6 +1622,8 @@ class ps(object):
             for bib in rlist.split():
                 self.delrider(bib)
             entry.set_text('')
+        elif acode == 'lap':
+            self.runlap = strops.confopt_posint(rlist)
         else:
             self.log.error('Ignoring invalid action.')
         glib.idle_add(self.delayed_announce)
@@ -1674,6 +1865,35 @@ class ps(object):
             self.ctrl_place_combo.set_active(0)
             self.ctrl_places.set_sensitive(True)
 
+    def spptsedit(self, cr, path, new_text, data=None):
+        """Sprint points edit."""
+        new_text = strops.reformat_biblist(new_text)
+        op = None
+        nextp = []
+        for nv in new_text.split():
+            if nv.isdigit():
+                nextp.append(int(nv))
+            else:
+                nextp = None
+                break
+        sid = self.sprints[path][SPRINT_COL_ID]
+        if nextp is not None and len(nextp) > 0:
+            self.sprintpoints[sid] = u' '.join(map(unicode, nextp))
+            op = nextp
+        else:
+            if sid in self.sprintpoints:
+                del self.sprintpoints[sid]
+        self.sprints[path][SPRINT_COL_POINTS] = op
+        self.recalculate()
+
+    def spptsstr(self, col, cr, model, iter, data=None):
+        """Format tod into text for listview."""
+        pv = model.get_value(iter, SPRINT_COL_POINTS)
+        if pv is not None and len(pv) > 0:
+            cr.set_property('text', u', '.join(map(unicode,pv)))
+        else:
+            cr.set_property('text', '')
+
     def todstr(self, col, cr, model, iter, data=None):
         """Format tod into text for listview."""
         st = model.get_value(iter, SPRINT_COL_200)
@@ -1686,11 +1906,13 @@ class ps(object):
     def reset_lappoints(self):
         """Update lap points allocations."""
         if self.masterslaps:
-            dist = self.meet.get_distance(self.distance, self.units)
-            if dist is not None and dist < 20000:
-                self.lappoints = 10
-            else:
-                self.lappoints = 20
+            self.lappoints = 10
+            ## OLD WAY WAS AUTO -> NOW JUST MANUALLY  
+            #dist = self.meet.get_distance(self.distance, self.units)
+            #if dist is not None and dist < 20000:
+                #self.lappoints = 10
+            #else:
+                #self.lappoints = 20
         else:
             self.lappoints = 20
 
@@ -1722,6 +1944,7 @@ class ps(object):
         self.evtype = event[u'type']
         self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
+        self.comments = []
 
         self.log = logging.getLogger('points')
         self.log.setLevel(logging.DEBUG)
@@ -1737,10 +1960,14 @@ class ps(object):
         self.sprintpoints = {}
         self.nopts = []
         self.sprintresults = []
+        self.laplabels = {}
+        self.sprintsource = {}
         self.auxmap = {}
 
         # race run time attributes
         self.onestart = False
+        self.runlap = None
+        self.lastrunlap = None
         self.readonly = not ui
         self.start = None
         self.lstart = None
@@ -1820,10 +2047,12 @@ class ps(object):
         uiutil.mkviewcoltxt(t, 'Sprint', SPRINT_COL_LABEL,
                              self.ps_sprint_cr_label_edited_cb,
                              expand=True)
-        uiutil.mkviewcoltod(t, '200m', cb=self.todstr)
+        #uiutil.mkviewcoltod(t, '200m', cb=self.todstr)
         uiutil.mkviewcoltxt(t, 'Places', SPRINT_COL_PLACES,
                              self.ps_sprint_cr_places_edited_cb,
                              expand=True)
+        uiutil.mkviewcoltod(t, 'Points', cb=self.spptsstr,
+                                         editcb=self.spptsedit)
         b.get_object('ps_sprint_win').add(t)
 
         # results pane
@@ -1852,7 +2081,9 @@ class ps(object):
                                 cb=self.ps_result_cr_laps_edited_cb)
         uiutil.mkviewcoltxt(t, 'Total', RES_COL_TOTAL, calign=1.0,
                                 width=50)
-        uiutil.mkviewcoltxt(t, 'Place', RES_COL_PLACE, calign=0.5,
+        uiutil.mkviewcoltxt(t, 'L/S', RES_COL_FINAL, calign=0.5,
+                                width=50)
+        uiutil.mkviewcoltxt(t, 'Rank', RES_COL_PLACE, calign=0.5,
                                 width=50)
         b.get_object('ps_result_win').add(t)
 
