@@ -77,6 +77,7 @@ COL_PLACE = 6
 # scb function key mappings
 key_startlist = 'F3'			# show starters in table
 key_results = 'F4'			# recalc/show result window
+key_lapdown = 'F11'			# decrement tv lap counter
 
 # timing function key mappings
 key_armstart = 'F5'			# arm for start/200m impulse
@@ -176,7 +177,10 @@ class race(object):
 
         for placegroup in placestr.split():
             for bib in placegroup.split(u'-'):
-                if bib not in placeset:
+                if bib == u'dnf':
+                    # HACK: skip one place for the 'last available' keirin
+                    place += 1
+                elif bib not in placeset:
                     placeset.add(bib)
                     r = self.getrider(bib)
                     if r is None:	# ensure rider exists at this point
@@ -229,9 +233,10 @@ class race(object):
                                         'lstart':'',
 					'comments':'',
                                         'finish':'',
+                                        'runlap':'',
 					'distance':defdistance,
 					'distunits':defdistunits,
-                                        'showinfo':'Yes',
+                                        'showinfo':'No',
                                         'inomnium':'No',
 					'startplace':'0',
                                         'autospec':'',
@@ -244,7 +249,7 @@ class race(object):
             cr.read(self.configpath)
         self.inomnium = strops.confopt_bool(cr.get('event', 'inomnium'))
         if self.inomnium:
-            self.seedsrc = 3	# fetch start list seeding from omnium
+            self.seedsrc = 1	# fetch start list seeding from omnium
         self.autospec = cr.get('event', 'autospec')
         rlist = cr.get('event', 'startlist').split()
         for r in rlist:
@@ -270,6 +275,7 @@ class race(object):
         self.set_timetype(cr.get('event', 'timetype'))
         self.distance = strops.confopt_dist(cr.get('event', 'distance'))
         self.units = strops.confopt_distunits(cr.get('event', 'distunits'))
+        self.runlap = strops.confopt_posint(cr.get('event','runlap'))
         if self.timetype != '200m' and self.event[u'laps']:
             # use event program to override
             self.units = 'laps'
@@ -329,7 +335,7 @@ class race(object):
             for r in self.riders:
                 auxmap.append([cnt, r[COL_BIB].decode('utf-8','replace'), strops.mark2int(r[COL_INFO].decode('utf-8','replace'))])
                 cnt += 1
-            if self.evtype == u'handicap':
+            if self.inomnium or self.evtype == u'handicap':
                 auxmap.sort(self.sort_handicap)
             else:
                 auxmap.sort(self.sort_riderno)
@@ -393,27 +399,49 @@ class race(object):
         if self.start is not None and self.finish is not None:
             et = self.finish - self.start
             ## UDP hack
-            #msg = unt4.unt4(
-                            #header=unichr(unt4.DC3) + u'N F$',
-                            #xx=0,
-                            #yy=0,
-                            #text=et.timestr(2)[0:12])
-            #self.meet.udptimer.sendto(msg.pack(),
-                                      #(self.meet.udpaddr,6789))
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'R F$',
+                            xx=0,
+                            yy=0,
+                            text=et.timestr(2)[0:12])
+            if not self.readonly:
+                self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
             self.time_lbl.set_text(et.timestr(2))
         elif self.start is not None:	# Note: uses 'local start' for RT
             runtm  = (tod.tod('now') - self.lstart).timestr(1)
             ## UDP hack
-            #msg = unt4.unt4(
-                            #header=unichr(unt4.DC3) + u'R F$',
-                            #xx=0,
-                            #yy=0,
-                            #text=runtm[0:12])
-            #self.meet.udptimer.sendto(msg.pack(),
-                                      #(self.meet.udpaddr,6789))
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'R F$',
+                            xx=0,
+                            yy=0,
+                            text=runtm[0:12])
+            if not self.readonly:
+                self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
             self.time_lbl.set_text(runtm)
+
+            if self.runlap is not None:
+                if self.runlap != self.lastrunlap:
+                    self.log.debug(u'SENT RUNLAP: ' + repr(self.runlap))
+                    lapmsg = (unicode(self.runlap) + u'    ').rjust(12)
+                    msg = unt4.unt4(header=unichr(unt4.DC3) + u'R F$',
+                                    xx=0, yy=6, text=lapmsg)
+                    if not self.readonly:
+                        self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
+                    self.lastrunlap = self.runlap
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).timestr(1))
+            if self.runlap and self.runlap != self.lastrunlap:
+                self.log.debug(u'SENT RUNLAP: ' + repr(self.runlap))
+                lapmsg = (unicode(self.runlap) + u'    ').rjust(12)
+                msg = unt4.unt4(header=unichr(unt4.DC3) + u'R F$',
+                                xx=0, yy=6, text=lapmsg)
+                if not self.readonly:
+                    self.meet.udptimer.sendto(msg.pack(),
+                                  (self.meet.udpaddr,6789))
+                self.lastrunlap = self.runlap
         else:
             self.time_lbl.set_text('')
 
@@ -479,9 +507,8 @@ class race(object):
         sec = None
         etype = self.event[u'type']
         twocol = True
-        #if not self.inomnium:
-        if False: #etype in [u'handicap']: # HACK - reqstartlist,u'keirin',u'sprint']:
-            sec = printing.section()
+        if not self.inomnium and not program and etype in [u'axe', u'run', u'handicap']:
+            sec = printing.section()	# one column overrides
         else:
             sec = printing.twocol_startlist()
         headvec = []
@@ -502,7 +529,7 @@ class race(object):
         sec.lines = []
         cnt = 0
         col2 = []
-        if self.inomnium:
+        if self.inomnium and len(self.riders) > 0:
             sec.lines.append([u' ',u' ',u'The Fence', None, None, None])
             col2.append([u' ',u' ',u'Sprinters Lane', None, None, None])
         for r in self.riders:
@@ -536,9 +563,14 @@ class race(object):
                 cnt += 1
         ret.append(sec)
 
+        ptype = u'Riders'
+        if etype == u'run':
+            ptype = u'Runners'
+        elif self.evtype == u'axe':
+            ptype = u'Axemen'
         if cnt > 0 and not program:
             sec = printing.bullet_text()
-            sec.lines.append([None, 'Total riders: ' + str(cnt)])
+            sec.lines.append([None, 'Total ' + ptype + ': ' + str(cnt)])
             ret.append(sec)
         return ret
 
@@ -568,6 +600,8 @@ class race(object):
         cw.set('event', 'distance', self.distance)
         cw.set('event', 'distunits', self.units)
         cw.set('event', 'timetype', self.timetype)
+        if self.runlap is not None:
+            cw.set('event', 'runlap', self.runlap)
         cw.set('event', 'autospec', self.autospec)
         cw.set('event', 'inomnium', self.inomnium)
         thecom = u''
@@ -702,11 +736,17 @@ class race(object):
             self.timerstat = 'armstart'
             uiutil.buttonchg(self.stat_but, uiutil.bg_armstart, 'Arm Start')
             self.meet.timer.arm(self.startchan)
+            if self.timetype == '200m':
+                # also accept C0 on sprint types
+                self.meet.timer.arm(timy.CHAN_START)
         elif self.timerstat == 'armstart':
             self.timerstat = 'idle'
             self.time_lbl.set_text('')
             uiutil.buttonchg(self.stat_but, uiutil.bg_none, 'Idle')
             self.meet.timer.dearm(self.startchan)
+            if self.timetype == '200m':
+                # also accept C0 on sprint types
+                self.meet.timer.dearm(timy.CHAN_START)
 
     def armfinish(self):
         """Toggle timer arm finish state."""
@@ -732,9 +772,11 @@ class race(object):
                                      line1=self.meet.racenamecat(self.event),
                                      line2=u'',
                                      timepfx=tp)
+        wastimer = self.timerwin
         self.timerwin = True
-        self.meet.scbwin.reset()
         if self.timerstat == 'finished':
+            if not wastimer:
+                self.meet.scbwin.reset()
             if self.start is not None and self.finish is not None:
                 elap = self.finish - self.start
                 self.meet.scbwin.settime(elap.timestr(2))
@@ -742,15 +784,18 @@ class race(object):
                 if dist:
                     self.meet.scbwin.setavg(elap.speedstr(dist))
                     ## UDP hack
-                    #spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
-                    #msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
-                                    #xx=1,
-                                    #yy=4,
-                                    #text=spt)
-                    #self.log.info(u'sending: ' + msg.text)
-                    #self.meet.udptimer.sendto(msg.pack(),
-                                              #(self.meet.udpaddr,6789))
+                    spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
+                    msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
+                                    xx=1,
+                                    yy=4,
+                                    text=spt)
+                    self.log.info(u'sending: ' + msg.text)
+                    if not self.readonly:
+                        self.meet.udptimer.sendto(msg.pack(),
+                                              (self.meet.udpaddr,6789))
             self.meet.scbwin.update()
+        else:
+            self.meet.scbwin.reset()
 
     def key_event(self, widget, event):
         """Race window key press handler."""
@@ -779,6 +824,10 @@ class race(object):
                     self.do_places()
                     glib.idle_add(self.delayed_announce)
                     return True
+                elif key == key_lapdown:
+                    if self.runlap is not None and self.runlap > 0:
+                        self.runlap -= 1
+                    return True
         return False
 
     def do_places(self):
@@ -801,7 +850,7 @@ class race(object):
                   (self.meet.scb.linelen-11, u'l'),(4,u'r')]
 
             ## TODO: provisional/final?
-            evtstatus = u'Result'
+            evtstatus = u'RESULT'
             self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
                                   head=self.meet.racenamecat(self.event),
                                   subhead=evtstatus,
@@ -924,6 +973,8 @@ class race(object):
             for bib in rlist.split():
                 self.delrider(bib)
             entry.set_text('')
+        elif acode == 'lap':
+            self.runlap = strops.confopt_posint(rlist)
         elif acode == 'out' and self.evtype == 'elimination':
             bib = rlist.strip()
             if self.eliminate(bib):
@@ -979,6 +1030,7 @@ class race(object):
                                          r[COL_BIB].decode('utf-8','replace'), r[COL_FIRSTNAME].decode('utf-8','replace'),
                                          r[COL_LASTNAME].decode('utf-8','replace'), r[COL_CLUB].decode('utf-8','replace')), 60)
                     self.meet.announce.postxt(21, 0, u'Out: ' + nrstr)
+                    self.log.info(u'Eliminated: ' + repr(bib))
                 else:
                     self.log.error(u'Rider in places or eliminated: ' + repr(bib))
             else:
@@ -1034,6 +1086,9 @@ class race(object):
     def starttrig(self, e):
         """React to start trigger."""
         if self.timerstat == 'armstart':
+            if self.distance and self.units == u'laps':
+                self.runlap = self.distance - 1
+                self.log.debug(u'SET RUNLAP: ' + repr(self.runlap))
             self.start = e
             self.lstart = tod.tod('now')
             self.setrunning()
@@ -1056,16 +1111,31 @@ class race(object):
 
     def rftimercb(self, e):
         """Handle rftimer event."""
-        if e.refid == '':       # got a trigger
+        if e.refid == '' or e.chan == u'STS':       # got a trigger
             #return self.starttrig(e)
             return False
+
+        # else assume this is a passing
+        r = self.meet.rdb.getrefid(e.refid)
+        if r is None:
+            self.log.info('Unknown tag: ' + e.refid + '@' + e.rawtime(2))
+            return False
+
+        bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
+        ser = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
+        if ser != self.series:
+            self.log.error(u'Ignored non-series rider: ' + repr(bib))
+            return False
+        r = self.getrider(bib)
+        if r is not None:
+            self.log.info(u'SAW ' + repr(bib))
         return False
 
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
             #chan = self.meet.timer.chanid(e.chan)
-        if chan == self.startchan:
+        if chan == self.startchan or chan == timy.CHAN_START:
             self.log.debug('Got a start impulse.')
             self.starttrig(e)
         elif chan == self.finchan:
@@ -1160,6 +1230,7 @@ class race(object):
         rcount = 0
         pcount = 0
         for r in self.riders:
+            plstr = u''
             rcount += 1
             rno = r[COL_BIB].decode('utf-8')
             rh = self.meet.newgetrider(rno, self.series)
@@ -1169,12 +1240,6 @@ class race(object):
             inf = r[COL_INFO].decode('utf-8','replace').strip()
             if self.evtype in [u'keirin', u'sprint']: # encirc draw no
                 inf = strops.drawno_encirc(inf)
-            # but suppress inf if within an omnium
-            if self.inomnium:
-                inf = None
-            if rh is not None and rh[u'ucicode']:
-                inf = rh[u'ucicode']   # overwrite by force
-            plstr = u''
             if r[COL_DNF]:
                 pcount += 1
                 if r[COL_INFO] in ['dns', 'dsq']:
@@ -1185,6 +1250,11 @@ class race(object):
             elif self.onestart and r[COL_PLACE] != u'':
                 plstr = r[COL_PLACE].decode('utf-8','replace') + u'.'
                 pcount += 1
+            # but suppress inf if within an omnium
+            if self.inomnium:
+                inf = None
+            if self.evtype != u'handicap' and rh is not None and rh[u'ucicode']:
+                inf = rh[u'ucicode']   # overwrite by force
             if plstr:	# don't emit a row for unplaced riders
                 if not first:
                     sec.lines.append([plstr,rno,rname,inf, None, None])
@@ -1248,6 +1318,8 @@ class race(object):
         self.readonly = not ui
         self.comments = []
         self.onestart = False
+        self.runlap = None
+        self.lastrunlap = None
         self.start = None
         self.lstart = None
         self.finish = None
