@@ -38,14 +38,14 @@ with a rider are unicode strings.
 		Club/team/state/nation field
 	cate	"Category"
 		A space separated list of rider category keys for this rider
-	uci	"UCI Code"
-		Rider's UCI code
+	uci	"UCI ID"
+		Rider's UCI ID
 	lice	"License No."
 		Rider's License no or namebank key
 	rfid	"RFID No"
 		Rider's RFID, or Transponder no (if relevant)
-	note	"Notes"
-		Additional notes on this rider
+	nati	"Nation DoB"
+		Rider's Nation and DoB (old UCI Code)
 
 Reserved number series:
 
@@ -94,12 +94,12 @@ RIDER_COLUMNS = {
 	u'seri':	u"Series",
 	u'firs':	u"First Name",
 	u'last':	u"Last Name",
-	u'club':	u"Club",
+	u'club':	u"Club/Team",
 	u'cate':	u"Category",
-	u'uci':		u"UCI Code",
+	u'uci':		u"UCI ID",
 	u'lice':	u"License No.",
 	u'rfid':	u"RFID No",
-	u'note':	u"Notes"
+	u'nati':	u"Nation DoB"
 }
 
 RIDER_COLUMN_CONVERTERS= {
@@ -289,9 +289,9 @@ class riderdb(object):
         self.log.debug(u'Saving riders to '+ repr(csvfile))
         with open(csvfile, 'wb') as f:
             cr = ucsv.UnicodeWriter(f)
-            cr.writerow([u'No.',u'First Name',u'Last Name',u'Club',
+            cr.writerow([u'No.',u'First Name',u'Last Name',u'Club/Team',
                          u'Category',u'Series(optional)',u'Refid',
-                          u'UCIcode',u'Note'])
+                          u'UCI ID',u'Nation DoB'])
             cr.writerows(self)	# check the unicode out-in-out-in
 
     def gotorow(self, ref=None):
@@ -353,7 +353,7 @@ class riderdb(object):
         return ret
         
     def mkview(self, bib=True, first=True, last=True, club=True,
-                     cat=False, series=True, refid=False, ucicode=False):
+                     cat=False, series=True, refid=False, ucicode=False, note=False):
         """Create and return view object for the model."""
         if self.view is not None:
             return self.view
@@ -405,6 +405,11 @@ class riderdb(object):
                             editcb=self.__editstart_cb)
             self.colmap[COL_UCICODE] = colcnt
             colcnt += 1
+        if note:
+            uiutil.mkviewcoltxt(v, 'Note', COL_NOTE, self.__editcol_cb, 
+                            editcb=self.__editstart_cb)
+            self.colmap[COL_NOTE] = colcnt
+            colcnt += 1
         self.view = v
         return self.view
 
@@ -422,6 +427,41 @@ class riderdb(object):
             i = self.model.iter_next(i)
         return ret
 
+    def get_partner(self, bib, series=u''):
+        """Return a reference to the 'partner' for the specified rider."""
+        ret = None
+        bib = bib.lower()
+        i = self.model.get_iter_first()
+        while i is not None:
+            if (self.get_value(i, COL_BIB) == bib
+                and self.get_value(i, COL_SERIES) == series):
+                ret = gtk.TreeRowReference(self.model,
+                                           self.model.get_path(i))
+                break
+            i = self.model.iter_next(i)
+
+        if i is not None:
+            # look up cat and note for partner link
+            cat = self.get_value(i, COL_CAT)
+            if cat and u'tandem' in cat.lower():
+                pno = self.get_value(i, COL_NOTE)
+                if pno:
+                    ret = self.getrider(pno, series)
+        return ret
+
+    def nextriderin(self, bib, series=u''):
+        """Return the rider following the specified bib in the series."""
+        ret = None
+        cur = self.getrider(bib, series)
+        if cur is not None:
+            i = self.model.iter_next(self.model.get_iter(cur.get_path()))
+            while i is not None:
+                if self.get_value(i, COL_SERIES) == series:
+                    ret = self.get_value(i, COL_BIB)
+                    break
+                i = self.model.iter_next(i)
+        return ret
+
     def nextriderno(self):
         """Try and return a new unique rider number string."""
         lmax = 1
@@ -433,14 +473,26 @@ class riderdb(object):
 
     def getrefid(self, refid):
         """Return a reference to the row with the given refid."""
+        ## altered to allow multiple references
         ret = None
         ck = refid.lower()
+        rno = None
+        rser = None
+        if u'riderno:' in ck: 
+            rno,rser = strops.bibstr2bibser(ck.split(u':')[-1])
         i = self.model.get_iter_first()
         while i is not None:
-            if self.get_value(i,COL_REFID).lower()==ck:
+            if rno and (self.get_value(i, COL_BIB) == rno
+                and self.get_value(i, COL_SERIES) == rser):
                 ret = gtk.TreeRowReference(self.model,
                                            self.model.get_path(i))
                 break
+            else:
+                for dbref in self.get_value(i,COL_REFID).lower().split():
+                    if dbref == ck:
+                        ret = gtk.TreeRowReference(self.model,
+                                           self.model.get_path(i))
+                        break
             i = self.model.iter_next(i)
         return ret
         
@@ -450,7 +502,7 @@ class riderdb(object):
         cat = cat.upper()
         i = self.model.get_iter_first()
         while i is not None:
-            if (self.get_value(i, COL_CAT).upper() == cat
+            if (cat in self.get_value(i, COL_CAT).upper().split()
                and self.get_value(i, COL_SERIES) == series):
                 ret += u' ' + self.get_value(i, COL_BIB)
             i = self.model.iter_next(i)
@@ -462,7 +514,7 @@ class riderdb(object):
         cat = cat.upper()
         i = self.model.get_iter_first()
         while i is not None:
-            if (self.get_value(i, COL_CAT).upper() == cat
+            if (cat in self.get_value(i, COL_CAT).upper().split()
                and self.get_value(i, COL_SERIES) == series):
                 ret.append(gtk.TreeRowReference(self.model,
                                            self.model.get_path(i)))
@@ -482,11 +534,12 @@ class riderdb(object):
         """Return a list of all the categories in the specified series."""
         ret = []
         for row in self.model:
-            ucat = row[COL_CAT].decode('utf-8').upper()
+            ucat = row[COL_CAT].decode('utf-8').upper().split()
             useries = row[COL_SERIES].decode('utf-8')
-            if ucat and ucat not in ret and (series is None
+            for rrcat in ucat:
+                if rrcat and rrcat not in ret and (series is None
                                              or useries == series):
-                ret.append(ucat)
+                    ret.append(rrcat)
         return ret
 
     def get_value(self, iter, column):
