@@ -27,6 +27,7 @@ import gobject
 
 import os
 import sys
+import csv
 import logging
 import random
 
@@ -38,7 +39,8 @@ from metarace import tod
 from metarace import eventdb
 from metarace import riderdb
 from metarace import thbc
-from metarace import wheeltime
+from metarace import rrs
+from metarace import rru
 from metarace import timy
 from metarace import telegraph
 from metarace import unt4
@@ -51,18 +53,23 @@ from metarace import namebank
 
 LOGHANDLER_LEVEL = logging.DEBUG
 ROADRACE_TYPES = {u'irtt':u'Road Time Trial',
+                  u'billy':u'Time Trial',
                   u'rms':u'Road Race',
                   u'trtt':u'Team Road Time Trial',
                   u'crit':u'Criterium/Kermesse',
                   u'rhcp':u'Handicap',
                   u'sportif':u'Sportif Ride',
-                  u'cross':u'Cyclocross Race'}
+                  u'crosslap':u'Cross Laps',
+                  u'cross':u'Cyclocross Race',
+                  u'hour24':u'24 Hour Road Race'}
 CONFIGFILE = u'config.json'
 LOGFILE = u'event.log'
 ROADMEET_ID = u'roadmeet_2.0'  # configuration versioning
 DEFAULT_RFID_HANDLER = u'thbc'
 RFID_HANDLERS = {u'thbc':thbc.thbc,
-                 u'wheeltime':wheeltime.wheeltime}
+                 u'rrs':rrs.rrs,
+                 u'rru':rru.rru}
+                 #u'wheeltime':wheeltime.wheeltime}
 # Bunches colourmap
 COLOURMAP=[['#ffa0a0','red',1.0,0.1,0.1],
            ['#a0ffa0','green',0.1,1.0,0.1],
@@ -99,6 +106,7 @@ class registerdlg(object):
         self.serent = b.get_object('series_entry')
         self.riderval = b.get_object('rider_val')
         self.autoinc = b.get_object('autoinc_but')
+        self.autotrack = b.get_object('autotrack_but')
         self.storeid = None     # TODO: track scanned rfid with changing bib
         b.connect_signals(self)
 
@@ -156,9 +164,16 @@ class registerdlg(object):
     def increment_rider(self):
         cbib = self.bibent.get_text().decode('utf-8').strip()
         if cbib.isdigit():
-            nbib = int(cbib) + 1
-            self.bibent.set_text(unicode(nbib))
-            self.bibent.activate()
+            if self.autotrack.get_active():
+                nbib = self.rdb.nextriderin(cbib,
+                                  self.serent.get_text().decode('utf-8'))
+                if nbib is not None:
+                    self.bibent.set_text(unicode(nbib))
+                #self.bibent.activate()	# no need to acitvate on extant no
+            else:
+                nbib = int(cbib) + 1
+                self.bibent.set_text(unicode(nbib))
+                self.bibent.activate()
         return False
 
     def register_tag(self, e):
@@ -262,6 +277,11 @@ class fakemeet:
                u'docindex':u'0',
                u'resultnos':u'Yes',
                u'uscbport':u'',
+               u'competitioncode': u'',
+               u'eventcode': u'',
+               u'racetype': u'',
+               u'competitortype': u'',
+               u'documentversion': u'',
                u'id':u''}})
         cr.add_section(u'roadmeet')
         cr.merge(metarace.sysconf, u'roadmeet')
@@ -280,9 +300,15 @@ class fakemeet:
         self.date_str = cr.get(u'roadmeet', u'date')
         self.organiser_str = cr.get(u'roadmeet', u'organiser')
         self.commissaire_str = cr.get(u'roadmeet', u'commissaire')
+        self.linkbase = cr.get(u'roadmeet', u'linkbase')
         self.logo = cr.get(u'roadmeet', u'logo')
         self.distance = strops.confopt_float(cr.get(u'roadmeet', u'distance'))
         self.docindex = strops.confopt_posint(cr.get(u'roadmeet', u'docindex'), 0)
+        self.competitioncode = cr.get(u'roadmeet', u'competitioncode')
+        self.eventcode = cr.get(u'roadmeet', u'eventcode')
+        self.racetype = cr.get(u'roadmeet', u'racetype')
+        self.competitortype = cr.get(u'roadmeet', u'competitortype')
+        self.documentversion = cr.get(u'roadmeet', u'documentversion')
         self.bibs_in_results = strops.confopt_bool(cr.get(u'roadmeet',
                                                           u'resultnos'))
     def event_configfile(self, evno):
@@ -308,6 +334,7 @@ class roadmeet:
         dlg = gtk.FileChooserDialog(u'Open new road meet', self.window,
             gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL,
             gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        dlg.set_current_folder(self.configpath)
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             self.configpath = dlg.get_filename()
@@ -334,6 +361,12 @@ class roadmeet:
         """Lookup cats and write them into the supplied entry."""
         entry.set_text(' '.join(self.rdb.listcats()))
         
+    def menu_race_properties_activate_cb(self, menuitem, data=None):
+        """Edit race specific properties."""
+        if self.curevent is not None:
+            self.log.debug(u'Editing race properties.')
+            self.curevent.edit_event_properties(self.window)
+
     ## TODO: call into curevent and fetch configurables + use generic props
     def menu_meet_properties_cb(self, menuitem, data=None):
         """Edit meet properties."""
@@ -454,6 +487,8 @@ class roadmeet:
 
     def menu_meet_fullscreen_toggled_cb(self, button, data=None):
         """Update fullscreen window view."""
+        ## get current state:
+        #gtk.gdk.WINDOW_STATE_FULLSCREEN
         if button.get_active():
             self.window.fullscreen()
         else:
@@ -502,6 +537,9 @@ class roadmeet:
         ofile = os.path.join(self.configpath, u'output.xls')
         with open(ofile, 'wb') as f:
             rep.output_xls(f)
+        ofile = os.path.join(self.configpath, u'output.html')
+        with open(ofile, 'wb') as f:
+            rep.output_html(f)
         return False
 
     def begin_print(self,  operation, context, rep):
@@ -587,12 +625,21 @@ class roadmeet:
             if self.etype == u'irtt':
                 from metarace import irtt
                 self.curevent = irtt.irtt(self, eventhdl, True)
+            elif self.etype == u'billy':
+                from metarace import billy
+                self.curevent = billy.billy(self, eventhdl, True)
             elif self.etype == u'trtt':
                 from metarace import trtt
                 self.curevent = trtt.trtt(self, eventhdl, True)
             elif self.etype == u'cross':
                 from metarace import cross
                 self.curevent = cross.cross(self, eventhdl, True)
+            elif self.etype == u'hour24':
+                from metarace import hour24
+                self.curevent = hour24.hour24(self, eventhdl, True)
+            elif self.etype == u'crosslap':
+                from metarace import crosslap
+                self.curevent = crosslap.crosslap(self, eventhdl, True)
             elif self.etype == u'sportif':
                 from metarace import sportif
                 self.curevent = sportif.sportif(self, eventhdl, True)
@@ -706,7 +753,8 @@ class roadmeet:
 
     def menu_import_riders_activate_cb(self, menuitem, data=None):
         """Add riders to database."""
-        sfile = uiutil.loadcsvdlg(u'Select rider file to import', self.window)
+        sfile = uiutil.loadcsvdlg(u'Select rider file to import',
+                                   self.window, self.configpath)
         if sfile is not None:
             with namebank.namebank() as n:            
                 self.rdb.load(sfile, namedb=n, overwrite=True)
@@ -715,7 +763,8 @@ class roadmeet:
 
     def menu_import_chipfile_activate_cb(self, menuitem, data=None):
         """Import a transponder chipfile."""
-        sfile = uiutil.loadcsvdlg(u'Select chipfile to import', self.window)
+        sfile = uiutil.loadcsvdlg(u'Select chipfile to import',
+                                   self.window, self.configpath)
         if sfile is not None:
             self.rdb.load_chipfile(sfile)
         else:
@@ -729,7 +778,7 @@ class roadmeet:
             return
 
         sfile = uiutil.loadcsvdlg(u'Select logfile to import',
-                                  self.window)
+                                  self.window, self.configpath)
         if sfile and os.path.isfile(sfile):
             self.timer.replay(sfile)
         else:
@@ -741,26 +790,39 @@ class roadmeet:
             self.log.info(u'No event open to import to.')
             return
         sfile = uiutil.loadcsvdlg(u'Select LIF file to import',
-                                  self.window)
+                                  self.window, self.configpath)
         if os.path.isfile(sfile):
             with open(sfile, 'rb') as f:
                 cr = ucsv.UnicodeReader(f)
                 ltime = None
                 count = 0
                 for r in cr:
-                    if len(r) > 8:
-                        if r[2] == u'':	# rider record
-                            if r[1].isalnum() and r[1] != u'No.':
-                                bib = r[1].strip().lower()
-                                rank = strops.confopt_posint(r[0], None)
-                                if bib and rank is not None:
-                                    rtime = tod.str2tod(r[6])
-                                    dtime = tod.str2tod(r[8])
-                                    if dtime is not None:
-                                        # this is a new bunch
-                                        ltime = rtime
-                                    self.curevent.setriderval(bib, rank, ltime)
-                                    count += 1
+                    ## TEMP: Race Result Import Override
+                    if len(r) > 5:
+                        rrank = strops.confopt_posint(r[0].split(u'.')[0],None)
+                        rno = r[1]
+                        rtim = tod.str2tod(r[5])
+                        rbon = tod.str2tod(r[4])
+                        if rrank is not None:
+                            #self.curevent.setrftime(rno, rrank, rtim, rbon)
+                            self.curevent.setriderval(rno, rrank, rtim, rbon)
+                        
+                    #if len(r) > 8:
+                        #if r[3][0:5] != u'Etape':
+                            #if r[1].isalnum() and r[1] != u'No.':
+                                #bib = r[1].strip().lower()
+                                #rank = strops.confopt_posint(r[0], None)
+                                #if bib and rank is not None:
+                                    #rftime = tod.str2tod(r[2])
+                                    #rtime = tod.str2tod(r[6])
+                                    #dtime = tod.str2tod(r[8])
+                                    #if rftime is not None:
+                                        #self.curevent.setrftime(bib, rank, rftime)
+                                    #elif dtime is not None:
+                                        ## this is a new bunch
+                                        #ltime = rtime
+                                        #self.curevent.setriderval(bib, rank, ltime)
+                                    #count += 1
                 self.log.info(u'Loaded ' + unicode(count) + u' riders from '
                                     + repr(sfile))
         else:
@@ -775,7 +837,7 @@ class roadmeet:
             return
         count = 0
         sfile = uiutil.loadcsvdlg(u'Select startlist file to import',
-                                  self.window)
+                                  self.window, self.configpath)
         if os.path.isfile(sfile):
             with open(sfile, 'rb') as f:
                 cr = ucsv.UnicodeReader(f)
@@ -803,7 +865,8 @@ class roadmeet:
     def menu_export_riders_activate_cb(self, menuitem, data=None):
         """Export rider database."""
         sfile = uiutil.savecsvdlg(u'Select file to export riders to',
-                                  self.window)
+                                  self.window,u'riders_export.csv',
+                                  self.configpath)
         if sfile is not None:
             self.rdb.save(sfile)
         else:
@@ -812,7 +875,7 @@ class roadmeet:
     def menu_export_chipfile_activate_cb(self, menuitem, data=None):
         """Export transponder chipfile from rider model."""
         sfile = uiutil.savecsvdlg(u'Select file to export refids to',
-                                   self.window)
+                                self.window, u'chipfile.csv', self.configpath)
         if sfile is not None:
             self.rdb.save_chipfile(sfile)
         else:
@@ -826,7 +889,7 @@ class roadmeet:
     
         rfilename = uiutil.savecsvdlg(u'Select file to save results to.',
                                        self.window,
-                                       u'results.csv')
+                                       u'results.csv', self.configpath)
         if rfilename is not None:
             with open(rfilename , 'wb') as f:
                 cw = ucsv.UnicodeWriter(f)
@@ -841,6 +904,12 @@ class roadmeet:
                             opr[i] = unicode(r[i].timeval)
                     cw.writerow(opr)
             self.log.info(u'Exported meet results to ' + repr(rfilename))
+            lifdat = self.curevent.lifexport()
+            if len(lifdat) > 0:
+                with open(u'lifexport.lif', u'wb') as f:
+                    cw = ucsv.UnicodeWriter(f, quoting=csv.QUOTE_MINIMAL)
+                    for r in lifdat:
+                        cw.writerow(r)
 
     def menu_export_startlist_activate_cb(self, menuitem, data=None):
         """Extract startlist from current event."""
@@ -850,7 +919,7 @@ class roadmeet:
     
         rfilename = uiutil.savecsvdlg(u'Select file to save startlist to.',
                                        self.window,
-                                       u'startlist.csv')
+                                       u'startlist.csv', self.configpath)
         if rfilename is not None:
             with open(rfilename , 'wb') as f:
                 cw = ucsv.UnicodeWriter(f)
@@ -926,8 +995,7 @@ class roadmeet:
 
         ofile = os.path.join(exportpath, filebase+u'.pdf')
         # HACK: metarace site should be in config
-        lb = os.path.join(u'http://metarace.com.au/site',
-                            self.mirrorpath, filebase)
+        lb = os.path.join(self.linkbase, self.mirrorpath, filebase)
         rep.canonical = u'.'.join([lb, u'pdf'])
         with open(ofile, 'wb') as f:
             rep.output_pdf(f)
@@ -937,10 +1005,10 @@ class roadmeet:
         ofile = os.path.join(exportpath, filebase+u'.json')
         with open(ofile, 'wb') as f:
             rep.output_json(f)
-        lt = [u'pdf', u'xls', u'txt', u'json']
-        ofile = os.path.join(exportpath, filebase+u'.txt')
+        lt = [u'pdf', u'xls', u'json']
+        ofile = os.path.join(exportpath, filebase+u'.html')
         with open(ofile, 'wb') as f:
-            rep.output_text(f, linkbase=lb, linktypes=lt)
+            rep.output_html(f, linkbase=lb, linktypes=lt)
 
     def menu_data_results_cb(self, menuitem, data=None):
         """Create live result report and export to MR"""
@@ -949,9 +1017,18 @@ class roadmeet:
         if self.curevent is None:
             return
 
+        if False:	# save current lif with export 
+            lifdat = self.curevent.lifexport()
+            if len(lifdat) > 0:
+                with open(u'lifexport.lif', u'wb') as f:
+                    cw = ucsv.UnicodeWriter(f, quoting=csv.QUOTE_MINIMAL)
+                    for r in lifdat:
+                        cw.writerow(r)
+
         # only build reports if the race is really stand-alone
-        if self.etype in [u'sportif'] or self.mirrorcmd == u'rsync':
-            self.export_result_maker()
+        if not u'mk' in self.mirrorcmd:
+            if self.etype in [u'sportif'] or self.mirrorcmd != u'echo':
+                self.export_result_maker()
 
         glib.idle_add(self.mirror_start)
         self.log.info(u'Race info export.')
@@ -1026,13 +1103,38 @@ class roadmeet:
         self.log.debug(u'START')
         self.timer.start_session()
         self.alttimer.setport(self.alttimer_port)
-        self.alttimer.sane()
+        self.alttimer.sane()	#!! sets delay to 1.0s!
         ## TODO: move this and the others into method, and include
         ## all road-ish events: 'rhcp', 'rms', None
         if self.etype == 'rms':	# road race finish requires 1.0s delay
             self.alttimer.write('DTF01.00')
+        else:
+            self.alttimer.delaytime('0.01')	# irtt likes no delay
         self.log.info(u'Re-connect/re-start attached timer.')
 
+    def restart_decoder(self, data=None):
+        """Request re-start of decoder."""
+        self.log.debug(u'START')
+        self.timer.start_session()
+        return None
+
+    def menu_timing_configure_activate_cb(self, menuitem, data=None):
+        """Attempt to re-configure the attached decoder from saved config."""
+        if not uiutil.questiondlg(self.window,
+                                  u'Re-configure Decoder Settings?',
+                                  u'Note: Passings will not be registered while decoder is being configured. Connection must be serial to update IP config.'):
+            self.log.debug(u'Config aborted.')
+            return False
+
+        self.log.debug(u'STOP')
+        self.timer.stop_session()
+        self.log.debug(u'SANE')
+        self.timer.sane()
+        self.log.debug(u'IPCONFIG')
+        glib.timeout_add_seconds(60, self.restart_decoder)
+        self.timer.ipconfig()
+        return None
+
     ## Help menu callbacks
     def menu_help_docs_cb(self, menuitem, data=None):
         """Display program help."""
@@ -1185,10 +1287,11 @@ class roadmeet:
         cw.set(u'roadmeet', u'uscbchan', self.uscbchan)
         cw.set(u'roadmeet', u'uscbopt', self.remote_enable)
         cw.set(u'roadmeet', u'timer', self.timer_port)
-        cw.set(u'roadmeet', u'rfidlvl', self.rfidlvl)
+        #cw.set(u'roadmeet', u'rfidlvl', self.rfidlvl)
         cw.set(u'roadmeet', u'alttimer', self.alttimer_port)
 
         cw.set(u'roadmeet', u'shortname', self.shortname)
+        cw.set(u'roadmeet', u'linkbase', self.linkbase)
         cw.set(u'roadmeet', u'title', self.title_str)
         cw.set(u'roadmeet', u'subtitle', self.subtitle_str)
         cw.set(u'roadmeet', u'document', self.document_str)
@@ -1204,6 +1307,11 @@ class roadmeet:
         cw.set(u'roadmeet', u'mirrorpath', self.mirrorpath)
         cw.set(u'roadmeet', u'mirrorcmd', self.mirrorcmd)
         cw.set(u'roadmeet', u'mirrorfile', self.mirrorfile)
+        cw.set(u'roadmeet', u'competitioncode', self.competitioncode)
+        cw.set(u'roadmeet', u'eventcode', self.eventcode)
+        cw.set(u'roadmeet', u'racetype', self.racetype)
+        cw.set(u'roadmeet', u'competitortype', self.competitortype)
+        cw.set(u'roadmeet', u'documentversion', self.documentversion)
 
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         self.log.debug(u'Saving meet config to ' + repr(cwfilename))
@@ -1255,15 +1363,21 @@ class roadmeet:
                u'distance':u'',
                u'docindex':u'0',
                u'timer':u'',
-               u'rfidlvl':[u'30', u'30'],
+               #u'rfidlvl':[u'30', u'30'],
                u'alttimer':u'',
                u'resultnos':u'Yes',
                u'uscbport':u'',
                u'uscbchan':u'',
+               u'linkbase':u'.',
                u'uscbopt':u'No',
                u'mirrorpath':u'',
                u'mirrorcmd':u'echo',
                u'mirrorfile':u'',
+               u'competitioncode': u'',
+               u'eventcode': u'',
+               u'racetype': u'',
+               u'competitortype': u'',
+               u'documentversion': u'',
                u'loglevel':unicode(logging.INFO),
                u'id':u''}})
         cr.add_section(u'roadmeet')
@@ -1305,14 +1419,14 @@ class roadmeet:
 
         # set timer port (decoder)
         self.set_timer(cr.get(u'roadmeet', u'timer'))
-        boxlvl = u'20'
-        stalvl = u'30'
-        nv = cr.get(u'roadmeet', u'rfidlvl')
-        if len(nv) > 1:
-            boxlvl = nv[0]
-            stalvl = nv[1]
-        self.timer.setlvl(boxlvl, stalvl)
-        self.rfidlvl = [boxlvl, stalvl]
+        #boxlvl = u'20'
+        #stalvl = u'30'
+        #nv = cr.get(u'roadmeet', u'rfidlvl')
+        #if len(nv) > 1:
+            #boxlvl = nv[0]
+            #stalvl = nv[1]
+        #self.timer.setlvl(boxlvl, stalvl)
+        #self.rfidlvl = [boxlvl, stalvl]
 
         # set alt timer port (timy)
         nport = cr.get(u'roadmeet', u'alttimer')
@@ -1334,11 +1448,17 @@ class roadmeet:
         self.logo = cr.get(u'roadmeet', u'logo')
         self.distance = strops.confopt_float(cr.get(u'roadmeet', u'distance'))
         self.docindex = strops.confopt_posint(cr.get(u'roadmeet', u'docindex'), 0)
+        self.linkbase = cr.get(u'roadmeet', u'linkbase')
         self.bibs_in_results = strops.confopt_bool(cr.get(u'roadmeet',
                                                           u'resultnos'))
         self.mirrorpath = cr.get(u'roadmeet', u'mirrorpath')
         self.mirrorcmd = cr.get(u'roadmeet', u'mirrorcmd')
         self.mirrorfile = cr.get(u'roadmeet', u'mirrorfile')
+        self.competitioncode = cr.get(u'roadmeet', u'competitioncode')
+        self.eventcode = cr.get(u'roadmeet', u'eventcode')
+        self.racetype = cr.get(u'roadmeet', u'racetype')
+        self.competitortype = cr.get(u'roadmeet', u'competitortype')
+        self.documentversion = cr.get(u'roadmeet', u'documentversion')
 
         # Re-Initialise rider and event databases
         self.rdb.clear()
@@ -1347,7 +1467,7 @@ class roadmeet:
         self.edb.load(os.path.join(self.configpath, u'events.csv'))
         event = self.edb.getfirst()
         if event is None:	# add a new event of the right type
-            event = self.edb.add_empty(evno=u'00')
+            event = self.edb.add_empty(evno=u'0')
             event[u'type'] = self.etype
         else:
             self.etype = event[u'type']
@@ -1389,10 +1509,13 @@ class roadmeet:
                 timer = self.timer
             prec = 4
             if timer is self.timer:
-                prec = 2	# transponders only read to hundredths
+                prec = 3	# transponders have reduced precision
             elif u'M' in evt.chan:
-                prec = 2
-            tvec = [evt.index, timer.unitno, evt.chan,
+                prec = 3
+            source = timer.unitno
+            if evt.source is not None:
+                source = evt.source
+            tvec = [evt.index, source, evt.chan,
                     evt.refid, evt.rawtime(prec)]
             self.scb.send_cmd(u'timer',unichr(unt4.US).join(tvec),
                                        self.timerchan)
@@ -1510,6 +1633,10 @@ class roadmeet:
                         self.alttimer.trig(timeval=tv[4], index=tv[0],
                                     chan=tv[2], refid=tv[3], sourceid=srcid)
                     else:
+                        if srcid == u'3km':
+                            thetime = tod.str2tod(tv[4])
+                            if thetime is not None:
+                                tv[4] = (thetime - tod.tod(u'30:00')).rawtime()
                         self.timer.trig(timeval=tv[4], index='REM',
                                     chan=tv[2], refid=tv[3], sourceid=srcid)
                 except Exception as e:
@@ -1543,6 +1670,7 @@ class roadmeet:
         self.date_str = u''
         self.organiser_str = u''
         self.commissaire_str = u''
+        self.linkbase = u'.'
         self.logo = u''
         self.distance = None
         self.docindex = 0
@@ -1566,7 +1694,7 @@ class roadmeet:
         self.timerchan = '#timing'	# raw timing channel on telegraph
         self.timer = thbc.thbc()	# default is Tag, may change later
         self.timer_port = u''
-        self.rfidlvl = [u'20',u'30']
+        #self.rfidlvl = [u'20',u'30']
         self.alttimer = timy.timy()
         self.alttimer_port = u''
         self.uscbport = u''
@@ -1577,6 +1705,11 @@ class roadmeet:
         self.mirrorcmd = u'echo'
         self.mirrorfile = u''
         self.mirror = None 
+        self.competitioncode = u''
+        self.eventcode = u''
+        self.racetype = u''
+        self.competitortype = u''
+        self.documentversion = u''
 
         b = gtk.Builder()
         b.add_from_file(os.path.join(metarace.UI_PATH, u'roadmeet.ui'))
@@ -1659,7 +1792,7 @@ class roadmeet:
         b.get_object('riders_box').add(self.rdb.mkview(cat=True,
                                                   series=True,
                                                   refid=True,
-                                                  ucicode=True))
+                                                  ucicode=True,note=True))
 
         # select event page in notebook.
         b.get_object('meet_nb').set_current_page(1)
