@@ -63,6 +63,20 @@ MAPWIDTH=STARTTIME*TIMETICK
 MAPHMARGIN=8
 MAPVMARGIN=6
 
+## interpolate between two x,y points
+def posinterp(pp1, pp2, t):
+    """Return an x,y point in the segment pp1, pp2 at offset t."""
+    xl = pp1[3]
+    yl = pp1[4]
+    tl = pp1[0]
+    xh = pp2[3]
+    yh = pp2[4]
+    th = pp2[0]
+    frac = (t-tl)/(th-tl)
+    ret = (xl + frac*(xh-xl),
+           yl + frac*(yh-yl))
+    return ret
+
 ## TODO: replace this fn
 def roundedrecMoonlight(cr,x,y,w,h,radius_x=4,radius_y=4):
     """Draw a rounded rectangle."""
@@ -133,6 +147,13 @@ class rr_announce(object):
         self.hide()
         glib.idle_add(self.app_destroy_handler)
     
+    def track_area_expose_event_cb(self, widget, event):
+        """Update desired portion of drawing area."""
+        x , y, width, height = event.area
+        widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL],
+                                    self.track_src, x, y, x, y, width, height)
+        return False
+
     def map_area_expose_event_cb(self, widget, event):
         """Update desired portion of drawing area."""
         x , y, width, height = event.area
@@ -156,6 +177,120 @@ class rr_announce(object):
         cr.set_source_rgb(0.2,0.2,0.2)
         cr.stroke()
 
+    def checks(self, cr, x, y, sz):
+        """Draw check at pos x,y."""
+        cr.move_to(x-sz,y-sz)
+        cr.set_source_rgb(0.0,0.0,0.0)
+        cr.rectangle(x-sz,y-sz,sz,sz)
+        cr.rectangle(x,y,sz,sz)
+        cr.fill()
+        cr.set_source_rgb(1.0,1.0,1.0)
+        cr.rectangle(x,y-sz,sz,sz)
+        cr.rectangle(x-sz,y,sz,sz)
+        cr.fill()
+
+    def climbs(self, cr, x, y, sz):
+        """Draw KoM icon at pos x,y."""
+        cr.set_source_rgb(0.0,0.4,0.0)
+        cr.move_to(x-sz,y-sz)
+        cr.line_to(x+sz,y-sz)
+        cr.line_to(x,y+sz)
+        cr.fill()
+
+    def encirc(self, cr, x, y, char, sz):
+        """Draw an enircled number atop a reference marker."""
+        # crosshairs
+        cz = 0.6*sz
+        cr.save()
+        cr.set_line_width(0.15*sz)
+        cr.set_source_rgb(0.0,0.0,0.0)
+        cr.move_to(x-cz, y-cz)
+        cr.line_to(x+cz, y+cz)
+        cr.move_to(x-cz, y+cz)
+        cr.line_to(x+cz, y-cz)
+        cr.stroke()
+
+        # encirc
+        cr.set_source_rgb(0.1,0.1,0.1)
+        cr.move_to(x, y+2.4*sz)
+        cr.arc(x, y+2.4*sz, 1.4*sz, 0, 6.3)
+        cr.fill()
+        # text
+        cr.move_to(x-0.7*sz, y+1.7*sz)
+        cr.scale(1.0,-1.0)
+        cr.set_source_rgb(1.0,1.0,1.0)
+        cr.set_font_size(2.0*sz)
+        cr.show_text(char)
+        cr.restore()
+
+    def track_redraw(self):
+        """Re-Draw lazy tracker."""
+        if self.track_src is None or not self.showtrack:
+            return
+        cr = self.track_src.cairo_create()
+        cr.identity_matrix()
+        # bg filled
+        cr.set_source_rgb(0.85,0.85,0.9)
+        cr.paint()
+        if self.profile is None:
+             return
+        width = self.track_winx
+        height = self.track_winy
+        # bounding box.
+        xl = self.track_bbox[0][0]
+        xh = self.track_bbox[1][0]
+        xm = xh-xl
+        yl = self.track_bbox[0][1]
+        yh = self.track_bbox[1][1]
+        ym = yh-yl
+        sf = (width-16)/xm
+        asf = (height-16)/ym
+        asz = xm/60.0
+        lsz = xm/150.0
+        if abs(asf) < abs(sf):
+            sf = asf		# use vert if too 'tall'
+            asz = ym/60.0
+            lsz = ym/150.0
+        xof = 8+(width-16)*(0-xl)/(xm)
+        yof = 8+(height-16)*yh/ym
+        cr.translate(xof, yof)
+        cr.scale(sf, -sf)
+        cr.set_line_width(lsz)
+        cr.set_source_rgb(0.05,0.05,0.95)
+        fp = self.profile[0]
+        lap = set() 
+        cr.move_to(fp[3],fp[4])
+        sp = True
+        for np in self.profile[1:]:
+            phash = (int(np[3]), int(np[4]))
+            if phash not in lap:
+                if sp:
+                    cr.line_to(np[3],np[4])
+                sp = True
+                lap.add(phash)
+            else:
+                sp = False
+                cr.move_to(np[3],np[4])
+        cr.stroke()
+
+
+        # finish line
+        lp = self.profile[-1]
+        self.checks(cr, lp[3], lp[4], asz)
+
+        # kom
+        lp = self.profile[27]
+        #self.climbs(cr, lp[3], lp[4], asz)
+
+        if self.lastmainpt is not None:
+            self.encirc(cr, self.lastmainpt[0], self.lastmainpt[1], u'M', asz)
+        if self.lastleadpt is not None:
+            self.encirc(cr, self.lastleadpt[0], self.lastleadpt[1], u'L', asz)
+        if self.lastchasept is not None:
+            self.encirc(cr, self.lastchasept[0], self.lastchasept[1], u'C', asz)
+        if self.lastsagpt is not None:
+            self.encirc(cr, self.lastsagpt[0], self.lastsagpt[1], u'S', asz)
+
     def map_redraw(self):
         """Lazy full map redraw method."""
         if self.map_src is None:
@@ -230,8 +365,15 @@ class rr_announce(object):
                cr.move_to(xp,MAPVMARGIN)
                cr.line_to(xp,MAPVMARGIN+dh)
                cr.stroke()
-        
-        
+        if self.distancemain is not None:
+           drem = 1000.0*self.distancemain
+           xof = (self.dmax-drem)/self.dmax
+           if xof < 1.0:
+               xp = MAPHMARGIN + dw*xof
+               cr.set_source_rgb(0.05,0.95,0.05)
+               cr.move_to(xp,MAPVMARGIN)
+               cr.line_to(xp,MAPVMARGIN+dh)
+               cr.stroke()
 
     def draw_ridermap(self, cr):
         width = self.map_winsz
@@ -293,6 +435,15 @@ class rr_announce(object):
         if inbox:
             self.do_bubble(cr, cnt, x1, x2)
 
+    def track_area_configure_event_cb(self, widget, event):
+        """Re-configure the drawing area and redraw the base image."""
+        x, y, width, height = widget.get_allocation()
+        self.track_winx = width
+        self.track_winy = height
+        self.track_src = gtk.gdk.Pixmap(widget.window, width, height)
+        self.track_redraw()
+        return True
+
     def map_area_configure_event_cb(self, widget, event):
         """Re-configure the drawing area and redraw the base image."""
         x, y, width, height = widget.get_allocation()
@@ -320,6 +471,42 @@ class rr_announce(object):
         self.riders.clear()
         self.redraw_flag = True
         
+    def projectpos(self, bunch, msg):
+        # receive a projected map position for main field
+        sr = float(msg)
+        if sr > 0 and sr < 1:
+            #print(u'search for : ' + repr(sr))
+            sp = self.profile[0]
+            np = None
+            for p in self.profile[1:]:
+                if p[0] > sr:
+                    np = p
+                    break
+                sp = p
+            npt = posinterp(sp, np, sr)
+            if bunch == u'main':
+                self.lastmainpt = npt
+            elif bunch == u'lead':
+                self.lastleadpt = npt
+            elif bunch == u'chase':
+                self.lastchasept = npt
+            elif bunch == u'sag':
+                self.lastsagpt = npt
+            self.redraw_flag = True
+
+    def lambert_fix(self, msg):
+        sr = msg.split(chr(unt4.US))
+        if len(sr) > 7:
+            lid = sr[0]		# lambert ident
+            tof = sr[1]
+            xof = sr[2]
+            yof = sr[3]
+            spd = sr[6]
+            tim = sr[7]
+            # compare tim to last fix time
+            # if out of order, ignore
+            # if new fix, save to lamb props and flag redraw
+
     def append_rider(self, msg):
         sr = msg.split(chr(unt4.US))
         if len(sr) == 5:
@@ -351,7 +538,7 @@ class rr_announce(object):
                         self.cur_bunchcnt,
                         COLOURMAP[self.cur_bunchid][0],
                         rftime]
-                elif rftime < self.last_time or rftime - self.last_time < tod.tod('1.0'):
+                elif rftime < self.last_time or rftime - self.last_time < tod.tod('5.0'):
                     # Case 2: Same bunch
                     self.last_time = rftime
                     self.cur_bunchcnt += 1
@@ -420,6 +607,16 @@ class rr_announce(object):
                 self.laptype = m.text
             else:
                 self.laptype = None     # force None for empty string
+        elif m.header == u'oncourse':
+            self.lambert_fix(m.text)
+        elif m.header == u'leadpos':
+            self.projectpos(u'lead', m.text)
+        elif m.header == u'mainpos':
+            self.projectpos(u'main', m.text)
+        elif m.header == u'chasepos':
+            self.projectpos(u'chase', m.text)
+        elif m.header == u'sagpos':
+            self.projectpos(u'sag', m.text)
         elif m.header == u'distance':
             self.distance = None
             try:
@@ -428,7 +625,16 @@ class rr_announce(object):
                     self.distance = a
                     self.redraw_flag = True
             except:
-                self.log.debug(u'Invalid distance: ' + repr(m.text))
+                self.log.debug(u'Invalid lead distance: ' + repr(m.text))
+        elif m.header == u'distancemain':
+            self.distancemain = None
+            try:
+                a = float(m.text)
+                if a > 0.1:
+                    self.distancemain = a
+                    self.redraw_flag = True	# might get two at a time
+            except:
+                self.log.debug(u'Invalid main distance: ' + repr(m.text))
         elif m.header == u'timerstat':
             self.timerstat = m.text
         elif m.header == u'bunches':
@@ -451,6 +657,7 @@ class rr_announce(object):
                                     u'groupcol':True,
                                     u'catcol':True,
                                     u'bunchmap':True,
+                                    u'showtrack':False,
                                     u'search':False,
                                     u'maxlaptime':'2:00',
                                     u'motd':MOTD}})
@@ -491,6 +698,9 @@ class rr_announce(object):
         if strops.confopt_bool(cr.get(u'road_announce', u'search')):
             self.search_ent.show()
             self.search_ent.set_sensitive(True)
+        self.showtrack = strops.confopt_bool(cr.get(u'road_announce', u'showtrack'))
+        if self.showtrack:
+            self.track_area.show()
         autoscroll = strops.confopt_bool(cr.get(u'road_announce',
                                                 u'autoscroll'))
         if autoscroll != self.autoscroll:
@@ -574,7 +784,16 @@ class rr_announce(object):
                 if erange < zrangemin:
                     self.zmin -= 0.5*(zrangemin - erange)
                     self.zmax = self.zmin + zrangemin
-                print(repr([zmin,zmax,dmax]))
+                #print(repr([zmin,zmax,dmax]))
+                routemeta = metarace.default_file(u'route.json')
+                rjdat = jsonconfig.config()
+                if os.path.exists(routemeta):
+                    with open(routemeta, 'rb') as g:
+                        rjdat.read(g)
+                    bbmin = rjdat.get(u'route', u'mapmin')
+                    bbmax = rjdat.get(u'route', u'mapmax')
+                    self.track_bbox = [[bbmin[u'x'],bbmin[u'y']],
+                                       [bbmax[u'x'],bbmax[u'y']]]
 
     def doautoscroll(self):
         """Automatically handle smooth scroll in text view after delay."""
@@ -682,7 +901,9 @@ class rr_announce(object):
         if self.redraw_flag:
             self.redraw_flag = False
             self.map_redraw()		# update src map
+            self.track_redraw()		# update src map
             self.map_area.queue_draw()	# queue copy to screen
+            self.track_area.queue_draw()	# queue copy to screen
 
         # check connection status
         if not self.io.connected():
@@ -749,6 +970,17 @@ class rr_announce(object):
         self.map_area.set_size_request(-1, 80)
         self.map_area.show()
 
+        self.track_winx = 0
+        self.track_winy = 0
+        self.track_xoft = 0
+        self.track_w = 0
+        self.track_area = b.get_object('track_area')
+        self.track_src = None
+        self.track_area.set_size_request(300, -1)
+        self.showtrack = False
+        ## defer to loadconfig?
+        self.track_area.hide()
+
         # lap & bunch status values
         self.cur_lap = tod.tod(0)
         self.cur_split = tod.tod(0)
@@ -786,7 +1018,7 @@ class rr_announce(object):
         uiutil.mkviewcoltxt(t, 'Rank', 0,width=45)
         uiutil.mkviewcoltxt(t, 'No.', 1,calign=1.0,width=45)
         uiutil.mkviewcoltxt(t, 'Rider', 2,expand=True,fixed=True)
-        uiutil.mkviewcoltxt(t, 'Cat', 3,calign=0.0)
+        #uiutil.mkviewcoltxt(t, 'Cat', 3,calign=0.0)
         uiutil.mkviewcoltxt(t, 'Time', 4,calign=1.0,width=80)
         uiutil.mkviewcoltxt(t, 'Bunch', 5,width=50,bgcol=6,calign=0.5)
         t.show()
@@ -804,10 +1036,15 @@ class rr_announce(object):
         self.finstr = None	# used with lapstr
         self.timerstat = u'running'     # default assume run state
         self.distance = None
+        self.distancemain = None
         self.laplbl = None
         self.laptype = None
         self.lapstart = None
         self.lapfin = None
+        self.lastmainpt = None
+        self.lastleadpt = None
+        self.lastchasept = None
+        self.lastsagpt = None
 
         # start timer
         self.tod = tod.tod(u'now').truncate(0)
