@@ -56,13 +56,13 @@ remotepath, mirrorcmd and data.
 
 Examples:
 
-Export via rsync/ssh to metarace web site
+Export via rsync/ssh to web site
 
    System config:
 
 	command = "rsync"
 	arguments = ["-av", "--rsh=ssh", "--rsync-path=bin/rsync",
-                     "{srcdir}", "metarace:html/site/{dstdir}"]
+                     "{srcdir}", "host:html/site/{dstdir}"]
 
    Progam:
  
@@ -72,7 +72,7 @@ Export via rsync/ssh to metarace web site
 
    This will create a subprocess and call the following command:
 
-       rsync -av --rsh=ssh --rsync-path=bin/rsync export metarace:html/site/
+       rsync -av --rsh=ssh --rsync-path=bin/rsync export host:html/site/t
 
    which will mirror the contents of the directory 'export' to the remote
    site under the directory html/site/t
@@ -181,13 +181,101 @@ class mirror(threading.Thread):
             arglist.insert(0,self.__mirrorcmd)
 
             self.log.debug(u'Calling subprocess: ' + repr(arglist))
-            ret = subprocess.check_call(arglist)
+            if self.__mirrorcmd == u'FTPLIB':
+                ret = ftpmirror(self.log, arglist)
+            else:
+                ret = subprocess.check_call(arglist)
             ## TODO: track process here and collect output to log.debug
             if self.__cb:
                 self.__cb(ret, self.__cbdata)
         except Exception as e:
             self.log.error(u'Error: ' + unicode(type(e)) + unicode(e))
-        self.log.debug(u'Complete: Returned ' + repr(ret))
+        self.log.info(u'Complete: Returned ' + repr(ret))
+
+TEMPPREFIX = u'.tmpchg_'
+# determine temp file for given file
+def tempfilename(efile):
+    ret = TEMPPREFIX + efile
+    return ret
+
+def ftpmirror(log, args):
+    """Update remote result using built in FTP file copy."""
+    import ftplib
+    import filecmp
+    import shutil
+
+    SERVERISBROKEN = False
+    if len(args) < 6:
+        log.error(u'Usage: FTPLIB local_path remote_path host user pass [broken]')
+        return -1
+    local = args[1]
+    remote = args[2]
+    host = args[3]
+    user = args[4]
+    pwd = args[5]
+    if len(args) > 6:
+        SERVERISBROKEN = True
+    if not os.path.exists(local):
+        log.error(u'Local mirror path does not exist.')
+        return -1
+    log.info(u'Copying ' + repr(local)
+              + ' to ' + repr(remote) + ' on ' + repr(host))
+    # walk local path, prepare a local list
+    props = {}
+    for (dp, dn, fn) in os.walk(local):
+        for f in fn:
+            if TEMPPREFIX not in f:
+                t = tempfilename(f)
+                props[os.path.join(dp,f)] = os.path.join(dp,t)
+    # check each file in local list for upload
+    writes = {}
+    for file in props:
+        tfile = props[file]
+        if os.path.exists(tfile):
+            if not filecmp.cmp(file, tfile, shallow=False):
+                # file is changed from last uploaded version
+                writes[file] = tfile
+        else:
+            writes[file] = tfile
+    ret = 0
+    tfile = None
+    if len(writes) > 0:
+        try:
+            fcnt = 0
+            log.debug(u'Connecting to remote host...')
+            f = ftplib.FTP()
+            log.debug(u'Connect: ' + repr(f.connect(host)))
+            log.debug(u'Login: ' + repr(f.login(user, pwd)))
+            log.debug(u'Remote path: ' + repr(f.cwd(remote)))
+            for file in sorted(writes):
+                tfile = writes[file]
+                # update temporary file
+                shutil.copyfile(file, tfile)
+                (path,tlip) = os.path.split(tfile)
+                (path,flip) = os.path.split(file)
+                with open(tfile, 'rb') as g:
+                    log.debug(u'Upload ' + repr(file) + u': '
+                          + repr(f.storbinary('STOR ' + tlip, g)))
+                if SERVERISBROKEN:
+                    # If the server is broken, delete before rename
+                    try:
+                        log.debug(u'Delete: ' + repr(f.delete(flip)))
+                    except:
+                        pass
+                log.debug(u'Rename: ' + repr(f.rename(tlip, flip)))
+                fcnt += 1
+            f.quit()
+            log.debug(u'Done, uploaded: ' + repr(fcnt))
+
+        except Exception as e:
+            if tfile is not None:
+                os.unlink(tfile)    # not windows safe
+            log.error('FTP error: ' + repr(e))
+            ret = -1
+    else:
+        log.debug(u'No changes to upload.')
+    return ret
+
 
 def http_notify(url, user=None, passwd=None, event=None, subevent=None,
                      stage=None, path=None, div=None, log=logging):
