@@ -58,7 +58,7 @@ from metarace import unt4
 from metarace import tod
 
 PAGE_INIT=10		# delay before table data starts displaying
-PAGE_DELAY=50		# def tenths of sec to hold each page of table
+PAGE_DELAY=60		# def tenths of sec to hold each page of table
 PAGE_ROWOFT=2		# first DHI row for table data
 DATE_FMT=('%a %d/%m/%y')
 
@@ -141,6 +141,7 @@ class scbclock(scbwin):
     def redraw(self):
         self.scb.setline(0, self.header,chan=u'#anascb')
         for i in range(1,self.scb.pagelen):
+            #print('sending clear line: ' + repr(i))
             self.scb.clrline(i,chan=u'#anascb')
         self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
@@ -202,10 +203,12 @@ class scbtt(scbwin):
         >>>>>>>>(3) hh:mm:ss.dcm 
 
     """
-    def __init__(self, scb=None, header=u'',line1=u'', line2=u''):
+    def __init__(self, scb=None, header=u'',line1=u'', line2=u'',subheader=u''):
         scbwin.__init__(self, scb)
-        self.header = header
+        self.header = header.strip().center(self.scb.linelen)
+        self.subheader = subheader.strip().center(self.scb.linelen)
         self.line1 = line1
+        self.nextline1 = line1
         self.line2 = line2
         self.nextline2 = line2
         self.curt1 = u''
@@ -225,6 +228,8 @@ class scbtt(scbwin):
         clrset = list(range(0,self.scb.pagelen))
         self.scb.setline(0, self.header,chan=u'#anascb')
         clrset.remove(0)
+        self.scb.setline(1, self.subheader,chan=u'#anascb')
+        clrset.remove(1)
         self.scb.setline(2+self.singleoft, self.line1,chan=u'#anascb')
         clrset.remove(2+self.singleoft)
         self.scb.setline(4+self.singleoft, self.line2,chan=u'#anascb')
@@ -232,6 +237,7 @@ class scbtt(scbwin):
         # only clear rows not already set above.
         for i in clrset:
             self.scb.clrline(i,chan=u'#anascb')
+        self.nextline1 = self.line1
         self.nextline2 = self.line2
         self.curt1 = u''
         self.nextt1 = u''
@@ -244,6 +250,10 @@ class scbtt(scbwin):
         self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
+    def setline1(self, line1str=u''):
+        """Replace the line 1 text."""
+        self.nextline1 = line1str
+ 
     def setline2(self, line2str=u''):
         """Replace the line 2 text."""
         self.nextline2 = line2str
@@ -282,10 +292,15 @@ class scbtt(scbwin):
                 self.curr2 = self.nextr2
                 self.curt2 = self.nextt2
                 flush = True
+            if self.line1 != self.nextline1:
+                self.line1 = self.nextline1
+                self.scb.setline(2+self.singleoft,
+                                 self.nextline1,chan=u'#anascb')
+                flush = True
             if self.line2 != self.nextline2:
-                # A hack to allow display of speed string at single finish
                 self.line2 = self.nextline2
-                self.scb.setline(4, self.nextline2,chan=u'#anascb')
+                self.scb.setline(4+self.singleoft,
+                                 self.nextline2,chan=u'#anascb')
                 flush = True
             self.count += 1
             if flush:
