@@ -185,11 +185,18 @@ class sportif(object):
                 self.maxmap[mcat] = mtm
 
         starters = cr.get(u'event', u'startlist')
-        # for a sportif - always sort by bib
+        # for a sportif - allow lookup on refid by riderno
         for r in starters:
-            self.addrider(r)
+            bib = r	# unless is a refid
+            sr = self.meet.rdb.getrefid(r)
+            if sr is not None:
+                ser = self.meet.rdb.getvalue(sr, riderdb.COL_SERIES)
+                if ser == self.series:
+                    # overwrite bib with lookup from riderdb
+                    bib = self.meet.rdb.getvalue(sr, riderdb.COL_BIB)
+            self.addrider(bib)
             if cr.has_option(u'riders', r):
-                nr = self.getrider(r)
+                nr = self.getrider(bib)
                 ril = cr.get(u'riders', r)      # rider op is vec
                 # bib = comment,rftod,rfseen...
                 lr = len(ril)
@@ -723,9 +730,9 @@ class sportif(object):
         r = self.meet.rdb.getrefid(e.refid)
         if r is None:
             self.log.info('Unknown tag: ' + e.refid + '@' + e.rawtime(2))
-            return False
-            #r = self.meet.rdb.addempty(e.refid, self.series)
-            #self.meet.rdb.editrider(r, refid=e.refid)
+            #return False
+            r = self.meet.rdb.addempty(e.refid, self.series)
+            self.meet.rdb.editrider(r, refid=e.refid)
 
         bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
         ser = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
