@@ -34,6 +34,7 @@ import metarace
 from metarace import tod
 from metarace import jsonconfig
 from metarace import timy
+from metarace import unt4
 from metarace import riderdb
 from metarace import scbwin
 from metarace import uiutil
@@ -129,6 +130,7 @@ class sprnd(object):
                                   u'contests':[],
                                   u'timerstat':None,
                                   u'showinfo':True,
+                                  u'comment':u'',
                                   u'autospec':u''
                                    },
                                 u'contests': { } })
@@ -144,6 +146,7 @@ class sprnd(object):
         self.info_expand.set_expanded(strops.confopt_bool(
                                   cr.get(u'event', u'showinfo')))
         self.autospec = cr.get(u'event', u'autospec')
+        self.comment = cr.get(u'event', u'comment')
         self.onestart = False
 
         # restore contests
@@ -293,6 +296,7 @@ class sprnd(object):
 
         cw.set(u'event', u'showinfo', self.info_expand.get_expanded())
         cw.set(u'event', u'timerstat', self.timerstat)
+        cw.set(u'event', u'comment', self.comment)
         cw.set(u'event', u'autospec', self.autospec)
         contestlist = [] 
         for c in self.contests:
@@ -345,6 +349,7 @@ class sprnd(object):
         self.curelap = None
         self.timerstat = 'idle'
         self.meet.timer.dearm(self.startchan)
+        self.meet.timer.dearm(timy.CHAN_START)
         self.meet.timer.dearm(self.finchan)
         uiutil.buttonchg(self.stat_but, uiutil.bg_none, 'Idle')
         self.stat_but.set_sensitive(True)
@@ -367,11 +372,13 @@ class sprnd(object):
             self.timerstat = 'armstart'
             uiutil.buttonchg(self.stat_but, uiutil.bg_armstart, 'Arm Start')
             self.meet.timer.arm(self.startchan)
+            self.meet.timer.arm(timy.CHAN_START)
         elif self.timerstat == 'armstart':
             self.timerstat = 'idle'
             self.time_lbl.set_text('')
             uiutil.buttonchg(self.stat_but, uiutil.bg_none, 'Idle')
             self.meet.timer.dearm(self.startchan)
+            self.meet.timer.dearm(timy.CHAN_START)
         return False	# for use in delayed callback
 
     def armfinish(self):
@@ -632,7 +639,7 @@ class sprnd(object):
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
-        if chan == self.startchan:
+        if chan == self.startchan or chan == timy.CHAN_START:
             self.log.debug(u'Got a start impulse.')
             self.starttrig(e)
         elif chan == self.finchan:
@@ -704,9 +711,24 @@ class sprnd(object):
             et = self.finish - self.start
             self.time_lbl.set_text(et.timestr(2))
             self.curelap = et
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'N F$',
+                            xx=0,
+                            yy=0,
+                            text=et.timestr(2)[0:12])
+            self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
         elif self.start is not None:    # Note: uses 'local start' for RT
-            self.time_lbl.set_text((tod.tod('now')
-                                      - self.lstart).timestr(1))
+            runtm  = (tod.tod('now') - self.lstart).timestr(1)
+            ## UDP hack
+            msg = unt4.unt4(
+                            header=unichr(unt4.DC3) + u'R F$',
+                            xx=0,
+                            yy=0,
+                            text=runtm[0:12])
+            self.meet.udptimer.sendto(msg.pack(),
+                                      (self.meet.udpaddr,6789))
+            self.time_lbl.set_text(runtm)
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).timestr(1))
         else:
@@ -932,6 +954,13 @@ class sprnd(object):
                                     cr[COL_B_STR].decode('utf-8'), None],
                                      None])
         ret.append(sec)
+
+        if self.comment:
+            sec = printing.bullet_text()
+            sec.subheading = u'Decisions of the commisaires panel'
+            sec.lines.append([None, self.comment])
+            ret.append(sec)
+
         return ret
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -989,6 +1018,7 @@ class sprnd(object):
         self.startchan = timy.CHAN_200
         self.finchan = timy.CHAN_FINISH
         self.contests = []
+        self.comment = u''
 
         self.contests = gtk.ListStore(
            gobject.TYPE_STRING,  # COL_CONTEST = 0
