@@ -81,6 +81,68 @@ CHAN_8 = 8
 CHAN_INT = 9  # complete the keypad - a key not from timy
 CHAN_UNKNOWN = -1
 
+# running number comparisons
+RUNNER_NOS = {
+ u'red': 0,
+ u'whi': 1,
+ u'blu': 2,
+ u'yel': 3,
+ u'grn': 4,
+ u'pin': 5,
+ u'bla': 6,
+ u'gry': 7,
+ u'ora': 8,
+ u'pur': 9,
+ u'rdw': 10,
+ u'blw': 11,
+ u'ylw': 12,
+ u'grw': 13
+}
+
+# UCI penalties: EN/Track
+UCITRACKCODES = {
+ u'A': u'warning',
+ u'B': u'fined [amt]',
+ u'C': u'relegated',
+ u'D': u'disqualified'
+}
+
+UCITRACKPEN = {
+ u'1': u'for not holding [his] line during the final sprint',
+ u'2': u'for riding on the blue band during the sprint',
+ u'3': u'for deliberately riding on the blue band during the race',
+ u'4': u'for not having held [his] line during the last 200 metres of the race',
+ u'5': u'for irregular movement to prevent [his] opponent from passing',
+ u'6': u'for dangerous riding in the final bend',
+ u'7': u'for dangerous riding during the race',
+ u'8': u"for entering the sprinter's lane when the opponent was already there",
+ u'9': u'for moving down towards the inside of the track when a rival was already there',
+ u'10': u'for moving down towards the inside of the track and forcing the other competitor off the track',
+ u'11': u'for crowding [his] opponent with the intention of causing [him] to slow down',
+ u'12': u'for moving outward with the intention of forcing the opponent to go up',
+ u'13': u'for going down too quickly after overtaking [his] opponent',
+ u'14': u'for deliberate and flagrant fault against [ext]',
+ u'15': u'for causing the crash of [his] opponent',
+ u'16': u'for having blocked an opponent',
+ u'21': u'for pushing [his] rival',
+ u'17': u'for being late at the start line',
+ u'19': u'for incorrect gestures',
+ u'20': u'for incorrect behaviour',
+ u'23': u'for incorrect behaviour or disrespect towards an official',
+ u'27': u'for protest with hands off handlebar',
+ u'30': u'for ignoring commissaires instructions to leave track after being overlapped',
+ u'31': u'for failure to obey commissaires instructions',
+ u'32': u'for failing to maintain proper control of the bicycle',
+ u'33': u'for taking off [his] helmet when on the track after passing the finish line',
+ u'18': u'for wearing only one number',
+ u'24': u'for foling or mutilating the race number',
+ u'22': u'for improper attire/advertising during the protocol ceremony',
+ u'25': u'for improper advertising on national jersey or short',
+ u'29': u'for not being ready with extra wheels or other equipment at the start',
+ u'28': u'for using two persons to give information the the [rider]',
+ u'29': u'qualified to [event] but did not start without justification'
+}
+
 def img_filename(path, basename=u''):
     """Return the 'best' filename for the given basename image."""
     ret = None
@@ -112,18 +174,24 @@ def riderno_key(bib):
     return bibstr_key(bib)
 
 def dnfcode_key(code):
-    """Return a dnf code sorting key."""
-    # '' [rel] dsq hd|otl dnf dns
-    dnfordmap = {u'':0, u'rel':2,
-                 u'dsq':2,
-                 u'hd':3,u'otl':3,
-                 u'dnf':4,
-                 u'dns':5}
+    """Return a rank/dnf code sorting key."""
+    # rank [rel] '' dsq hd|otl dnf dns
+    dnfordmap = {
+                 u'rel':8000,
+                 u'':8500,
+                 u'hd':8800,u'otl':8800,
+                 u'dnf':9000,
+                 u'dns':9500,
+                 u'dsq':10000,}
     ret = 0
     if code is not None:
         code = code.lower()
         if code in dnfordmap:
             ret = dnfordmap[code]
+        else:
+            code = code.strip(u'.')
+            if code.isdigit():
+                ret = int(code)
     return ret
 
 def bibstr_key(bibstr=u''):
@@ -133,11 +201,14 @@ def bibstr_key(bibstr=u''):
     if bib.isdigit():
         bval = int(bib)
     else:
-        sbib = bib.translate(INTEGER_UTRANS)
-        if sbib:
+        sbib = bib.translate(INTEGER_UTRANS).strip()
+        if sbib and sbib.isdigit():
             bval = int(sbib)
         else:
-            bval = id(bib)
+            if bib.lower()[0:3] in RUNNER_NOS:
+                bval = RUNNER_NOS[bib.lower()[0:3]]
+            else:
+                bval = id(bib)
     sval = 0
     if ser != u'':
         sval = ord(ser[0])<<12
@@ -281,7 +352,7 @@ def truncpad(srcline, length, align='l', elipsis=True):
     return ret
 
 def search_name(namestr):
-    return namestr.translate(BIBLIST_UTRANS).strip().lower().encode('ascii','ignore')
+    return unicode(namestr).translate(RIDERNO_UTRANS).strip().lower().encode('ascii','ignore')
 
 def resname_bib(bib, first, last, club):
     """Return rider name formatted for results with bib (champs/live)."""
@@ -380,9 +451,10 @@ def reformat_riderlist(riderstr, rdb=None, series=u''):
         else:
             ret += u' ' + nr
     # pass 3: reorder and join for return
-    rvec = list(set(ret.split()))
-    rvec.sort(key=riderno_key)
-    return u' '.join(rvec)
+    #rvec = list(set(ret.split()))
+    ##rvec.sort(key=riderno_key)	# don't lose ordering for seeds
+    #return u' '.join(rvec)
+    return ret
 
 def placeset(spec=u''):
     """Convert a place spec into an ordered set of place ints."""
@@ -480,11 +552,22 @@ def confopt_distunits(confstr):
     else:
         return 'metres' 
 
+def confopt_int(confstr, default=None):
+    """Check and return a valid integer."""
+    ret = default
+    try:
+        ret = int(confstr)
+    except:
+        pass	# ignore errors and fall back on default
+    return ret
+
 def confopt_posint(confstr, default=None):
     """Check and return a valid positive integer."""
     ret = default
     try:
         ret = int(confstr)
+        if ret < 0:
+            ret = default
     except:
         pass	# ignore errors and fall back on default
     return ret
@@ -585,6 +668,7 @@ def titlesplit(src=u'', linelen=24):
                 ret[line] += u' ' + word
     return ret
 
+
 class countback(object):
     __hash__ = None
     """Simple dict wrapper for countback store/compare."""
@@ -692,4 +776,11 @@ class countback(object):
                 ret = a > b
                 break
         return ret
+    def __add__(self, other):
+        """Add two countbacks together and return a new cb>=self >=other."""
+        if type(other) is not countback: return NotImplemented
+        ret = countback(str(self))
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
+            ret[i] += other[i]
+        return ret
 
