@@ -33,6 +33,8 @@ import socket
 import random
 import time
 import glib
+import json
+import zlib
 import metarace
 
 from metarace import unt4
@@ -55,11 +57,12 @@ TELEGRAPH_FAKEHOST=u'o'*IRC_FAKEHOSTLEN
 TELEGRAPH_CHANNEL=u'#agora'	# default channel
 TELEGRAPH_PRIVATESERVERS=[u'127.0.0.1',u'localhost',
                           u'192.168.95.64',u'192.168.96.64']
-TELEGRAPH_OPERNAME=u'metairc'
-TELEGRAPH_OPERPASS=u'ogfPOHYkaw'
+TELEGRAPH_OPERNAME=u'operator'
+TELEGRAPH_OPERPASS=u'password'
 
 # dispatch thread queue commands
-TCMDS = ('EXIT', 'PORT', 'MSG')
+# temp: RMSG is 'raw' message for new-style json commands
+TCMDS = ('EXIT', 'PORT', 'MSG', 'RMSG')
 
 # command read queue timeout value
 QUEUE_TIMEOUT = 2
@@ -112,7 +115,6 @@ class telegraph(threading.Thread):
     """
 
     ### GTK main thread methods
-
     def add_channel(self, channel):
         """Add channel to the set of joined channels on reconnect."""
         if channel:
@@ -128,10 +130,50 @@ class telegraph(threading.Thread):
         """Clear the specified line in DHI database."""
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),erl=True), chan)
 
+    def rcv_obj(self, mtxt):
+        """Unpack the provided message to python object."""
+        ## convenience function
+        ret = None
+        try:
+            ret = json.loads(mtxt)
+        except Exception as e:
+            self.log.error(u'Error receiving object: ' + repr(e))
+        return ret
+
+    def message_sig(self, msg):
+        """Return a 'signature' for supplied message."""
+        return str(zlib.adler32(msg) & 0xffffffff)
+
+    def send_rep(self, obj=None, chan=None):
+        """Pack the provided object into JSON and send."""
+        msg = None
+        try:
+            msg = json.dumps(obj, separators=(',',':'))
+            #self.log.info(u'dumped value is of type: ' + repr(type(msg)))
+            mlen = str(len(msg))
+            msig = self.message_sig(msg)  
+            self.queue.put_nowait(('RMSG',
+                                   ' '.join(['J', msig, mlen, msg]),
+                                   chan))
+        except Exception as e:
+            self.log.error(u'Error serialising report: ' + repr(e))
+        
+    def send_obj(self, hdr=None, obj=None, chan=None):
+        """Pack the provided object into JSON and send as a new style cmd."""
+        try:
+            self.send_cmd(hdr,json.dumps(obj),chan)
+        except Exception as e:
+            self.log.error(u'Error telegraphing object: ' + repr(e))
+
     def send_cmd(self, hdr=None, txt=None, chan=None):
         """Send the provided new style command."""
         self.sendmsg(unt4.unt4(header=hdr,text=txt), chan)
 
+    def send_cmdlist(self, cmdlist, chan=None):
+        """Send the provided new/old style command."""
+        encmsg = u' '.join(cmdlist).encode('utf-8','ignore')
+        self.queue.put_nowait(('RMSG', encmsg, chan))
+
     def set_title(self, line, chan=None):
         """Update the announcer's title line."""
         self.sendmsg(unt4.unt4(header='title',text=line), chan)
@@ -154,7 +196,8 @@ class telegraph(threading.Thread):
 
     def add_rider(self, rvec, header_txt='rider', chan=None):
         """Send a rider vector to the announcer."""
-        self.sendmsg(unt4.unt4(header=header_txt,text=chr(unt4.US).join(rvec)),chan)
+        self.sendmsg(unt4.unt4(header=header_txt,
+                               text=chr(unt4.US).join(rvec)),chan)
 
     def gfx_overlay(self, newov, chan=None):
         """Update graphic channel overlay."""
@@ -187,27 +230,24 @@ class telegraph(threading.Thread):
 
     def setline(self, line, msg, chan=None):
         """Set the specified DHI database line to msg."""
-        #msg = msg[0:self.linelen].ljust(self.linelen)
-        msg = msg + ' ' * 100
-        #msg = msg + ' ' * (self.linelen - len(msg))
+        msg = msg[0:self.linelen].ljust(self.linelen)
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg),chan)
 
     def flush(self):
         """Send a flush packet if possible (or required?)."""
+        ##TODO
         pass
 
     def linefill(self, line, char='_', chan=None):
         """Use char to fill the specified line."""
-        msg = char * 100
-        #msg = char * self.linelen
+        ## Why was this set to 100?
+        #msg = char * 100
+        msg = char * self.linelen
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg), chan)
 
     def postxt(self, line, oft, msg, chan=None):
         """Position msg at oft on line in DHI database."""
-        assert oft >= 0, 'Offset should be >= 0'
-        #if oft < self.linelen:
-        if True:
-            #msg = msg[0:(self.linelen-oft)]
+        if oft >= 0:
             self.sendmsg(unt4.unt4(xx=int(oft),yy=int(line),text=msg), chan)
 
     def setoverlay(self, newov, chan=None):
@@ -233,10 +273,6 @@ class telegraph(threading.Thread):
         """Suspend calling thread until cqueue is empty."""
         self.queue.join()
 
-    #def add_channel(self, addchan=''):
-        #"""Request additional channel."""
-        #self.queue.put_nowait(('ADDCHAN', addchan))
-
     def rejoin_channel(self, newchan=''):
         """Request another attempt to join channel."""
         self.queue.put_nowait(('CHAN', newchan))
@@ -296,7 +332,6 @@ class telegraph(threading.Thread):
         # If the server does not issue MODE, this is assumed to be an
         # arbitrartily long string, long enough to catch most cases.
         src = self.il.nm_to_n(e.source()).lower()
-        #src = e.source().lower()
         dest = e.target().lower()
         if src == self.nick.lower():
             self.srvid = e.source()
@@ -306,37 +341,86 @@ class telegraph(threading.Thread):
                              + ''.join(e.arguments()).decode('utf-8'))
 
     def msg_cb(self, c, e):
-        """Handle message in channel."""
+        """Handle privmsg."""
         source = self.il.nm_to_n(e.source()).lower()
         dest = e.target().lower()
-        body = unt4.decode(''.join(e.arguments()).decode('utf-8'))
-        #self.log.debug('Message from ' + repr(source) 
-                            #+ ', to: ' + repr(dest)
-                              #+ ' :: ' + repr(body))
+        body = ''.join(e.arguments())	# RCV bytes!
+        # determine receive stream and append to appropriate buffer
         if self.pub_cb is not None:
             # add buffer if required
             if source not in self.rdbuf:
                 self.rdbuf[source] = {}
             sbuf = self.rdbuf[source]
             if dest not in sbuf:
-                sbuf[dest] = u''
+                sbuf[dest] = ''
             sbuf[dest] += body	# append payload to relevant buffer
-            oidx = sbuf[dest].find(chr(unt4.SOH))
-            eidx = sbuf[dest].find(chr(unt4.EOT))
-            # collect all packets in buffer
-            while eidx >= 0:
-                if oidx >= 0 and eidx >= 0 and eidx > oidx:
-                    msgtxt = sbuf[dest][oidx:eidx+1]
-                    sbuf[dest] = sbuf[dest][eidx+1:]	# remove prefix
-                    glib.idle_add(self.pub_cb,
+            while len(sbuf[dest]) > 0:
+                # attempt to process current buffer content
+                if sbuf[dest][0] == 'J':
+                    # process the buffer as a JSON report
+                    joft = sbuf[dest].find('{')
+                    blen = len(sbuf[dest])
+                    if joft > 6:	# implies blen > 0
+                        # enough length for sig and buflen, start decode
+                        mvec = sbuf[dest].split(None, 3)
+                        if len(mvec) == 4:
+                            dlen = strops.confopt_posint(mvec[2])
+                            if len(mvec[3]) >= dlen:
+                                data = mvec[3][0:dlen]
+                                dsig = self.message_sig(data)
+                                if dsig == mvec[1]:
+                                    try:
+                                        o = json.loads(data)
+                                        glib.idle_add(self.pub_cb,o,source,dest)
+                                    except Exception as e:
+                                        self.log.error(u'Error reading ob: '
+                                                       + repr(e))
+                                else:
+                                    self.log.info(u'Invalid signature: '
+                                                  + repr(sbuf[dest]))
+                                nstart = joft+dlen
+                                sbuf[dest] = sbuf[dest][nstart:]
+                            else:
+                                # awaiting more data
+                                break
+                        else:
+                            self.log.info(u'Skipping malformed report packet: '
+                                           + repr(sbuf[dest]))
+                            sbuf[dest]=sbuf[dest][1:]	# chop off marker
+                            # terminal
+                    else:
+                        if len(sbuf[dest]) > 40:
+                            self.log.info(u'Skipping malformed report packet: '
+                                           + repr(sbuf[dest]))
+                        sbuf[dest]=sbuf[dest][1:]	# chop off marker
+                elif sbuf[dest][0] == '<':
+                    data = unt4.decode(sbuf[dest])
+                    # attempt to decode as UNT4 encoded list
+                    oidx = data.find(chr(unt4.SOH))
+                    eidx = data.find(chr(unt4.EOT))
+                    # process one chunk and then unravel
+                    if oidx >= 0 and eidx >= 0 and eidx > oidx:
+                        msgtxt = data[oidx:eidx+1]
+                        glib.idle_add(self.pub_cb,
                             unt4.unt4(unt4str=msgtxt.decode('utf-8','replace')),
                             source, dest)
-                elif eidx >= 0:	# discard partial message (not complete)
-                    sbuf[dest] = sbuf[dest][eidx+1:]
-                oidx = sbuf[dest].find(chr(unt4.SOH))
-                eidx = sbuf[dest].find(chr(unt4.EOT))
+                        datlen = len(unt4.encode(data[0:eidx+1]))
+                        sbuf[dest] = sbuf[dest][datlen:]
+                    elif eidx >= 0: # discard partial msg (not yet complete)
+                        datlen = len(unt4.encode(data[0:eidx+1]))
+                        sbuf[dest] = sbuf[dest][datlen:]
+                    else:
+                        # awaiting more data
+                        break
+                else:
+                    # assume buffer contains a complete utf8 command list
+                    glib.idle_add(self.pub_cb,
+                                  sbuf[dest].decode('utf-8','ignore').split(),
+                                  source,dest)
+                    sbuf[dest] = ''	# truncate buffer
+
         
-    def __init__(self, linelen=32):
+    def __init__(self, linelen=28):
         """Constructor."""
         threading.Thread.__init__(self) 
         self.running = False
@@ -350,7 +434,7 @@ class telegraph(threading.Thread):
 
         ## HACK
         self.linelen = linelen
-        self.pagelen = 8
+        self.pagelen = 7
         self.encoding = 'utf-8'
 
 
@@ -403,6 +487,9 @@ class telegraph(threading.Thread):
             self.opername = metarace.sysconf.get(u'telegraph', u'opername')
         if metarace.sysconf.has_option(u'telegraph', u'operpass'):
             self.operpass = metarace.sysconf.get(u'telegraph', u'operpass')
+        if metarace.sysconf.has_option(u'sender', u'pagelen'):
+            self.pagelen = strops.confopt_posint(metarace.sysconf.get(
+                                  u'sender', u'pagelen'),self.pagelen)
         if metarace.sysconf.has_option(u'telegraph', u'ping_interval'):
             ck = tod.str2tod(metarace.sysconf.get(u'telegraph',
                                                     u'ping_interval'))
@@ -504,14 +591,12 @@ class telegraph(threading.Thread):
         """Output the message in as many blocks as are required."""
         # :NICK!USER@SPOOF PRIVMSG #CHAN :payload \r\n
         maxlen = IRC_MAXMSGLEN - (IRC_PRIVHDRLEN + len(chan) + len(self.srvid))
-        #self.log.debug(u'Truncating with: ' + repr(chan)
-                       #+ u',' + repr(self.srvid))
         pos = 0
         while pos < len(msg):
             self.ic.privmsg(chan, msg[pos:pos+maxlen])
             pos += maxlen
         if not self.localsrv:
-            # avoid channel flooding by sleep here
+            # try to avoid channel flooding by sleep here
             time.sleep(0.3+0.8*random.random())
 
     def run(self):
@@ -549,7 +634,13 @@ class telegraph(threading.Thread):
                 while True:
                     m = self.queue.get(timeout=QUEUE_TIMEOUT)
                     self.queue.task_done()
-                    if m[0] == 'MSG' and self.host not in [None,
+                    if m[0] == 'RMSG' and self.host not in [None,
+                                                   '', 'none', 'NULL']:
+                        chan = self.channel
+                        if len(m) > 2 and m[2]:
+                            chan = m[2]
+                        self.__streammsg(chan,m[1])
+                    elif m[0] == 'MSG' and self.host not in [None,
                                                    '', 'none', 'NULL']:
                         chan = self.channel
                         if len(m) > 2 and m[2]:
@@ -596,8 +687,8 @@ class fakeirc(object):
     def disconnect(self):
         pass
 
-    def connect(self, host=None, port=None, nick=None, data=None):
-        """Fake an IOError to effectively shut down object."""
+    def connect(self, server=None, username=None, host=None, port=None, nickname=None, ircname=None, nick=None, data=None):
+        """Fake an IOError to shut down object."""
         raise IOError('IRC library not present.')
 
     def close(self, data=None):
@@ -629,24 +720,41 @@ if __name__ == '__main__':
         te.set_portstr(force=True)
         return False
 
-    def delaycmd(te):
-        print(u'Delayed cmd.')
+    cvec = []
+    for i in range(0,18):
+        cvec.append(i)
 
-    def msgcb(cmd, nick, chan):
-        print(u'Telegraph: ' + repr(nick) + u'/' + repr(chan)
-                       + u' :: ' + repr(cmd.pack()))
-        return False
+    def delaycmd(te):
+        obj = {u'type':u'test',
+               u'xoft':0,
+               u'yoft':10,
+               u'len':1024,
+               u'compt': cvec,
+               u'string':u' pksdjh sdkjh sfkjh sfjkhsf kj sfh jksfh jksf hksf hjksf hk sfhjk sfhjk sfhjk sfhjk sfhjklain old unicode \u2764',
+               u'vector':[u'asdfghfdsasdfghgfdsasdfghgfdsasdfghfdsasdf',
+                          u'543wu8s0du09ns', u'qw9idpoudpiuoiwed',
+                          u'tyhgfdsertyhjnfderthvdsthjhgfdsrtgh',
+                          u'2tyujnvdsertyjnhgoidfsjfogijfsdoihsodighusf',
+                          0,1,2,3,4],
+               u'hash':{u'a':u'adfg',u'b':u'ire'},
+               u'Null':None}
+        te.send_rep(obj)
 
     logging.basicConfig()
     metarace.init()
     t = telegraph()
+    def msgcb(cmd, nick, chan):
+        print(u'Telegraph: ' + repr(nick) + u'/' + repr(chan)
+                       + u' :: ' + repr(type(cmd)) + u'/' + repr(cmd))
+        return False
+
     #t.debug = True
     try:
         t.start()
         t.add_channel('#otherone')
         t.set_portstr(force=True)
         t.set_pub_cb(msgcb)
-        glib.timeout_add_seconds(40, delaycmd, t)
+        glib.timeout_add_seconds(15, delaycmd, t)
         metarace.mainloop()
     except:
         t.exit()
