@@ -15,13 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
-## TODO : Replay from file as per the thbc module
-##        complete case testing
-##        complete serial testing
-##        check arm/armlock/callback in main loop
-##        repair trackmeet integration
-
 """Alge Timy I/O helper.
 
 This module provides a thread object which interfaces with an
@@ -83,15 +76,15 @@ from metarace import tod
 from metarace import strops
 
 # System default timy serial port
-DEFPORT = u'/dev/ttyUSB0'
-TIMYPORT = u'/dev/ttyUSB0'
+DEFPORT = u'/dev/ttyS0'
+TIMYPORT = u'/dev/ttyS0'
 MAINPORT = u'/dev/ttyUSB0'
 BACKUPPORT = u'/dev/ttyUSB1'
 ENCODING = 'cp437'	# Timy serial interface encoding
 
 # TIMY serial baudrate
 TIMY_BAUD = 38400	# baudrate
-TIMY_CTSRTS = True	# hardware flow override in default config
+TIMY_CTSRTS = False	# hardware flow override in default config
 
 # thread queue commands -> private to timy thread
 TCMDS = ('EXIT', 'PORT', 'MSG', 'TRIG', 'RCLR')
@@ -114,14 +107,24 @@ CHAN_UNKNOWN = -1
 LAYOUTS = {
 	'DISC':{
 		'len':250.0,
-		'chans':[CHAN_FINISH, CHAN_200, CHAN_PB, CHAN_100, CHAN_PA],
+		'chans':[CHAN_FINISH, CHAN_200, 6, CHAN_PB, CHAN_100,
+                         7, 8, CHAN_PA],
 		'offsets':{CHAN_FINISH:0.0,
                            CHAN_200:50.0,
+                           6:100.0,		# 50m split
                            CHAN_PB:112.5,
                            CHAN_100:150.0,
+                           7:175.0,		# quarter lap split
+                           8:200.0,		# 150m split
                            CHAN_PA:237.5}
 		}
-	}
+}
+
+CHANDELAY = {
+    0:tod.ZERO, 1:tod.ZERO, 2:tod.ZERO, 3:tod.ZERO,
+    4:tod.ZERO, 5:tod.ZERO, 6:tod.ZERO, 7:tod.ZERO,
+    8:tod.ZERO, 9:tod.ZERO
+}
 
 CR = chr(0x0d)
 LF = chr(0x0a)
@@ -172,7 +175,7 @@ def chan2id(chanstr='0'):
     """Return a channel ID for the provided string, without fail."""
     ret = CHAN_UNKNOWN
     if (type(chanstr) in [unicode, str] and len(chanstr) > 1
-        and chanstr[0] == u'C' and chanstr[1].isdigit()):
+        and chanstr[0].upper() == u'C' and chanstr[1].isdigit()):
         ret = int(chanstr[1])
     else:
         try:
@@ -220,6 +223,9 @@ class timy(threading.Thread):
         self.setcb()	# init but allow overwrite after loadconf
         if port is not None:
             self.setport(port)
+        self.chandelay = {}
+        for c in CHANDELAY:
+            self.chandelay[c] = CHANDELAY[c]
 
     def __defcallback(self, evt=None):
         """Default callback is a tod log entry."""
@@ -351,8 +357,8 @@ class timy(threading.Thread):
 	    PRE4		- 10,000th sec precision
 	    RR0			- Round by 'cut'
 	    BE1			- Beep on
-	    DTS01.00		- Start delay 1.0
-	    DTF01.00		- Finish & intermediate delay 1.0
+	    DTS02.00		- Start delay 2.0
+	    DTF02.00		- Finish & intermediate delay 2.0
 	    EMU0		- Running time off
 	    PRINTER0		- Printer off
 	    PRIIGN1		- Don't print all impulses to receipt
@@ -374,7 +380,7 @@ class timy(threading.Thread):
 
         """
         for msg in ['TIMYINIT', 'NSF', 'PROG?', 'KL0', 'CHK1', 'PRE4',
-                    'RR0', 'BE1', 'DTS01.00', 'DTF01.00', 'EMU0',
+                    'RR0', 'BE1', 'DTS02.00', 'DTF02.00', 'EMU0',
                     'PRINTER0', 'PRIIGN1',
                     'DTPMetarace ' + metarace.VERSION, 'PRILF']:
             self.write(msg)
@@ -414,6 +420,7 @@ class timy(threading.Thread):
         msg = msg.rstrip()	# remove cr/lf if present
         tsum = 0
         csum = 0
+        #ltime = tod.now()
         if len(msg) == 28:
             # assume checksum present, grab it and truncate msg
             tsum = timy_getsum(msg[26:28])
@@ -424,8 +431,15 @@ class timy(threading.Thread):
             if tsum == csum:
                 e = msg.split()
                 if len(e) == 4:
-                    ret = tod.tod(timeval = e[2], index = e[0], chan = e[1],
-                                                  source = self.name)
+                    cid = chan2id(e[1])
+                    iv = tod.str2tod(e[2])
+                    if iv is not None and cid in self.chandelay:
+                        tv = self.chandelay[cid] + iv
+                        ret = tod.tod(timeval=tv.timeval, index=e[0],
+                                      chan=e[1], source=self.name) #,
+                                      # ltime=ltime.timeval)
+                    else:
+                        self.log.error(u'Invalid message: ' + repr(msg))
                 else:
                     self.log.error(u'Invalid message: ' + repr(msg))
             else:
@@ -498,9 +512,10 @@ class timy(threading.Thread):
             ch = self.port.read(1)
 
     def run(self):
-        """Called via threading.Thread.start()."""
         running = True
         self.log.debug(u'Starting')
+
+        # re-read configs from sysconf
         baudrate = TIMY_BAUD
         if metarace.sysconf.has_option(u'timy', u'baudrate'):
             baudrate = strops.confopt_posint(metarace.sysconf.get(u'timy',
@@ -509,6 +524,16 @@ class timy(threading.Thread):
         if metarace.sysconf.has_option(u'timy', u'ctsrts'):
             ctsrts = strops.confopt_bool(metarace.sysconf.get(u'timy',
                                                u'ctsrts'))
+        if metarace.sysconf.has_option(u'timy', u'chandelay'):
+            nd = metarace.sysconf.get(u'timy', u'chandelay')
+            for cv in nd:
+                c = chan2id(cv)
+                if c in self.chandelay:
+                    nv = tod.str2tod(nd[cv])
+                    self.chandelay[c] = nv
+                    self.log.debug(u'Set channel delay ' + repr(c) + 
+                                   u' : ' + nv.rawtime(4))
+
         while running:
             try:
                 # Read phase
@@ -522,7 +547,7 @@ class timy(threading.Thread):
                 
                 # Write phase
                 if type(m) is tuple and type(m[0]) is str and m[0] in TCMDS:
-                    if m[0] == 'MSG' and self.port is not None and not self.error:
+                    if m[0]=='MSG' and self.port is not None and not self.error:
                         self.log.debug(u'Sending rawmsg ' + repr(m[1]))
                         self.port.write(m[1].encode(ENCODING,'replace'))
                     elif m[0] == 'TRIG':
