@@ -36,7 +36,7 @@ A time of day object includes:
 
    - timeval : decimal tod in seconds (eg 1.2345, 4506.9023, etc)
    - index   : 4 character identifier string (eg '1' to '9999')
-   - chan    : 3 character channel string from source (eg 'C0', 'C2M', etc)
+   - chan    : channel string from source (eg 'C0', 'C2M', etc)
    - refid   : string reference id, used for RFID tag events (eg '75ae7f')
    - source  : string identifier for timer that sourced the event (eg 'main')
 
@@ -126,6 +126,16 @@ TOD_RE=re.compile(r'^(?:(?:(\d{1,2})[h:-])?(\d{1,2})[:\'-])?(\d{1,2}(?:[\.\"]\d+
 # assume no run over change
 DSTHACK = time.timezone - time.localtime().tm_isdst * 3600
 
+def str2agg(timeval=u''):
+    """Return agg for given string without fail."""
+    ret = None
+    if timeval is not None and timeval != u'':
+        try:
+            ret = agg(timeval)
+        except:
+            pass
+    return ret
+
 def str2tod(timeval=u''):
     """Return tod for given string without fail."""
     ret = None
@@ -136,6 +146,21 @@ def str2tod(timeval=u''):
             pass
     return ret
 
+def dec2hm(dectod=None):
+    """Return hours and minutes only."""
+    strtod = None
+    if dectod is not None: 		# conditional here?
+        if dectod >= 3600:	# NOTE: equal compares fine w/decimal
+            fmt = u'{0}:{1:02}' # 'HH:MM'
+            strtod = fmt.format(int(dectod)//3600,
+                (int(dectod)%3600)//60)
+        elif dectod >= 60:	# weird.. minutes only
+            strtod = u'{0}'.format(int(dectod)//60)
+        else:
+            strtod = u''	# No minutes to convert?
+    return strtod
+
+
 def dec2str(dectod=None, places=4, zeros=False, hoursep=u'h', minsep=u':'):
     """Return formatted string for given tod decimal value.
 
@@ -153,8 +178,7 @@ def dec2str(dectod=None, places=4, zeros=False, hoursep=u'h', minsep=u':'):
 
     """
     strtod = None
-
-    assert places >= 0 and places <= 5, 'places not in range [0, 5]'
+    #assert places >= 0 and places <= 5, 'places not in range [0, 5]'
     
     if dectod is not None: 		# conditional here?
         if zeros or dectod >= 3600:	# NOTE: equal compares fine w/decimal
@@ -223,13 +247,15 @@ class tod(object):
         chan -- channel string
         refid -- a reference identifier string
         source -- timer source identifier or u'' if not sourced
+        ltime -- wall clock when received
 
         """
 
         self.index = index[0:4]	# numeric raises exception here
-        self.chan = chan[0:3]
+        self.chan = chan ## chan[0:3]
         self.refid = refid
         self.source = source
+        self.ltime = None
         if type(timeval) in [unicode, str]:
             self.timeval = str2dec(timeval)
         elif type(timeval) is tod:
@@ -248,8 +274,8 @@ class tod(object):
 
     def __repr__(self):
         """Return object representation string."""
-        return "tod({0}, {1}, {2}, {3})".format(repr(self.timeval),
-            repr(self.index), repr(self.chan), repr(self.refid))
+        return "tod({0}, {1}, {2}, {3}, {4})".format(repr(self.timeval),
+            repr(self.index), repr(self.chan), repr(self.refid), repr(self.source))
 
     def refstr(self, places=4):
         """Return 'normalised' string form.
@@ -296,15 +322,22 @@ class tod(object):
                                              hoursep, minsep),
             QUANT_TWID[places], QUANT_OPAD[places])
 
-    def meridian(self):
+    def meridian(self, mstr=None, secs=True):
         """Return a 12hour time of day with meridian."""
+        ret = None
         med = u'am'
         if self.timeval >= 43200:
             med = u'pm'
+        if mstr is not None:
+            med = mstr
         tv = self.timeval % 43200
         if tv < 3600:
             tv += 43200	# 12am/12pm 
-        return dec2str(tv, 0, hoursep=u':', minsep=u':') + med
+        if secs:
+            ret = dec2str(tv, 0, hoursep=u':', minsep=u':') + med
+        else:
+            ret = dec2hm(tv) + med
+        return ret
 
     def rawtime(self, places=4, zeros=False, hoursep=u'h', minsep=u':'):
         """Return time string component of the tod, without padding."""
@@ -408,7 +441,7 @@ class agg(tod):
     """
     def __init__(self, timeval=0, index=u'', chan=u'', refid=u'', source=u''):
         self.index = index[0:4]
-        self.chan = chan[0:3]
+        self.chan = chan ## chan[0:3]
         self.refid = refid
         self.source = source	# not really relevant in agg
         if type(timeval) in [unicode, str]:
@@ -447,6 +480,7 @@ class agg(tod):
 # ToD 'constants'
 ZERO = tod()			# common cases
 ONE = tod(u'1.0')
+MINUTE = tod(u'1:00')
 MAXELAP = tod(u'23h30:00')	# max displayed elapsed time
 MAX = tod(u'23h59:59.9999')	# largest val possible
 
@@ -454,6 +488,7 @@ MAX = tod(u'23h59:59.9999')	# largest val possible
 # these are impossible tods that still sort correctly
 FAKETIMES = {
  u'catch':ZERO,
+ u'w/o':ZERO,
  u'max':MAX.copy(),
  u'caught':MAX.copy(),
  u'rel':MAX.copy(),	# last place -> checka
@@ -468,6 +503,13 @@ for c in [u'caught', u'rel', u'abort', u'otl', u'dsq', u'dnf', u'dns']:
     FAKETIMES[c].timeval += cof
     cof += extra
 
+## Convenience functions for speed conversions
+def dr2t(dist, rate):
+    """Convert distance (m) and rate (km/h) to time."""
+    d = float(dist)
+    r = float(rate)/3.6
+    return tod(unicode(d/r))
+
 class todlist():
     """ToD list helper class for managing splits and ranks."""
     def __init__(self, lbl=u''):
@@ -487,13 +529,15 @@ class todlist():
         """Return current 0-based rank for given bib."""
         ret = None
         i = 0
+        r = 0
         last = None
         for lt in self.__store:
             if last is not None:
                 if lt != last:
-                    i += 1
+                    r = i
+            i += 1
             if lt.refid == bib and lt.index == series:
-                ret = i
+                ret = r
                 break
             last = lt
         return ret
@@ -509,6 +553,15 @@ class todlist():
             else:
                 i += 1
 
+    def remove_first(self, bib, series=u''):
+        i = 0
+        while i < len(self.__store):
+            if self.__store[i].refid==bib and self.__store[i].index==series:
+                del self.__store[i]
+                break
+            else:
+                i += 1
+
     def insert(self, t, bib=None, series=u''):
         """Insert t into ordered list."""
         ret = None
