@@ -29,6 +29,8 @@ import sys
 import logging
 ## HACK
 import socket
+import threading
+import time
 
 import metarace
 
@@ -52,18 +54,21 @@ from metarace import ps
 from metarace import ittt
 from metarace import f200
 from metarace import flap
+from metarace import hour
 from metarace import sprnd
+#from metarace import spfin
 from metarace import omnium
-from metarace import hotlap
+#from metarace import hotlap
 from metarace import classification
 from metarace import printing
 from metarace import uiutil
 
 LOGFILE = u'event.log'
-LOGHANDLER_LEVEL = logging.DEBUG
+LOGHANDLER_LEVEL = logging.INFO	# check 
 CONFIGFILE = u'config.json'
-TRACKMEET_ID = u'trackmeet_1.8'	# configuration versioning
+TRACKMEET_ID = u'trackmeet_1.9'	# configuration versioning
 EXPORTPATH = u'export'
+MAXREP = 10000	# communique max number
 
 def mkrace(meet, event, ui=True):
     """Return a race object of the correct type."""
@@ -85,8 +90,12 @@ def mkrace(meet, event, ui=True):
         ret = flap.flap(meet, event, ui)
     elif etype in [u'flying 200']:
         ret = f200.f200(meet, event, ui)
+    elif etype in [u'hour']:
+        ret = hour.hourrec(meet, event, ui)
     elif etype in [u'sprint round', 'sprint final']:
         ret = sprnd.sprnd(meet, event, ui)
+    #elif etype in ['sprint final']:
+        #ret = spfin.spfin(meet, event, ui)
     else:
         ret = race.race(meet, event, ui)
     return ret
@@ -127,7 +136,7 @@ class trackmeet:
         """Display meet information on scoreboard."""
         self.gemini.clear()
         self.clock.clicked()
-        self.announce.gfx_overlay(0, self.graphicscb)
+        self.announce.gfx_overlay(2, self.graphicscb)
 
     def menu_meet_properties_cb(self, menuitem, data=None):
         """Edit meet properties."""
@@ -289,11 +298,14 @@ class trackmeet:
             lb = u''
             lt = []
             if self.mirrorpath:
-                lb = os.path.join(u'/site', self.mirrorpath, exportfile)
+                lb = os.path.join(self.linkbase, self.mirrorpath, exportfile)
                 lt = [u'pdf', u'xls', u'json']
             ofile = os.path.join(self.exportpath, exportfile+u'.txt')
             with open(ofile, 'wb') as f:
                 rep.output_text(f, linkbase=lb, linktypes=lt)
+            ofile = os.path.join(self.exportpath, exportfile+u'.html')
+            with open(ofile, 'wb') as f:
+                rep.output_html(f, linkbase=lb, linktypes=lt)
         
         if not doprint:
             return False
@@ -336,6 +348,32 @@ class trackmeet:
             self.printprefs = dlg.get_print_settings()
             self.log.info(u'Updated print preferences.')
 
+    def find_communique(self, lookup):
+        """Find or allocate a communique number."""
+        ret = None
+        cnt = 1
+        noset = set()
+        for c in self.commalloc:
+            if c == lookup:	# previous allocation
+                ret = self.commalloc[c]
+                self.log.debug(u'Found allocation: ' + ret + u' -> ' + lookup)
+                break
+            else:
+                noset.add(self.commalloc[c])
+        if ret is None:		# not yet allocated
+            while True:
+                ret = unicode(cnt)
+                if ret not in noset:
+                    self.commalloc[lookup] = ret	# write back
+                    self.log.debug(u'Add allocation: ' + ret + u' -> ' + lookup)
+                    break
+                else:
+                    cnt += 1
+                    if cnt > MAXREP:
+                        self.log.error(u'Gave up looking for communique no.')
+                        break	# safer
+        return ret
+
     ## Event action callbacks
     def eventdb_cb(self, evlist, reptype=None):
         """Make a report containing start lists for the events listed."""
@@ -344,8 +382,13 @@ class trackmeet:
         #       allows for creation of reports manually crafted.
         secs = []
         reptypestr = reptype.title()
+        lsess = None
         for eno in evlist:
             e = self.edb[eno]
+            nsess = e['sess']
+            if nsess != lsess and lsess is not None:
+                secs.append(printing.pagebreak(0.1))
+            lsess = nsess
             h = mkrace(self, e, False)
             h.loadconfig()
             if reptype == 'startlist':
@@ -363,7 +406,7 @@ class trackmeet:
             h.destroy()
             secs.append(printing.pagebreak())
         if len(secs) > 0:
-            self.log.debug(reptype + u' callback on events: ' + u', '.join(evlist))
+            reporthash = reptype + u', '.join(evlist)
             if self.communiques:	# prompt for communique no
                 #commno = uiutil.communique_dialog(self.meet.window)
 
@@ -373,13 +416,18 @@ class trackmeet:
                                    stxt='', ftxt='')
                 gtk.gdk.threads_leave()                                                     
                 if len(rvec) > 1 and rvec[0] == 1:
-                    if rvec[1]:
+                    commno = self.find_communique(reporthash)
+                    if rvec[1]:	# it's a revision
+                        commno += rvec[1]
+                    if commno is not None:
                         reptypestr = (u'Communiqu\u00e9 '
-                                   + rvec[1] + ': ' + reptypestr)
+                                   + commno + ': ' + reptypestr)
                     if rvec[2]:
-                        ## TODO:
-                        #secs.append(sigbox)
-                        pass
+                        msgsec = printing.bullet_text()
+                        msgsec.subheading = u'Revision ' + repr(rvec[1])
+                        msgsec.lines.append([u'',rvec[2]])
+                        ## signature
+                        secs.append(msgsec)
             self.print_report(secs, docstr=reptypestr, exportfile=u'trackmeet_' + reptype)
 				# no need to export file now, since deadlock
 				# resolved with async print op
@@ -430,6 +478,21 @@ class trackmeet:
                                               eh[u'pref'], eh[u'info'])
             self.scbwin.reset()
             self.curevent.delayed_announce()
+            self.announce.gfx_overlay(1, self.graphicscb)
+            einf = {}
+            for key in eh:
+                einf[key] = eh[key]
+
+            self.announce.send_obj(u'gfxobj',{u'overlay':u'gfxinfo',
+                                    u'evid':eh[u'evid'],
+                                    u'einf':einf,
+                                    u'pref':eh[u'pref'],
+                                    u'info':eh[u'info'],
+                                    u'prog':eh[u'prog'],
+                               
+                                    u'title':self.titlestr,
+                                    u'subtitle':self.subtitlestr
+                                   }, self.graphicscb)
 
     def menu_race_properties_activate_cb(self, menuitem, data=None):
         """Edit properties of open race if possible."""
@@ -524,6 +587,44 @@ class trackmeet:
             else:                    
                 race.addrider(st)
 
+    def autoplace_riders(self, race, autospec=u'', infocol=None):
+        """Try to fetch the startlist from race result info."""
+        places = {}
+        for egroup in autospec.split(u';'):
+            self.log.debug(u'Autospec group: ' + repr(egroup))
+            specvec = egroup.split(u':')
+            if len(specvec) == 2:
+                evno = specvec[0].strip()
+                if evno not in self.autorecurse:
+                    self.autorecurse.add(evno)
+                    placeset = strops.placeset(specvec[1])
+                    e = self.edb[evno]
+                    if e is not None:
+                        h = mkrace(self, e, False)
+                        h.loadconfig()
+                        for ri in h.result_gen():
+                            if type(ri[1]) is int and ri[1] in placeset:
+                                rank = ri[1]
+                                if rank not in places:
+                                    places[rank] = []
+                                places[rank].append(ri[0])
+                        h.destroy()
+                    else:
+                        self.log.warn(u'Autospec event number not found: '
+                                        + repr(evno))
+                    self.autorecurse.remove(evno)
+                else:
+                    self.log.debug(u'Ignoring loop in auto startlist: '
+                                   + repr(evno))
+            else:
+                self.log.warn(u'Ignoring erroneous autospec group: '
+                               + repr(egroup))
+        ret = ''
+        for place in sorted(places):
+            ret += ' ' + '-'.join(places[place])
+        self.log.debug(u'Place set: ' + ret)
+        return ret
+
     def autostart_riders(self, race, autospec=u'', infocol=None):
         """Try to fetch the startlist from race result info."""
         # Dubious: infocol allows selection of seed info
@@ -577,6 +678,7 @@ class trackmeet:
         """Close the currently opened race."""
         if self.curevent is not None:
             self.timer.setcb()
+            self.rftimer.setcb()
             self.curevent.hide()
             self.race_box.remove(self.curevent.frame)
             self.curevent.destroy()
@@ -589,6 +691,9 @@ class trackmeet:
 
     def race_evno_change(self, old_no, new_no):
         """Handle a change in a race number."""
+        if self.curevent is not None and self.curevent.evno == old_no:
+            self.log.warn(u'Ignoring change to open event: ' + repr(old_no))
+            return False
         oldconf = self.event_configfile(old_no)
         if os.path.isfile(oldconf):
             newconf = self.event_configfile(new_no)
@@ -596,15 +701,36 @@ class trackmeet:
                 os.rename(newconf, newconf + u'.old')
             os.rename(oldconf, newconf)
         self.log.info(u'Race ' + repr(old_no) + u' changed to ' + repr(new_no))
+        return True
 
     ## Data menu callbacks.
     def menu_data_rego_activate_cb(self, menuitem, data=None):
-        """Open rider registration dialog."""
-        self.log.warn(u'TODO :: Rider registration dlg...')
+        """Open rider rego dialog."""
+        self.log.warn(u'TODO :: Rider rego dlg...')
 
     def menu_data_import_activate_cb(self, menuitem, data=None):
-        """Open rider import dialog."""
-        self.log.warn(u'TODO :: Rider import dlg...')
+        """Re-load event and rider info from disk."""
+        if not uiutil.questiondlg(self.window,
+            u'Re-load event and rider data from disk?',
+            u'Note: The current event will be closed.'):
+            self.log.debug(u'Re-load events & riders aborted.')
+            return False
+
+        cureventno = None
+        if self.curevent is not None:
+            cureventno = self.curevent.evno
+            self.close_event()
+
+        self.rdb.clear()
+        self.edb.clear()
+        self.edb.load(os.path.join(self.configpath, u'events.csv'))
+        self.rdb.load(os.path.join(self.configpath, u'riders.csv'))
+        self.reload_riders()
+
+        if cureventno and cureventno in self.edb:
+            self.open_event(self.edb[cureventno])
+        else:
+            self.log.warn(u'Running event was removed from the event list.')
 
     def menu_data_result_activate_cb(self, menuitem, data=None):
         """Export final result."""
@@ -617,9 +743,15 @@ class trackmeet:
     def finalresult(self):
         provisional = self.provisional	# may be overridden below
         sections = []
+        lastsess = None
         for e in self.edb:
             r = mkrace(self, e, False)
             if e[u'resu']:    # include in result
+                nsess = e[u'sess']
+                if nsess != lastsess:
+                    sections.append(printing.pagebreak(0.10)) # force break
+                    self.log.debug(u'Break between events: ' + repr(e[u'evid']) + u' with ' + repr(lastsess) + u' != ' + repr(nsess))
+                lastsess = nsess
                 if r.evtype in [u'break', u'session']:
                     sec = printing.section()
                     sec.heading = u' '.join([e[u'pref'], e[u'info']]).strip()
@@ -726,13 +858,8 @@ class trackmeet:
         s.heading = 'Abbreviations'
         s.lines = tlist
         if len(s.lines) > 0:
-            r.add_section(s)
-
-        # Add live result info
-        s = printing.bullet_text()
-        s.heading = 'Startlists, Results & Standings'
-        s.lines.append([None, u'Live results, startlists and standings available online at: http://metarace.com.au/'])
-        r.add_section(s)
+            pass
+            #r.add_section(s)
 
         filebase = os.path.basename(self.configpath)
         if filebase in [u'', u'.']:
@@ -747,6 +874,10 @@ class trackmeet:
         with open(ofile, 'wb') as f:
             r.output_text(f)
             self.log.info(u'Exported text program to ' + repr(ofile))
+        ofile = os.path.join(self.configpath, u'export', filebase + u'.html')
+        with open(ofile, 'wb') as f:
+            r.output_html(f)
+            self.log.info(u'Exported html program to ' + repr(ofile))
         ofile = os.path.join(self.configpath, u'export', filebase + u'.xls')
         with open(ofile, 'wb') as f:
             r.output_xls(f)
@@ -773,7 +904,32 @@ class trackmeet:
             self.log.error(u'Error exporting event data: ' + unicode(e))
             raise
 
+    def updatenexprev(self):
+        self.nextlinks = {}
+        self.prevlinks = {}
+        prev = None
+        prevl = None
+        for eh in self.edb:
+            if eh[u'inde'] or eh[u'resu']:     # include in index?
+                evno = eh[u'evid']
+                referno = None
+                if eh[u'type'] not in [u'break', u'session']:
+                    referno = evno
+                if eh[u'refe']: # overwrite ref no, even on specials
+                    referno = eh[u'refe']
+                linkfile = None
+                if referno:
+                    linkfile = u'event_' + unicode(referno).translate(
+                                                   strops.WEBFILE_UTRANS)
+                    if prev is not None:
+                        self.prevlinks[evno] = prevl
+                        self.nextlinks[prev] = linkfile
+                    prev = evno
+                    prevl = linkfile
+
     def updateindex(self):
+            self.reload_riders()	# force update from rdb !HACK
+            self.updatenexprev()	# re-compute next/prev link struct
             dirty = {}
             ## riders
             self.db.execute(u'Delete FROM Cycling WHERE TypeID=%s',
@@ -843,11 +999,22 @@ class trackmeet:
                     pfilebase += u'_program'
                 pfile = os.path.join(self.configpath, u'export',
                                          pfilebase + u'.pdf')
+                rfilebase = os.path.basename(self.configpath)
+                if rfilebase in [u'', u'.']:
+                    rfilebase = u'result'
+                else:
+                    rfilebase += u'_result'
+                rfile = os.path.join(self.configpath, u'export',
+                                         rfilebase + u'.pdf')
+
                 lt = []
                 lb = None
-                if os.path.exists(pfile):
+                if os.path.exists(rfile):
                     lt = [u'pdf', u'xls', u'txt', u'json']
-                    lb = os.path.join(u'/site', self.mirrorpath, pfilebase)
+                    lb = os.path.join(self.linkbase, self.mirrorpath, rfilebase)
+                elif os.path.exists(pfile):
+                    lt = [u'pdf', u'xls', u'txt', u'json']
+                    lb = os.path.join(self.linkbase, self.mirrorpath, pfilebase)
 
                 sec = printing.event_index(u'eventindex')
                 sec.heading = 'Index of Events'
@@ -873,6 +1040,9 @@ class trackmeet:
                 ofile = os.path.join(self.exportpath, basename + u'.txt')
                 with open(ofile, 'wb') as f:
                     orep.output_text(f, linkbase=lb, linktypes=lt)
+                ofile = os.path.join(self.exportpath, basename + u'.html')
+                with open(ofile, 'wb') as f:
+                    orep.output_html(f, linkbase=lb, linktypes=lt)
                 jbase = basename + u'.json'
                 jfile = os.path.join(u'/site', self.mirrorpath, jbase)
                 ofile = os.path.join(self.exportpath, jbase)
@@ -929,8 +1099,23 @@ class trackmeet:
 
     def menu_data_export_activate_cb(self, menuitem, data=None):
         """Export race data."""
+        if not self.exportlock.acquire(False):
+            self.log.info(u'Export already in progress.')
+            return None # allow only one entry
+        if self.exporter is not None:
+            self.log.warn(u'Export in progress, re-run required.')
+            return False
+        try:
+            self.exporter = threading.Thread(target=self.__run_data_export)
+            self.exporter.start()
+            self.log.info(u'Created export worker : ' + repr(self.exporter))
+        finally:
+            self.exportlock.release()
+ 
+    def __run_data_export(self):
         try:
             self.log.debug('Exporting race info.')
+            self.updatenexprev()	# re-compute next/prev link struct
 
             # determine 'dirty' events 	## TODO !!
             dmap = {}
@@ -1069,7 +1254,14 @@ class trackmeet:
                         orep.reportstatus = u'provisional'
                     else:
                         orep.reportstatus = u'final'
-                    orep.indexlink = 'program'	# url to program of events
+
+                    # in page links
+                    orep.indexlink = u'program'	# url to program of events
+                    if evno in self.prevlinks:
+                        orep.prevlink = self.prevlinks[evno]
+                    if evno in self.nextlinks:
+                        orep.nextlink = self.nextlinks[evno]
+
                     # update files and trigger mirror
                     if r.onestart:	# output result
                         outsec = r.result_report()
@@ -1083,14 +1275,18 @@ class trackmeet:
                                                    strops.WEBFILE_UTRANS)
                     ofile = os.path.join(self.exportpath, basename + u'.txt')
                     with open(ofile, 'wb') as f:
-                        orep.output_text(f)	# with links -> not yet
+                        orep.output_text(f)
+                    ofile = os.path.join(self.exportpath, basename + u'.html')
+                    with open(ofile, 'wb') as f:
+                        orep.output_html(f)
                     jbase = basename + u'.json'
                     jfile = os.path.join(u'/site', self.mirrorpath, jbase)
                     ofile = os.path.join(self.exportpath, jbase)
                     with open(ofile, 'wb') as f:
-                        orep.output_json(f)	# with links -> not yet
+                        orep.output_json(f)
                     dirty[evno] = jfile
                 r.destroy()
+                time.sleep(0.001)	# force coop sched
             glib.idle_add(self.mirror_start, dirty)
             self.log.info(u'Race info export.')
         except Exception as e:
@@ -1113,8 +1309,11 @@ class trackmeet:
         """Select timer scoreboard overlay."""
         self.gemini.clear()
         self.scbwin = None
-        self.scb.setoverlay(unt4.OVERLAY_CLOCK)
-        self.log.debug(u'Selected scoreboard timer overlay.')
+        self.announce.gfx_overlay(2, self.graphicscb)
+        self.scb.clrall()
+        ## hack for DAK
+        #for i in range(0, self.scb.pagelen+1):
+            #self.scb.clrline(i)
 
     def menu_scb_logo_activate_cb(self, menuitem, data=None):
         """Select logo and display overlay."""
@@ -1127,17 +1326,23 @@ class trackmeet:
         """Select blank scoreboard overlay."""
         self.gemini.clear()
         self.scbwin = None
-        self.scb.setoverlay(unt4.OVERLAY_BLANK)
+        #self.scb.setoverlay(unt4.OVERLAY_BLANK)
+        self.announce.gfx_overlay(0, self.graphicscb)
+        self.scb.clrall()
+        ## hack send out the temp, and press
+        self.scb.sendmsg(unt4.unt4(header='DC',text='17.345'))
+        self.scb.sendmsg(unt4.unt4(header='BP',text='1002.3'))
         ## hack for DAK
-        for i in range(0, self.scb.pagelen+1):
-            self.scb.clrline(i)
+        #for i in range(0, self.scb.pagelen+1):
+            #self.scb.clrline(i)
         self.log.debug(u'Selected scoreboard blank overlay.')
 
     def menu_scb_result_activate_cb(self, menuitem, data=None):
         """Select result scoreboard overlay."""
         self.gemini.clear()
         self.scbwin = None
-        self.scb.setoverlay(unt4.OVERLAY_MATRIX)
+        self.announce.gfx_overlay(1, self.graphicscb)
+        #self.scb.setoverlay(unt4.OVERLAY_MATRIX)
         self.log.debug(u'Selected scoreboard result overlay.')
 
     def menu_scb_connect_activate_cb(self, menuitem, data=None):
@@ -1206,13 +1411,18 @@ class trackmeet:
     def menu_timing_reconnect_activate_cb(self, menuitem, data=None):
         """Reconnect timers and initialise."""
         self.main_timer.setport(self.main_port)
-        self.main_timer.sane()
+        if self.main_port:
+            self.main_timer.sane()
         self.backup_timer.setport(self.backup_port)
-        self.backup_timer.sane()
+        if self.backup_port:
+            self.backup_timer.sane()
         self.rftimer.setport(self.rftimer_port)
-        self.rftimer.setlvl(box=u'50', sta='//')
-        self.rftimer.sync()
-        self.rftimer.status()
+        if self.rftimer_port:
+            self.rftimer.stop_session()
+            self.rftimer.sane()
+            self.rftimer.start_session()
+            self.rftimer.sync()
+            self.rftimer.status()
         self.log.info(u'Re-connect and initialise attached timers.')
 
     ## Help menu callbacks
@@ -1245,19 +1455,35 @@ class trackmeet:
     ## Timer callbacks
     def menu_clock_timeout(self):
         """Update time of day on clock button."""
+    
         if not self.running:
             return False
         else:
             tt = tod.tod(u'now')
             self.clock_label.set_text(tt.meridian())
 
-            # check for completion in the rsync module
+            # check for completion in the export workers
             if self.mirror is not None:
                 if not self.mirror.is_alive():	# replaces join() non-blocking
                     self.mirror = None
+            if self.exporter is not None:
+                if not self.exporter.is_alive():# replaces join() non-blocking
+                    self.log.info(u'Deleting complete exporter: ' + repr(self.exporter))
+                    self.exporter = None
+                else:
+                    self.log.info(u'Incomplete exporter: ' + repr(self.exporter))
             #if type(self.scbwin) is scbwin.scbclock:
                 #self.gemini.ctick(tt)	# dubious
             #self.announce.postxt(0,72,tt)
+
+            if self.dbport != u'':
+                self.dbwatch += 1
+                if self.dbwatch % 10 == 0:
+                    self.dbwatch = 0
+                    if not self.db.connected():
+                        self.log.warn(u'DB Watchdog - Reconnect.')
+                        self.db.setport(self.dbport)
+
         return True
 
     def timeout(self):
@@ -1275,11 +1501,33 @@ class trackmeet:
 
     ## Timy utility methods.
     def printimp(self, printimps=True):
-        """Enable or disable printing of timing impulses on Timy."""
-        if self.timerprint:
-             self.main_timer.printer(printimps)
-        self.main_timer.printimp(printimps)
-        self.backup_timer.printimp(printimps)
+        """Deprecated impulse print option - removed for re-print."""
+        pass
+        #"""Enable or disable printing of timing impulses on Timy."""
+        #if self.timerprint:
+             #self.main_timer.printer(printimps)
+        #self.main_timer.printimp(printimps)
+        #self.backup_timer.printimp(printimps)
+
+    def timer_reprint(self, event=u'', trace=[]):
+        self.main_timer.printer(True)	# turn on printer
+        self.main_timer.printimp(False)	# suppress intermeds
+        self.timer.printline(u'')
+        self.timer.printline(u'')
+        self.timer.printline(u'')
+        self.timer.printline(self.titlestr)
+        self.timer.printline(self.subtitlestr)
+        self.timer.printline(u'')
+        if event:
+            self.timer.printline(event)
+            self.timer.printline(u'')
+        for l in trace:
+            self.timer.printline(l)
+        self.timer.printline(u'')
+        self.timer.printline(u'')
+        self.timer.printline(u'')
+        self.timer.printline(u'')
+        self.main_timer.printer(False)
 
     def delayimp(self, dtime):
         """Set the impulse delay time."""
@@ -1288,12 +1536,16 @@ class trackmeet:
 
     def timer_log_straight(self, bib, msg, tod, prec=4):
         """Print a tod log entry on the Timy receipt."""
-        self.timer.printline(u'{0:3} {1: >4}: '.format(bib[0:3],
-                              unicode(msg)[0:4]) + tod.timestr(prec))
+        ret = u'{0:3} {1: >4}: '.format(bib[0:3],
+                              unicode(msg)[0:4]) + tod.timestr(prec)
+        self.timer.printline(ret)
+        return ret
 
     def timer_log_msg(self, bib, msg):
         """Print the given msg entry on the Timy receipt."""
-        self.timer.printline(u'{0:3} '.format(bib[0:3]) + unicode(msg)[0:20])
+        ret = u'{0:3} '.format(bib[0:3]) + unicode(msg)[0:20]
+        self.timer.printline(ret)
+        return ret
 
     def event_string(self, evno):
         """Switch to suppress event no in delayed announce screens."""
@@ -1389,6 +1641,10 @@ class trackmeet:
         self.backup_timer.exit(msg)
         self.rftimer.exit(msg)
         print (u'Waiting for workers to exit...')
+        if self.exporter is not None:
+            print(u'\tresult exporter.')
+            self.exporter.join()
+            self.exporter = None
         if self.mirror is not None:
             print(u'\tlive result mirror.')
             self.mirror.join()
@@ -1466,6 +1722,7 @@ class trackmeet:
             cw.set(u'trackmeet', u'curevent', lastevent)
         cw.set(u'trackmeet', u'timerprint', self.timerprint)
         cw.set(u'trackmeet', u'maintimer', self.main_port)
+        cw.set(u'trackmeet', u'udptimer', self.udpaddr)
         cw.set(u'trackmeet', u'backuptimer', self.backup_port)
         cw.set(u'trackmeet', u'rftimer', self.rftimer_port)
         cw.set(u'trackmeet', u'gemini', self.gemport)
@@ -1485,6 +1742,7 @@ class trackmeet:
         cw.set(u'trackmeet', u'logos', self.logos)
         cw.set(u'trackmeet', u'mirrorpath', self.mirrorpath)
         cw.set(u'trackmeet', u'mirrorcmd', self.mirrorcmd)
+        cw.set(u'trackmeet', u'linkbase', self.linkbase)
         cw.set(u'trackmeet', u'http_notify_host', self.http_notify_host)
         cw.set(u'trackmeet', u'http_notify_user', self.http_notify_user)
         cw.set(u'trackmeet', u'http_notify_pass', self.http_notify_pass)
@@ -1493,6 +1751,7 @@ class trackmeet:
         cw.set(u'trackmeet', u'showevno', self.showevno)
         cw.set(u'trackmeet', u'provisional', self.provisional)
         cw.set(u'trackmeet', u'communiques', self.communiques)
+        cw.set(u'trackmeet', u'commalloc', self.commalloc)	# map
         cw.set(u'trackmeet', u'tracklayout', self.tracklayout)
         cw.set(u'trackmeet', u'tracklen_n', unicode(self.tracklen_n))  # poss val?
         cw.set(u'trackmeet', u'tracklen_d', unicode(self.tracklen_d))
@@ -1507,6 +1766,12 @@ class trackmeet:
         # save out print settings
         self.printprefs.to_file(os.path.join(self.configpath, u'print.prf'))
 
+    def reload_riders(self):
+        # make a prelim mapped rider struct
+        self.ridermap = {}
+        for s in self.rdb.listseries():
+            self.ridermap[s] = self.rdb.mkridermap(s)
+
     def loadconfig(self):
         """Load meet config from disk."""
         cr = jsonconfig.config({u'trackmeet':{u'maintimer':u'',
@@ -1532,8 +1797,10 @@ class trackmeet:
                                         u'commissaire':u'',
                                         u'logos':u'',
                                         u'curevent':u'',
+                                        u'udptimer':None,
                                         u'mirrorcmd':u'',
                                         u'mirrorpath':u'',
+                                        u'linkbase':u'.',
                                         u'http_notify_host':None,
                                         u'http_notify_user':None,
                                         u'http_notify_pass':None,
@@ -1542,6 +1809,7 @@ class trackmeet:
                                         u'tracklayout':u'',
                                         u'provisional':False,
                                         u'communiques':False,
+                                        u'commalloc':{},
                                         u'id':u''}})
         cr.add_section(u'trackmeet')
         cr.merge(metarace.sysconf, u'trackmeet')
@@ -1573,11 +1841,15 @@ class trackmeet:
             self.main_timer.sane()
 
         ## UDP Timer
-        #self.udptimer = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        ##self.udptimer.settimeout(0.2)
-        #self.udptimer.setsockopt(socket.SOL_SOCKET,
-                                 #12, 6)
-        #self.udpaddr = '192.168.0.20'
+        self.udpaddr = cr.get(u'trackmeet', u'udptimer')
+        if not self.udpaddr:
+            self.udpaddr = u'127.0.0.1'
+        self.udptimer = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        try:
+            # attempt to set SO_PRIORITY(12) (fails on MacOSX)
+            self.udptimer.setsockopt(socket.SOL_SOCKET, 12, 6)
+        except:
+            pass
 
         # set backup timer port
         nport = cr.get(u'trackmeet', u'backuptimer')
@@ -1642,6 +1914,7 @@ class trackmeet:
         self.logos = cr.get(u'trackmeet', u'logos')
         self.mirrorpath = cr.get(u'trackmeet', u'mirrorpath')
         self.mirrorcmd = cr.get(u'trackmeet', u'mirrorcmd')
+        self.linkbase = cr.get(u'trackmeet', u'linkbase')
         self.http_notify_host = cr.get(u'trackmeet', u'http_notify_host')
         self.http_notify_user = cr.get(u'trackmeet', u'http_notify_user')
         self.http_notify_pass = cr.get(u'trackmeet', u'http_notify_pass')
@@ -1653,6 +1926,8 @@ class trackmeet:
         self.showevno = strops.confopt_bool(cr.get(u'trackmeet', u'showevno'))
         self.provisional = strops.confopt_bool(cr.get(u'trackmeet', u'provisional'))
         self.communiques = strops.confopt_bool(cr.get(u'trackmeet', u'communiques'))
+        # communique allocations -> fixed once only
+        self.commalloc = cr.get(u'trackmeet', u'commalloc')
 
         # track length
         n = strops.confopt_posint(cr.get(u'trackmeet', u'tracklen_n'),0)
@@ -1677,10 +1952,7 @@ class trackmeet:
         self.edb.clear()
         self.edb.load(os.path.join(self.configpath, u'events.csv'))
         self.rdb.load(os.path.join(self.configpath, u'riders.csv'))
-        # make a prelim mapped rider struct
-        self.ridermap = {}
-        for s in self.rdb.listseries():
-            self.ridermap[s] = self.rdb.mkridermap(s)
+        self.reload_riders()
 
         cureventno = cr.get(u'trackmeet', u'curevent')
         if cureventno and cureventno in self.edb:
@@ -1703,7 +1975,7 @@ class trackmeet:
         # add grpahic scb if required...
         self.graphicscb = cr.get(u'trackmeet', u'graphicscb')
         if self.graphicscb:
-            glib.timeout_add_seconds(30, self.gfxscb_connect)
+            glib.timeout_add_seconds(10, self.gfxscb_connect)
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
@@ -1737,11 +2009,8 @@ class trackmeet:
             self.rdb.editrider(ref=dbr, club=value)
         else:
             self.log.debug(u'Attempt to edit other rider column: ' + repr(col))
-
-        # lazy, full re-read. Should go the other way :/
-        self.ridermap = {}
-        for s in self.rdb.listseries():
-            self.ridermap[s] = self.rdb.mkridermap(s)
+        ## lazy, full re-read. Should go the other way :/
+        self.reload_riders()
 
     def get_clubmode(self):
         return self.clubmode
@@ -1784,6 +2053,9 @@ class trackmeet:
         self.showevno = True
         self.provisional = False
         self.communiques = False
+        self.nextlinks = {}
+        self.prevlinks = {}
+        self.commalloc = {}
         self.timerport = None
         self.tracklen_n = 250	# numerator
         self.tracklen_d = 1	# d3nominator
@@ -1795,11 +2067,12 @@ class trackmeet:
         self.exportpending = False
         self.mirrorpath = u''	# default rsync mirror path
         self.mirrorcmd = u''	# default rsync mirror command
+        self.linkbase = u'.'
         self.http_notify_host = None
         self.http_notify_user = None
         self.http_notify_pass = None
         self.http_notify_event = None
-        #self.udpaddr = '192.168.0.20'
+        self.udpaddr = '192.168.0.21'
 
         # printer preferences
         paper = gtk.paper_size_new_custom('metarace-full',
@@ -1815,7 +2088,8 @@ class trackmeet:
 
         # hardware connections
         self.scb = sender.mksender(u'NULL')
-        self.announce = telegraph.telegraph(self.scb.linelen)
+        self.announce = telegraph.telegraph(80)
+        #self.announce = telegraph.telegraph(self.scb.linelen)
         self.scbport = u'NULL'
         self.annport = u''
         self.timerprint = False	# enable timer printer?
@@ -1830,7 +2104,10 @@ class trackmeet:
         self.gemport = u''
         self.db = dbexport.dbexport()	# hack for CA Track Nats-> to be incl
         self.dbport = u''
-        self.mirror = None
+        self.dbwatch = 0
+        self.mirror = None	# file mirror thread
+        self.exporter = None	# export worker thread... for speedy cb
+        self.exportlock = threading.Lock()	# one only exporter
         self.graphicscb = u''	# no graphic scb to connect to
 
         b = gtk.Builder()
@@ -1873,7 +2150,10 @@ class trackmeet:
         self.lh = loghandler.textViewHandler(self.log_buffer,
                       self.log_view, self.log_scroll)
         ## HACK: debug on console for rftimer testing
-        self.lh.setLevel(thbc.RFID_LOG_LEVEL)	# show info up in log view
+        #self.lh.setLevel(thbc.RFID_LOG_LEVEL)	# show info up in log view
+        #self.lh.setLevel(timy.TIMER_LOG_LEVEL)	# show info up in log view
+        self.sh.setLevel(logging.INFO)	# show info upon status bar
+        #self.sh.setLevel(17)	# show info upon status bar
         self.lh.setFormatter(f)
         self.log.addHandler(self.lh)
 
@@ -1881,7 +2161,7 @@ class trackmeet:
         self.rdb = riderdb.riderdb()
         self.ridermap = {}
 
-        b.get_object('rider_box').add(self.rdb.mkview())
+        b.get_object('rider_box').add(self.rdb.mkview(ucicode=True,note=True))
 
         # get event db and pack into scrolled pane
         self.edb = eventdb.eventdb()
@@ -1889,8 +2169,7 @@ class trackmeet:
         self.edb.set_result_cb(self.eventdb_cb, 'result')
         self.edb.set_program_cb(self.eventdb_cb, 'program')
         b.get_object('event_box').add(self.edb.mkview())
-        ## TODO: event number change handler
-        #self.edb.set_evno_change_cb(self.race_evno_change)
+        self.edb.set_evno_change_cb(self.race_evno_change)
 
 	# now, connect each of the race menu types if present in builder
         for etype in self.edb.racetypes:
