@@ -90,7 +90,7 @@ class wtio(threading.Thread):
     thread object through the command queue.
 
     """
-    def __init__(self, addr=None, cqueue=None, log=None):
+    def __init__(self, addr=None, cqueue=None, log=None, srcid=None):
         """Construct wheeltime I/O thread.
 
         Named parameters:
@@ -107,6 +107,7 @@ class wtio(threading.Thread):
         self.addr = addr
         self.rdbuf = ''
         self.cb = None
+        self.srcid = srcid
         self.running = False
 
     def setcb(self, cb=None):
@@ -152,7 +153,8 @@ class wtio(threading.Thread):
                                    s[30:32], int(s[32:34], 16))
                     istr = '00:0'	# emulate THbC info, but zero for WT
                     chanstr = 'BOX'
-                    t = tod.tod(timestr, index=istr, chan=chanstr, refid=tagid)
+                    t = tod.tod(timestr, index=istr, chan=chanstr,
+                                         refid=tagid, source=self.srcid)
                     if self.cb is not None:
                         glib.idle_add(self.cb, t)
                     self.cqueue.put_nowait(('RFID', t))	# notify only now
@@ -172,7 +174,8 @@ class wtio(threading.Thread):
                 istr = '00:0'
                 chanstr = 'MAN'
                 tagid = ''	# emulate THbC manual trigger (lstripped)
-                t = tod.tod(timestr, index=istr, chan=chanstr, refid=tagid)
+                t = tod.tod(timestr, index=istr, chan=chanstr,
+                                     refid=tagid, source=self.srcid)
                 if self.cb is not None:
                     glib.idle_add(self.cb, t)
                 self.cqueue.put_nowait(('RFID', t))	# notify only now
@@ -284,7 +287,8 @@ class wheeltime(threading.Thread):
                      str(t[0])[2:], t[1], t[2], t[6], t[3], t[4], t[5])
         self.cqueue.put_nowait(('MSG', 'ab000701' + datestr + '\n'))
 
-    def trig(self, timeval='now', index='FAKE', chan='MAN', refid='0'):
+    def trig(self, timeval='now', index='FAKE', chan='MAN',
+                   refid='0', sourceid=None):
         """Create a fake timing event.
 
            Generate a new tod object to mimic a message as requested
@@ -292,7 +296,10 @@ class wheeltime(threading.Thread):
            'now' time in the calling thread.
 
         """
-        t = tod.tod(timeval, index, chan, refid.lstrip('0'))
+        src=self.name
+        if sourceid is not None:
+           src=sourceid
+        t = tod.tod(timeval, index, chan, refid.lstrip('0'), source=src)
         self.cqueue.put_nowait(('TRIG', t))
 
     def replay(self, filename=''):
@@ -365,7 +372,7 @@ class wheeltime(threading.Thread):
                         self.addr = m[1]
                         self.log.debug('Re-Connect wheeltime addr: ' + str(m[1]))
                         self.io = wtio(addr=self.addr, cqueue=self.cqueue,
-                                       log=self.log)
+                                       log=self.log, srcid=self.name)
                         self.io.setcb(self.__cb)
                         self.io.start()
                     else:
